@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: GPL-2.0
-/* Copyright(c) 2013 - 2020 Intel Corporation. */
+/* Copyright(c) 2013 - 2021 Intel Corporation. */
 
 /* this lets the macros that return timespec64 or structs compile cleanly with
  * W=2
@@ -17,13 +17,14 @@
  * operate with the nanosecond field directly without fear of overflow.
  *
  * Much like the 82599, the update period is dependent upon the link speed:
- * At 40Gb link or no link, the period is 1.6ns.
- * At 10Gb link, the period is multiplied by 2. (3.2ns)
+ * At 40Gb, 25Gb, or no link, the period is 1.6ns.
+ * At 10Gb or 5Gb link, the period is multiplied by 2. (3.2ns)
  * At 1Gb link, the period is multiplied by 20. (32ns)
  * 1588 functionality is not supported at 100Mbps.
  */
 #define I40E_PTP_40GB_INCVAL		0x0199999999ULL
 #define I40E_PTP_10GB_INCVAL_MULT	2
+#define I40E_PTP_5GB_INCVAL_MULT	2
 #define I40E_PTP_1GB_INCVAL_MULT	20
 #define I40E_ISGN			0x80000000
 
@@ -39,6 +40,12 @@ enum i40e_ptp_pin {
 	GPIO_4
 };
 
+enum i40e_can_set_pins_t {
+	CANT_DO_PINS = -1,
+	CAN_SET_PINS,
+	CAN_DO_PINS
+};
+
 static struct ptp_pin_desc sdp_desc[] = {
 	/* name     idx      func      chan */
 	{"SDP3_2", SDP3_2, PTP_PF_NONE, 0},
@@ -74,6 +81,7 @@ static const char * const i40e_ptp_gpio_pin_state2str[] = {
 };
 
 enum i40e_ptp_led_pin_state {
+	led_end = -2,
 	low = 0,
 	high,
 };
@@ -145,7 +153,7 @@ static const struct i40e_ptp_pins_settings
 	{in_B,	off,	out_B,		low,	high,	high,	low},
 	{in_B,	in_A,	out_B,		low,	high,	high,	low},
 	{in_B,	out_A,	out_B,		low,	low,	high,	high},
-	{end,	end,	end,		end,	end,	end,	end}
+	{end,	end,	end,	led_end, led_end, led_end, led_end}
 };
 
 static int i40e_ptp_set_pins(struct i40e_pf *pf,
@@ -176,7 +184,7 @@ static void i40e_ptp_extts0_work(struct work_struct *work)
 	event.timestamp = (((u64)hi) << 32) | lo;
 
 	event.type = PTP_CLOCK_EXTTS;
-	event.index = 0;
+	event.index = hw->pf_id;
 
 	/* fire event */
 	ptp_clock_event(pf->ptp_clock, &event);
@@ -195,27 +203,35 @@ static bool i40e_is_ptp_pin_dev(struct i40e_hw *hw)
 }
 
 /**
- * i40e_can_do_pins - check possibility of manipulating the pins
+ * i40e_can_set_pins - check possibility of manipulating the pins
  * @pf: board private structure
  *
  * Check if all conditions are satisfied to manipulate PTP pins.
- * Return true if pins can be manipulated or false otherwise.
+ * Return CAN_SET_PINS if pins can be set on a specific PF or
+ * return CAN_DO_PINS if pins can be manipulated within a NIC or
+ * return CANT_DO_PINS otherwise.
  **/
-static bool i40e_can_do_pins(struct i40e_pf *pf)
+static enum i40e_can_set_pins_t i40e_can_set_pins(struct i40e_pf *pf)
 {
 	if (!i40e_is_ptp_pin_dev(&pf->hw)) {
 		dev_warn(&pf->pdev->dev,
 			 "PTP external clock not supported.\n");
-		return false;
+		return CANT_DO_PINS;
 	}
 
-	if (!pf->ptp_pins || pf->hw.pf_id) {
+	if (!pf->ptp_pins) {
 		dev_warn(&pf->pdev->dev,
-			 "PTP PIN reading allowed for PF0 only.\n");
-		return false;
+			 "PTP PIN manipulation not allowed.\n");
+		return CANT_DO_PINS;
 	}
 
-	return true;
+	if (pf->hw.pf_id) {
+		dev_warn(&pf->pdev->dev,
+			 "PTP PINs should be accessed via PF0.\n");
+		return CAN_DO_PINS;
+	}
+
+	return CAN_SET_PINS;
 }
 
 /**
@@ -590,6 +606,10 @@ static int i40e_ptp_enable_pin(struct i40e_pf *pf, unsigned int chan,
 	struct i40e_ptp_pins_settings pins;
 	int pin_index;
 
+	/* Use PF0 to set pins. Return success for user space tools */
+	if (pf->hw.pf_id)
+		return 0;
+
 	/* Preserve previous state of pins that we don't touch */
 	pins.sdp3_2 = pf->ptp_pins->sdp3_2;
 	pins.sdp3_3 = pf->ptp_pins->sdp3_3;
@@ -622,12 +642,9 @@ static int i40e_ptp_enable_pin(struct i40e_pf *pf, unsigned int chan,
 
 		*pin = i40e_pin_state(chan, func);
 	} else {
-		if (pins.sdp3_2 == i40e_pin_state(chan, func))
-			pins.sdp3_2 = off;
-		if (pins.sdp3_3 == i40e_pin_state(chan, func))
-			pins.sdp3_3 = off;
-		if (pins.gpio_4 == i40e_pin_state(chan, func))
-			pins.gpio_4 = off;
+		pins.sdp3_2 = off;
+		pins.sdp3_3 = off;
+		pins.gpio_4 = off;
 	}
 
 	return i40e_ptp_set_pins(pf, &pins) ? -EINVAL : 0;
@@ -670,7 +687,7 @@ static int i40e_ptp_feature_enable(struct ptp_clock_info *ptp,
 }
 
 /**
- * i40e_ptp_update_latch_events - Read I40E_PRTTSYN_STAT_1 and latch events
+ * i40e_ptp_get_rx_events - Read I40E_PRTTSYN_STAT_1 and latch events
  * @pf: the PF data structure
  *
  * This function reads I40E_PRTTSYN_STAT_1 and updates the corresponding timers
@@ -896,6 +913,66 @@ void i40e_ptp_rx_hwtstamp(struct i40e_pf *pf, struct sk_buff *skb, u8 index)
 	i40e_ptp_convert_to_hwtstamp(skb_hwtstamps(skb), ns);
 }
 
+/**
+ * i40e_ptp_get_link_speed_hw - get the link speed
+ * @pf: Board private structure
+ *
+ * Calculate link speed depending on the link status.
+ * Return the link speed.
+ **/
+static enum i40e_aq_link_speed i40e_ptp_get_link_speed_hw(struct i40e_pf *pf)
+{
+	bool link_up = pf->hw.phy.link_info.link_info & I40E_AQ_LINK_UP;
+	enum i40e_aq_link_speed link_speed = I40E_LINK_SPEED_UNKNOWN;
+	struct i40e_hw *hw = &pf->hw;
+
+	if (link_up) {
+		struct i40e_link_status *hw_link_info = &hw->phy.link_info;
+
+		i40e_aq_get_link_info(hw, true, NULL, NULL);
+		link_speed = hw_link_info->link_speed;
+	} else {
+		enum i40e_prt_mac_link_speed prtmac_linksta;
+		u64 prtmac_pcs_linksta;
+
+		prtmac_linksta = (rd32(hw, I40E_PRTMAC_LINKSTA(0))
+			& I40E_PRTMAC_LINKSTA_MAC_LINK_SPEED_MASK)
+			>> I40E_PRTMAC_LINKSTA_MAC_LINK_SPEED_SHIFT;
+
+		if (prtmac_linksta == I40E_PRT_MAC_LINK_SPEED_40GB) {
+			link_speed = I40E_LINK_SPEED_40GB;
+		} else {
+			i40e_aq_debug_read_register(hw,
+						    I40E_PRTMAC_PCS_LINK_STATUS1(0),
+						    &prtmac_pcs_linksta,
+						    NULL);
+
+			prtmac_pcs_linksta = (prtmac_pcs_linksta
+			& I40E_PRTMAC_PCS_LINK_STATUS1_LINK_SPEED_MASK)
+			>> I40E_PRTMAC_PCS_LINK_STATUS1_LINK_SPEED_SHIFT;
+
+			switch (prtmac_pcs_linksta) {
+			case I40E_PRT_MAC_PCS_LINK_SPEED_100MB:
+				link_speed = I40E_LINK_SPEED_100MB;
+				break;
+			case I40E_PRT_MAC_PCS_LINK_SPEED_1GB:
+				link_speed = I40E_LINK_SPEED_1GB;
+				break;
+			case I40E_PRT_MAC_PCS_LINK_SPEED_10GB:
+				link_speed = I40E_LINK_SPEED_10GB;
+				break;
+			case I40E_PRT_MAC_PCS_LINK_SPEED_20GB:
+				link_speed = I40E_LINK_SPEED_20GB;
+				break;
+			default:
+				link_speed = I40E_LINK_SPEED_UNKNOWN;
+			}
+		}
+	}
+
+	return link_speed;
+}
+
 /**
  * i40e_ptp_set_increment - Utility function to update clock increment rate
  * @pf: Board private structure
@@ -906,19 +983,20 @@ void i40e_ptp_rx_hwtstamp(struct i40e_pf *pf, struct sk_buff *skb, u8 index)
  **/
 void i40e_ptp_set_increment(struct i40e_pf *pf)
 {
-	struct i40e_link_status *hw_link_info;
+	enum i40e_aq_link_speed link_speed;
 	struct i40e_hw *hw = &pf->hw;
 	u64 incval;
 	u32 mult;
 
-	hw_link_info = &hw->phy.link_info;
+	link_speed = i40e_ptp_get_link_speed_hw(pf);
 
-	i40e_aq_get_link_info(&pf->hw, true, NULL, NULL);
-
-	switch (hw_link_info->link_speed) {
+	switch (link_speed) {
 	case I40E_LINK_SPEED_10GB:
 		mult = I40E_PTP_10GB_INCVAL_MULT;
 		break;
+	case I40E_LINK_SPEED_5GB:
+		mult = I40E_PTP_5GB_INCVAL_MULT;
+		break;
 	case I40E_LINK_SPEED_1GB:
 		mult = I40E_PTP_1GB_INCVAL_MULT;
 		break;
@@ -955,6 +1033,7 @@ void i40e_ptp_set_increment(struct i40e_pf *pf)
 	/* Update the base adjustement value. */
 	WRITE_ONCE(pf->ptp_adj_mult, mult);
 	smp_mb(); /* Force the above update. */
+	i40e_ptp_set_1pps_signal_hw(pf);
 }
 
 /**
@@ -985,7 +1064,7 @@ int i40e_ptp_get_ts_config(struct i40e_pf *pf, struct ifreq *ifr)
  **/
 static void i40e_ptp_free_pins(struct i40e_pf *pf)
 {
-	if (pf->hw.pf_id == 0 && i40e_is_ptp_pin_dev(&pf->hw)) {
+	if (i40e_is_ptp_pin_dev(&pf->hw)) {
 		kfree(pf->ptp_pins);
 		kfree(pf->ptp_caps.pin_config);
 		pf->ptp_pins = NULL;
@@ -1057,6 +1136,24 @@ static void i40e_ptp_set_led_hw(struct i40e_hw *hw,
 	}
 }
 
+/**
+ * i40e_ptp_init_leds_hw - init LEDs
+ * @hw: pointer to a hardware structure
+ *
+ * Set initial state of LEDs
+ **/
+static void i40e_ptp_init_leds_hw(struct i40e_hw *hw)
+{
+	wr32(hw, I40E_GLGEN_GPIO_CTL(I40E_LED2_0),
+	     I40E_GLGEN_GPIO_CTL_LED_INIT);
+	wr32(hw, I40E_GLGEN_GPIO_CTL(I40E_LED2_1),
+	     I40E_GLGEN_GPIO_CTL_LED_INIT);
+	wr32(hw, I40E_GLGEN_GPIO_CTL(I40E_LED3_0),
+	     I40E_GLGEN_GPIO_CTL_LED_INIT);
+	wr32(hw, I40E_GLGEN_GPIO_CTL(I40E_LED3_1),
+	     I40E_GLGEN_GPIO_CTL_LED_INIT);
+}
+
 /**
  * i40e_ptp_set_pins_hw - Set HW GPIO pins
  * @pf: Board private structure
@@ -1100,10 +1197,13 @@ static void i40e_ptp_set_pins_hw(struct i40e_pf *pf)
 static int i40e_ptp_set_pins(struct i40e_pf *pf,
 			     struct i40e_ptp_pins_settings *pins)
 {
+	enum i40e_can_set_pins_t pin_caps = i40e_can_set_pins(pf);
 	int i = 0;
 
-	if (!i40e_can_do_pins(pf))
+	if (pin_caps == CANT_DO_PINS)
 		return -EOPNOTSUPP;
+	else if (pin_caps == CAN_DO_PINS)
+		return 0;
 
 	if (pins->sdp3_2 == invalid)
 		pins->sdp3_2 = pf->ptp_pins->sdp3_2;
@@ -1156,7 +1256,7 @@ int i40e_ptp_set_pins_ioctl(struct i40e_pf *pf, struct ifreq *ifr)
 	struct i40e_ptp_pins_settings pins;
 	int err;
 
-	if (!i40e_can_do_pins(pf))
+	if (i40e_can_set_pins(pf) != CAN_SET_PINS)
 		return -EOPNOTSUPP;
 
 	err = copy_from_user(&pins, ifr->ifr_data, sizeof(pins));
@@ -1176,7 +1276,7 @@ int i40e_ptp_set_pins_ioctl(struct i40e_pf *pf, struct ifreq *ifr)
  **/
 int i40e_ptp_alloc_pins(struct i40e_pf *pf)
 {
-	if (pf->hw.pf_id || !i40e_is_ptp_pin_dev(&pf->hw))
+	if (!i40e_is_ptp_pin_dev(&pf->hw))
 		return 0;
 
 	pf->ptp_pins =
@@ -1195,6 +1295,11 @@ int i40e_ptp_alloc_pins(struct i40e_pf *pf)
 	pf->ptp_pins->led3_0 = high;
 	pf->ptp_pins->led3_1 = high;
 
+	/* Use PF0 to set pins in HW. Return success for user space tools */
+	if (pf->hw.pf_id)
+		return 0;
+
+	i40e_ptp_init_leds_hw(&pf->hw);
 	i40e_ptp_set_pins_hw(pf);
 
 	return 0;
@@ -1209,7 +1314,7 @@ int i40e_ptp_alloc_pins(struct i40e_pf *pf)
  **/
 int i40e_ptp_get_pins(struct i40e_pf *pf, struct ifreq *ifr)
 {
-	if (!i40e_can_do_pins(pf))
+	if (i40e_can_set_pins(pf) != CAN_SET_PINS)
 		return -EOPNOTSUPP;
 
 	return copy_to_user(ifr->ifr_data, pf->ptp_pins,
@@ -1405,9 +1510,6 @@ static int i40e_init_pin_config(struct i40e_pf *pf)
 {
 	int i;
 
-	if (pf->hw.pf_id != 0)
-		return -EOPNOTSUPP;
-
 	pf->ptp_caps.n_pins = 3;
 	pf->ptp_caps.n_ext_ts = 2;
 	pf->ptp_caps.pps = 1;
@@ -1661,7 +1763,7 @@ static ssize_t i40e_sysfs_ptp_pins_write(struct kobject *kobj,
  **/
 static void i40e_ptp_pins_sysfs_init(struct i40e_pf *pf)
 {
-	if (pf->hw.pf_id != 0 || !i40e_is_ptp_pin_dev(&pf->hw))
+	if (!i40e_is_ptp_pin_dev(&pf->hw))
 		return;
 
 	pf->ptp_kobj = kobject_create_and_add("ptp_pins", &pf->pdev->dev.kobj);
@@ -1780,9 +1882,11 @@ void i40e_ptp_stop(struct i40e_pf *pf)
 			 pf->vsi[pf->lan_vsi]->netdev->name);
 	}
 
-	i40e_ptp_set_pin_hw(hw, I40E_SDP3_2, off);
-	i40e_ptp_set_pin_hw(hw, I40E_SDP3_3, off);
-	i40e_ptp_set_pin_hw(hw, I40E_GPIO_4, off);
+	if (i40e_is_ptp_pin_dev(&pf->hw)) {
+		i40e_ptp_set_pin_hw(hw, I40E_SDP3_2, off);
+		i40e_ptp_set_pin_hw(hw, I40E_SDP3_3, off);
+		i40e_ptp_set_pin_hw(hw, I40E_GPIO_4, off);
+	}
 
 	regval = rd32(hw, I40E_PRTTSYN_AUX_0(0));
 	regval &= ~I40E_PRTTSYN_AUX_0_PTPFLAG_MASK;
