@@ -1,5 +1,5 @@
 /* SPDX-License-Identifier: GPL-2.0 */
-/* Copyright (C) 2018-2019, Intel Corporation. */
+/* Copyright (C) 2018-2021, Intel Corporation. */
 
 #ifndef _ICE_H_
 #define _ICE_H_
@@ -17,9 +17,9 @@
 #include <linux/cpumask.h>
 #include <linux/rtnetlink.h>
 #include <linux/if_vlan.h>
-#ifdef HAVE_NETDEV_SB_DEV
+#ifdef HAVE_NDO_DFWD_OPS
 #include <linux/if_macvlan.h>
-#endif /* HAVE_NETDEV_SB_DEV */
+#endif /* HAVE_NDO_DFWD_OPS */
 #include <linux/dma-mapping.h>
 #include <linux/pci.h>
 #include <linux/workqueue.h>
@@ -30,6 +30,7 @@
 #include <linux/timer.h>
 #include <linux/delay.h>
 #include <linux/bitmap.h>
+#include <linux/hashtable.h>
 #include <linux/log2.h>
 #include <linux/ip.h>
 #include <linux/sctp.h>
@@ -38,6 +39,7 @@
 #include <linux/if_bridge.h>
 #include <linux/string.h>
 #include <linux/ctype.h>
+#include <linux/sizes.h>
 #ifdef HAVE_XDP_SUPPORT
 #include <linux/bpf.h>
 #include <linux/filter.h>
@@ -49,9 +51,12 @@
 #if IS_ENABLED(CONFIG_NET_DEVLINK)
 #include <net/devlink.h>
 #endif /* CONFIG_NET_DEVLINK */
-#if IS_ENABLED(CONFIG_DCB)
-#include <scsi/iscsi_proto.h>
-#endif /* CONFIG_DCB */
+#ifdef HAVE_CONFIG_DIMLIB
+#include <linux/dim.h>
+#else
+#include "kcompat_dim.h"
+#endif
+#include "ice_ddp.h"
 #include "ice_devids.h"
 #include "ice_type.h"
 #include "ice_txrx.h"
@@ -60,17 +65,16 @@
 #include "ice_common.h"
 #include "ice_flow.h"
 #include "ice_sched.h"
-#include <linux/mfd/core.h>
+#include "auxiliary_bus.h"
 #include <linux/idr.h>
 #include "ice_idc_int.h"
 #include "virtchnl.h"
-#include "ice_virtchnl_pf.h"
 #include "ice_sriov.h"
+#include "ice_vf_mbx.h"
 #include "ice_ptp.h"
-#include "ice_cgu.h"
-#include "ice_cgu_ops.h"
-#include "ice_cgu_util.h"
 #include "ice_fdir.h"
+#include "ice_vdcm.h"
+#include "ice_siov.h"
 #ifdef HAVE_AF_XDP_ZC_SUPPORT
 #include "ice_xsk.h"
 #endif /* HAVE_AF_XDP_ZC_SUPPORT */
@@ -105,7 +109,10 @@
 #include <linux/atomic.h>
 #include <linux/jiffies.h>
 #include "ice_arfs.h"
+#include "ice_repr.h"
+#include "ice_eswitch.h"
 #include "ice_vsi_vlan_ops.h"
+#include "ice_gnss.h"
 
 extern const char ice_drv_ver[];
 #define ICE_BAR0		0
@@ -114,13 +121,17 @@ extern const char ice_drv_ver[];
 #define ICE_MAX_CSR_SPACE	(8 * 1024 * 1024 - 64 * 1024)
 #endif /* CONFIG_DEBUG_FS */
 #define ICE_REQ_DESC_MULTIPLE	32
-#define ICE_MIN_NUM_DESC	64
-#define ICE_MAX_NUM_DESC	8160
-#define ICE_DFLT_MIN_RX_DESC	512
+
 #ifdef HAVE_PF_RING
+#define ICE_MIN_NUM_DESC	64
+#define ICE_MAX_NUM_DESC	4096
+#define ICE_DFLT_MIN_RX_DESC	512
 #define ICE_DFLT_NUM_RX_DESC	4096
 #define ICE_DFLT_NUM_TX_DESC	4096
 #else
+#define ICE_MIN_NUM_DESC	64
+#define ICE_MAX_NUM_DESC	8160
+#define ICE_DFLT_MIN_RX_DESC	512
 #define ICE_DFLT_NUM_RX_DESC	2048
 #define ICE_DFLT_NUM_TX_DESC	256
 #endif
@@ -131,20 +142,17 @@ extern const char ice_drv_ver[];
 #define ICE_MAX_NUM_VMDQ_VSI	16
 #define ICE_MAX_TXQ_VMDQ_VSI	4
 #define ICE_MAX_RXQ_VMDQ_VSI	4
-#ifdef HAVE_NETDEV_SB_DEV
+#ifdef HAVE_NDO_DFWD_OPS
 #define ICE_MAX_MACVLANS	64
 #endif
+#define ICE_MAX_SCALABLE	100
 #define ICE_DFLT_TRAFFIC_CLASS	BIT(0)
 #define ICE_INT_NAME_STR_LEN	(IFNAMSIZ + 16)
-#ifdef FWLOG_SUPPORT
-/* if FW logging is on, then bump the admin q size to avoid overflows */
-#define ICE_AQ_LEN		1023
-#else
 #define ICE_AQ_LEN		192
-#endif /* FWLOG_SUPPORT */
 #define ICE_MBXSQ_LEN		64
 #define ICE_SBQ_LEN		64
 #define ICE_FDIR_MSIX		2
+#define ICE_ESWITCH_MSIX	1
 #define ICE_MIN_LAN_MSIX	1
 #define ICE_OICR_MSIX		1
 #define ICE_RDMA_NUM_AEQ_MSIX	4
@@ -194,8 +202,9 @@ extern const char ice_drv_ver[];
 #define ICE_TX_CTX_DESC(R, i) (&(((struct ice_tx_ctx_desc *)((R)->desc))[i]))
 #define ICE_TX_FDIRDESC(R, i) (&(((struct ice_fltr_desc *)((R)->desc))[i]))
 
-#define ICE_ACL_ENTIRE_SLICE	1
-#define ICE_ACL_HALF_SLICE	2
+#define ICE_ACL_ENTIRE_SLICE         1
+#define ICE_ACL_HALF_SLICE           2
+#define ICE_TCAM_DIVIDER_THRESHOLD   6
 
 /* Minimum BW limit is 500 Kbps for any scheduler node */
 #define ICE_MIN_BW_LIMIT		500
@@ -204,6 +213,27 @@ extern const char ice_drv_ver[];
  */
 #define ICE_BW_KBPS_DIVISOR		125
 
+#if defined(HAVE_TC_FLOWER_ENC) && defined(HAVE_TC_INDIR_BLOCK)
+#define ICE_GTP_TNL_WELLKNOWN_PORT 2152
+#endif /* HAVE_TC_FLOWER_ENC && HAVE_TC_INDIR_BLOCK */
+
+#ifdef HAVE_TC_SETUP_CLSFLOWER
+/* prio 5..7 can be used as advanced switch filter priority. Default recipes
+ * have prio 4 and below, hence prio value between 5..7 can be used as filter
+ * prio for advanced switch filter (advanced switch filter means it needs
+ * new recipe to be created to represent specified extraction sequence because
+ * default recipe extraction sequence does not represent custom extraction)
+ */
+#define ICE_SWITCH_FLTR_PRIO_QUEUE     7
+/* prio 6 is reserved for future use (e.g. switch filter with L3 fields +
+ * (Optional: IP TOS/TTL) + L4 fields + (optionally: TCP fields such as
+ * SYN/FIN/RST))
+ */
+#define ICE_SWITCH_FLTR_PRIO_RSVD      6
+#define ICE_SWITCH_FLTR_PRIO_VSI       5
+#define ICE_SWITCH_FLTR_PRIO_QGRP      ICE_SWITCH_FLTR_PRIO_VSI
+#endif /* ifdef HAVE_TC_SETUP_CLSFLOWER*/
+
 /* Macro for each VSI in a PF */
 #define ice_for_each_vsi(pf, i) \
 	for ((i) = 0; (i) < (pf)->num_alloc_vsi; (i)++)
@@ -244,14 +274,19 @@ extern const char ice_drv_ver[];
 
 #define ice_pf_to_dev(pf) (&((pf)->pdev->dev))
 
+struct ice_fwlog_user_input {
+	unsigned long events;
+	u8 log_level;
+};
 
-enum ice_channel_fltr_type {
-	ICE_CHNL_FLTR_TYPE_INVALID,
-	ICE_CHNL_FLTR_TYPE_SRC_PORT,
-	ICE_CHNL_FLTR_TYPE_DEST_PORT,
-	ICE_CHNL_FLTR_TYPE_SRC_DEST_PORT, /* for future use cases */
-	ICE_CHNL_FLTR_TYPE_TENANT_ID,
-	ICE_CHNL_FLTR_TYPE_LAST /* must be last */
+enum ice_feature {
+	ICE_F_DSCP,
+	ICE_F_PTP_EXTTS,
+	ICE_F_CGU,
+	ICE_F_PHY_RCLK,
+	ICE_F_SMA_CTRL,
+	ICE_F_GNSS,
+	ICE_F_MAX
 };
 
 struct ice_channel {
@@ -274,28 +309,19 @@ struct ice_channel {
 	atomic_t fd_queue;
 	/* packets services thru' inline-FD filter */
 	u64 fd_pkt_cnt;
-	enum ice_channel_fltr_type fltr_type;
+	u8 inline_fd:1;
+	u8 qps_per_poller;
+	u32 poller_timeout;
 	struct ice_vsi *ch_vsi;
 };
 
 #ifdef HAVE_TC_CB_AND_SETUP_QDISC_MQPRIO
 /* To convert BPS BW parameter into Mbps*/
 #define ICE_BW_MBIT_PS_DIVISOR	125000 /* rate / (1000000 / 8) Mbps */
-#define ICE_MAX_MQPRIO_TCF		8 /*Max number of Traffic Classifiers*/
-
-struct ice_qreg_info {
-	u16 qoffset;
-	u16 qcount;
-	u8 netdev_tc;	/* Netdev TC index if netdev associated */
-};
-
-struct ice_tcf_qreg_cfg {
-	u8 num_qreg; /* number of Traffic classifier*/
-	u8 ena_tcf; /* Rx map */
-	struct ice_qreg_info qreg_info[ICE_MAX_MQPRIO_TCF];
-};
 #endif /* HAVE_TC_CB_AND_SETUP_QDISC_MQPRIO */
 
+#define ICE_ADQ_MAX_QPS	256
+
 struct ice_txq_meta {
 	u32 q_teid;	/* Tx-scheduler element identifier */
 	u16 q_id;	/* Entry in VSI's txq_map bitmap */
@@ -320,7 +346,7 @@ struct ice_tc_cfg {
 struct ice_res_tracker {
 	u16 num_entries;
 	u16 end;
-	u16 list[1];
+	u16 list[];
 };
 
 struct ice_qs_cfg {
@@ -338,15 +364,6 @@ struct ice_sw {
 	struct ice_pf *pf;
 	u16 sw_id;		/* switch ID for this switch */
 	u16 bridge_mode;	/* VEB/VEPA/Port Virtualizer */
-	struct ice_vsi *dflt_vsi;	/* default VSI for this switch */
-	u8 dflt_vsi_ena:1;	/* true if above dflt_vsi is enabled */
-	u16 stats_id;			/* stats counter for this switch */
-	u16 flags;
-	struct ice_eth_stats stats;
-	struct ice_eth_stats stats_prev;
-	struct ice_veb_up_stats veb_stats;
-	struct ice_veb_up_stats veb_stats_prev;
-	u8 stat_offsets_loaded:1;
 };
 
 enum ice_pf_state {
@@ -355,9 +372,9 @@ enum ice_pf_state {
 	ICE_NEEDS_RESTART,
 	ICE_PREPARED_FOR_RESET,	/* set by driver when prepared */
 	ICE_RESET_OICR_RECV,		/* set by driver after rcv reset OICR */
-	ICE_PFR_REQ,			/* set by driver and peers */
-	ICE_CORER_REQ,		/* set by driver and peers */
-	ICE_GLOBR_REQ,		/* set by driver and peers */
+	ICE_PFR_REQ,			/* set by driver and aux drivers */
+	ICE_CORER_REQ,		/* set by driver and aux drivers */
+	ICE_GLOBR_REQ,		/* set by driver and aux drivers */
 	ICE_CORER_RECV,		/* set by OICR handler */
 	ICE_GLOBR_RECV,		/* set by OICR handler */
 	ICE_EMPR_RECV,		/* set by OICR handler */
@@ -383,7 +400,6 @@ enum ice_pf_state {
 	ICE_VF_DIS,
 	ICE_CFG_BUSY,
 	ICE_SERVICE_SCHED,
-	ICE_PTP_TX_TS_READY,
 	ICE_SERVICE_DIS,
 	ICE_FD_FLUSH_REQ,
 	ICE_OICR_INTR_DIS,		/* Global OICR interrupt disabled */
@@ -399,7 +415,6 @@ enum ice_pf_state {
 enum ice_vsi_state {
 	ICE_VSI_DOWN,
 	ICE_VSI_NEEDS_RESTART,
-	ICE_VSI_BUSY,
 	ICE_VSI_NETDEV_ALLOCD,
 	ICE_VSI_NETDEV_REGISTERED,
 	ICE_VSI_UMAC_FLTR_CHANGED,
@@ -411,7 +426,14 @@ enum ice_vsi_state {
 
 enum ice_chnl_feature {
 	ICE_CHNL_FEATURE_FD_ENA, /* for side-band flow-director */
-	ICE_CHNL_FEATURE_INLINE_FD_ENA, /* for inline flow-director */
+	/* using the SO_MARK socket option will trigger skb->mark to be set.
+	 * Driver should act on skb->mark of not (to align flow to HW queue
+	 * binding) is additionally controlled via ethtool private flag and
+	 * when that private flag is tunred ON/OFF, this feature flags is
+	 * set/reset. This feature flag is used to determine if driver should
+	 * act or not when skb->mark is set.
+	 */
+	ICE_CHNL_FEATURE_INLINE_FD_MARK_ENA,
 	/* for pkt based inspection optimization - related to SW triggered
 	 * interrupt from napi_poll for channel enabled vector
 	 */
@@ -423,146 +445,6 @@ enum ice_chnl_feature {
 	ICE_CHNL_FEATURE_NBITS		/* must be last */
 };
 
-#ifdef HAVE_TC_SETUP_CLSFLOWER
-#define ICE_TC_FLWR_FIELD_DST_MAC		0x01
-#define ICE_TC_FLWR_FIELD_SRC_MAC		0x02
-#define ICE_TC_FLWR_FIELD_VLAN			0x04
-#define ICE_TC_FLWR_FIELD_DEST_IPV4		0x08
-#define ICE_TC_FLWR_FIELD_SRC_IPV4		0x10
-#define ICE_TC_FLWR_FIELD_DEST_IPV6		0x20
-#define ICE_TC_FLWR_FIELD_SRC_IPV6		0x40
-#define ICE_TC_FLWR_FIELD_DEST_L4_PORT		0x80
-#define ICE_TC_FLWR_FIELD_SRC_L4_PORT		0x100
-#define ICE_TC_FLWR_FIELD_TENANT_ID		0x200
-#define ICE_TC_FLWR_FIELD_ENC_DEST_IPV4		0x400
-#define ICE_TC_FLWR_FIELD_ENC_SRC_IPV4		0x800
-#define ICE_TC_FLWR_FIELD_ENC_DEST_IPV6		0x1000
-#define ICE_TC_FLWR_FIELD_ENC_SRC_IPV6		0x2000
-#define ICE_TC_FLWR_FIELD_ENC_DEST_L4_PORT	0x4000
-#define ICE_TC_FLWR_FIELD_ENC_SRC_L4_PORT	0x8000
-#define ICE_TC_FLWR_FIELD_ENC_DST_MAC		0x10000
-
-/* TC flower supported filter match */
-#define ICE_TC_FLWR_FLTR_FLAGS_DST_MAC		ICE_TC_FLWR_FIELD_DST_MAC
-#define ICE_TC_FLWR_FLTR_FLAGS_VLAN		ICE_TC_FLWR_FIELD_VLAN
-#define ICE_TC_FLWR_FLTR_FLAGS_DST_MAC_VLAN	(ICE_TC_FLWR_FIELD_DST_MAC | \
-						 ICE_TC_FLWR_FIELD_VLAN)
-#define ICE_TC_FLWR_FLTR_FLAGS_IPV4_DST_PORT	(ICE_TC_FLWR_FIELD_DEST_IPV4 | \
-						 ICE_TC_FLWR_FIELD_DEST_L4_PORT)
-#define ICE_TC_FLWR_FLTR_FLAGS_IPV4_SRC_PORT	(ICE_TC_FLWR_FIELD_DEST_IPV4 | \
-						 ICE_TC_FLWR_FIELD_SRC_L4_PORT)
-#define ICE_TC_FLWR_FLTR_FLAGS_IPV6_DST_PORT	(ICE_TC_FLWR_FIELD_DEST_IPV6 | \
-						 ICE_TC_FLWR_FIELD_DEST_L4_PORT)
-#define ICE_TC_FLWR_FLTR_FLAGS_IPV6_SRC_PORT	(ICE_TC_FLWR_FIELD_DEST_IPV6 | \
-						 ICE_TC_FLWR_FIELD_SRC_L4_PORT)
-
-#define ICE_TC_FLOWER_MASK_32	0xFFFFFFFF
-#define ICE_TC_FLOWER_MASK_16	0xFFFF
-#define ICE_TC_FLOWER_VNI_MAX	0xFFFFFFU
-
-#ifdef HAVE_TC_INDIR_BLOCK
-struct ice_indr_block_priv {
-	struct net_device *netdev;
-	struct ice_netdev_priv *np;
-	struct list_head list;
-};
-#endif /* HAVE_TC_INDIR_BLOCK */
-
-struct ice_tc_flower_action {
-	u32 tc_class;
-	enum ice_sw_fwd_act_type fltr_act;
-};
-
-struct ice_tc_vlan_hdr {
-	__be16 vlan_id; /* Only last 12 bits valid */
-#ifdef HAVE_FLOW_DISSECTOR_VLAN_PRIO
-	u16 vlan_prio; /* Only last 3 bits valid (valid values: 0..7) */
-#endif
-};
-
-struct ice_tc_l2_hdr {
-	u8 dst_mac[ETH_ALEN];
-	u8 src_mac[ETH_ALEN];
-	u16 n_proto;    /* Ethernet Protocol */
-};
-
-struct ice_tc_l3_hdr {
-	u8 ip_proto;    /* IPPROTO value */
-	union {
-		struct {
-			struct in_addr dst_ip;
-			struct in_addr src_ip;
-		} v4;
-		struct {
-			struct in6_addr dst_ip6;
-			struct in6_addr src_ip6;
-		} v6;
-	} ip;
-#define dst_ipv6	ip.v6.dst_ip6.s6_addr32
-#define dst_ipv6_addr	ip.v6.dst_ip6.s6_addr
-#define src_ipv6	ip.v6.src_ip6.s6_addr32
-#define src_ipv6_addr	ip.v6.src_ip6.s6_addr
-#define dst_ipv4	ip.v4.dst_ip.s_addr
-#define src_ipv4	ip.v4.src_ip.s_addr
-
-	u8 tos;
-	u8 ttl;
-};
-
-struct ice_tc_l4_hdr {
-	__be16 dst_port;
-	__be16 src_port;
-};
-
-struct ice_tc_flower_lyr_2_4_hdrs {
-	/* L2 layer fields with their mask */
-	struct ice_tc_l2_hdr l2_key;
-	struct ice_tc_l2_hdr l2_mask;
-	struct ice_tc_vlan_hdr vlan_hdr;
-	/* L3 (IPv4[6]) layer fields with their mask */
-	struct ice_tc_l3_hdr l3_key;
-	struct ice_tc_l3_hdr l3_mask;
-
-	/* L4 layer fields with their mask */
-	struct ice_tc_l4_hdr l4_key;
-	struct ice_tc_l4_hdr l4_mask;
-};
-
-struct ice_tc_flower_fltr {
-	struct hlist_node tc_flower_node;
-
-	/* cookie becomes filter_rule_id if rule is added successfully */
-	unsigned long cookie;
-
-	/* add_adv_rule returns information like recipe ID, rule_id. Store
-	 * those values since they are needed to remove advanced rule
-	 */
-	u16 rid;
-	u16 rule_id;
-	/* this could be queue/vsi_idx (sw handle)/queue_group, depending upon
-	 * destination type
-	 */
-	u16 dest_id;
-	/* if dest_id is vsi_idx, then need to store destination VSI ptr */
-	struct ice_vsi *dest_vsi;
-
-	/* Parsed TC flower configuration params */
-	struct ice_tc_flower_lyr_2_4_hdrs outer_headers;
-	struct ice_tc_flower_lyr_2_4_hdrs inner_headers;
-	u16 vsi_num;
-	__be32 tenant_id;
-	u32 flags;
-#define ICE_TC_FLWR_TNL_TYPE_NONE        0xff
-	u8 tunnel_type;
-	struct ice_tc_flower_action	action;
-
-	/* cache ptr which is used wherever needed to communicate netlink
-	 * messages
-	 */
-	struct netlink_ext_ack *extack;
-};
-#endif /* HAVE_TC_SETUP_CLSFLOWER */
-
 /* This is to be used only when channels are configured, to track state
  * at PF level, whether it should use RSS or inline flow-director and this
  * state gets set/reset appropriately as the HW flow-director table becomes
@@ -581,10 +463,10 @@ struct ice_vsi {
 	struct ice_port_info *port_info; /* back pointer to port_info */
 	struct ice_ring **rx_rings;	 /* Rx ring array */
 	struct ice_ring **tx_rings;	 /* Tx ring array */
-#ifdef HAVE_NETDEV_SB_DEV
+#ifdef HAVE_NDO_DFWD_OPS
 	/* Initial VSI tx_rings array when L2 offload is off */
 	struct ice_ring **base_tx_rings;
-#endif /* HAVE_NETDEV_SB_DEV */
+#endif /* HAVE_NDO_DFWD_OPS */
 	struct ice_q_vector **q_vectors; /* q_vector array */
 
 	irqreturn_t (*irq_handler)(int irq, void *data);
@@ -596,7 +478,6 @@ struct ice_vsi {
 	u32 tx_busy;
 	u32 rx_buf_failed;
 	u32 rx_page_failed;
-	u32 rx_gro_dropped;
 #ifdef ICE_ADD_PROBES
 	u32 rx_page_reuse;
 #endif /* ICE_ADD_PROBES */
@@ -606,7 +487,7 @@ struct ice_vsi {
 	u16 vsi_num;			/* HW (absolute) index of this VSI */
 	u16 idx;			/* software index in pf->vsi[] */
 
-	s16 vf_id;			/* VF ID for SR-IOV VSIs */
+	struct ice_vf *vf;		/* VF associated with this VSI */
 
 	u16 ethtype;			/* Ethernet protocol for pause frame */
 	u16 num_gfltr;
@@ -621,12 +502,6 @@ struct ice_vsi {
 	u8 *rss_lut_user;	/* User configured lookup table entries */
 	u8 rss_lut_type;	/* used to configure Get/Set RSS LUT AQ call */
 
-#if IS_ENABLED(CONFIG_NET_DEVLINK)
-	/* devlink port data */
-	struct devlink_port devlink_port;
-	bool devlink_port_registered;
-#endif /* CONFIG_NET_DEVLINK */
-
 	/* aRFS members only allocated for the PF VSI */
 #define ICE_MAX_RFS_FILTERS	0xFFFF
 #define ICE_MAX_ARFS_LIST	1024
@@ -652,10 +527,10 @@ struct ice_vsi {
 	u8 irqs_ready:1;
 	u8 current_isup:1;		 /* Sync 'link up' logging */
 	u8 stat_offsets_loaded:1;
-	struct ice_vsi_vlan_ops vlan_ops;
+	struct ice_vsi_vlan_ops inner_vlan_ops;
+	struct ice_vsi_vlan_ops outer_vlan_ops;
 	u16 num_vlan;
 
-
 	/* queue information */
 	u8 tx_mapping_mode;		 /* ICE_MAP_MODE_[CONTIG|SCATTER] */
 	u8 rx_mapping_mode;		 /* ICE_MAP_MODE_[CONTIG|SCATTER] */
@@ -678,6 +553,7 @@ struct ice_vsi {
 	u8 xdp_mapping_mode;		 /* ICE_MAP_MODE_[CONTIG|SCATTER] */
 #endif /* HAVE_XDP_SUPPORT */
 #ifdef HAVE_AF_XDP_ZC_SUPPORT
+	unsigned long *af_xdp_zc_qps;    /* tracks AF_XDP ZC enabled qps */
 #ifndef HAVE_AF_XDP_NETDEV_UMEM
 	struct xdp_umem **xsk_umems;
 	u16 num_xsk_umems_used;
@@ -687,17 +563,7 @@ struct ice_vsi {
 
 #ifdef HAVE_TC_CB_AND_SETUP_QDISC_MQPRIO
 	struct tc_mqprio_qopt_offload mqprio_qopt;/* queue parameters */
-	struct ice_tcf_qreg_cfg tcf_qreg_cfg;
 #endif /* HAVE_TC_CB_AND_SETUP_QDISC_MQPRIO */
-	DECLARE_BITMAP(ptp_tx_idx, INDEX_PER_QUAD);
-	struct sk_buff *ptp_tx_skb[INDEX_PER_QUAD];
-	struct hwtstamp_config tstamp_config;
-	struct ptp_clock_info ptp_caps;
-	struct ptp_clock *ptp_clock;
-	u32 tx_hwtstamp_skipped;
-	u8 ptp_tx:1;
-	enum ice_time_ref_freq time_ref_freq;
-	enum ice_mstr_tmr_mode mstr_tmr_mode;
 
 	/* Channel Specific Fields */
 	struct ice_vsi *tc_map_vsi[ICE_CHNL_MAX_TC];
@@ -744,6 +610,7 @@ struct ice_vsi {
 	u16 old_ena_tc;
 
 	struct ice_channel *ch;
+	u8 num_tc_devlink_params;
 
 	/* setup back reference, to which aggregator node this VSI
 	 * corresponds to
@@ -819,7 +686,7 @@ struct ice_q_vector {
 	u16 reg_idx;
 	u8 num_ring_rx;			/* total number of Rx rings in vector */
 	u8 num_ring_tx;			/* total number of Tx rings in vector */
-	u8 itr_countdown;		/* when 0 should adjust adaptive ITR */
+	u8 wb_on_itr:1;			/* if true, WB on ITR is enabled */
 	/* in usecs, need to use ice_intrl_to_usecs_reg() before writing this
 	 * value to the device
 	 */
@@ -836,6 +703,8 @@ struct ice_q_vector {
 	struct ice_channel *ch;
 
 	char name[ICE_INT_NAME_STR_LEN];
+
+	u16 total_events;	/* net_dim(): number of interrupts processed */
 	/* This tracks current state of vector, BUSY_POLL or INTR */
 #define ICE_CHNL_IN_BP			BIT(ICE_CHNL_VECTOR_IN_BP)
 	/* This tracks prev state of vector, BUSY_POLL or INTR */
@@ -898,24 +767,30 @@ struct ice_q_vector {
 #ifdef ADQ_PERF_COUNTERS
 	struct ice_q_vector_ch_stats ch_stats;
 #endif /* ADQ_PERF_COUNTERS */
+	u64 last_wd_jiffy;
 } ____cacheline_internodealigned_in_smp;
 
 enum ice_pf_flags {
 	ICE_FLAG_FLTR_SYNC,
 	ICE_FLAG_VMDQ_ENA,
-#ifdef HAVE_NETDEV_SB_DEV
+#ifdef HAVE_NDO_DFWD_OPS
 	ICE_FLAG_MACVLAN_ENA,
-#endif /* HAVE_NETDEV_SB_DEV */
+#endif /* HAVE_NDO_DFWD_OPS */
 	ICE_FLAG_IWARP_ENA,
 	ICE_FLAG_RSS_ENA,
 	ICE_FLAG_SRIOV_ENA,
 	ICE_FLAG_SRIOV_CAPABLE,
+	ICE_FLAG_SIOV_ENA,
+	ICE_FLAG_SIOV_CAPABLE,
 	ICE_FLAG_DCB_CAPABLE,
 	ICE_FLAG_DCB_ENA,
 	ICE_FLAG_FD_ENA,
-	ICE_FLAG_PTP_ENA,		/* NVM PTP support */
+	ICE_FLAG_PTP_SUPPORTED,		/* NVM PTP support */
 	ICE_FLAG_PTP,			/* PTP successfully initialized */
-	ICE_FLAG_PEER_ENA,
+	ICE_FLAG_AUX_ENA,
+	ICE_FLAG_PLUG_AUX_DEV,
+	ICE_FLAG_UNPLUG_AUX_DEV,
+	ICE_FLAG_MTU_CHANGED,
 	ICE_FLAG_ADV_FEATURES,
 #ifdef NETIF_F_HW_TC
 	ICE_FLAG_TC_MQPRIO,		/* support for Multi queue TC */
@@ -929,19 +804,28 @@ enum ice_pf_flags {
 	ICE_FLAG_BASE_R_FEC,
 #endif /* !ETHTOOL_GFECPARAM */
 	ICE_FLAG_FW_LLDP_AGENT,
-	ICE_FLAG_CHNL_INLINE_FD_ENA,
+	ICE_FLAG_CHNL_INLINE_FD_MARK_ENA,
 	ICE_FLAG_CHNL_PKT_INSPECT_OPT_ENA,
 	ICE_FLAG_CHNL_PKT_CLEAN_BP_STOP_ENA,
 	ICE_FLAG_CHNL_PKT_CLEAN_BP_STOP_CFG,
+	ICE_FLAG_MOD_POWER_UNSUPPORTED,
+	ICE_FLAG_PHY_FW_LOAD_FAILED,
 	ICE_FLAG_ETHTOOL_CTXT,		/* set when ethtool holds RTNL lock */
 	ICE_FLAG_LEGACY_RX,
 	ICE_FLAG_VF_TRUE_PROMISC_ENA,
 	ICE_FLAG_MDD_AUTO_RESET_VF,
+	ICE_FLAG_VF_VLAN_PRUNING,
 	ICE_FLAG_LINK_LENIENT_MODE_ENA,
+	ICE_FLAG_ESWITCH_CAPABLE,
+	ICE_FLAG_DPLL_FAST_LOCK,
+	ICE_FLAG_DPLL_MONITOR,
+	ICE_FLAG_EXTTS_FILTER,
+	ICE_FLAG_GNSS,			/* GNSS successfully initialized */
+	ICE_FLAG_ALLOW_FEC_DIS_AUTO,
 	ICE_PF_FLAGS_NBITS		/* must be last */
 };
 
-#ifdef HAVE_NETDEV_SB_DEV
+#ifdef HAVE_NDO_DFWD_OPS
 struct ice_macvlan {
 	struct list_head list;
 	int id;
@@ -950,8 +834,13 @@ struct ice_macvlan {
 	struct ice_vsi *vsi;
 	u8 mac[ETH_ALEN];
 };
-#endif /* HAVE_NETDEV_SB_DEV */
+#endif /* HAVE_NDO_DFWD_OPS */
 
+struct ice_switchdev_info {
+	struct ice_vsi *control_vsi;
+	struct ice_vsi *uplink_vsi;
+	bool is_running;
+};
 
 enum ice_tnl_state {
 	ICE_TNL_SET_TO_ADD,
@@ -971,7 +860,6 @@ struct ice_tnl_entry {
 	struct list_head node;
 };
 
-
 struct ice_agg_node {
 	u32 agg_id;
 #define ICE_MAX_VSIS_IN_AGG_NODE	64
@@ -979,25 +867,45 @@ struct ice_agg_node {
 	u8 valid;
 };
 
-enum ice_flash_update_preservation {
-	/* Preserve all settings and fields */
-	ICE_FLASH_UPDATE_PRESERVE_ALL = 0,
-	/* Preserve limited fields, such as VPD, PCI serial ID, MACs, etc */
-	ICE_FLASH_UPDATE_PRESERVE_LIMITED,
-	/* Return all fields to factory settings */
-	ICE_FLASH_UPDATE_PRESERVE_FACTORY_SETTINGS,
-	/* Do not perform any preservation */
-	ICE_FLASH_UPDATE_PRESERVE_NONE,
+#ifdef HAVE_DEVLINK_HEALTH
+enum ice_mdd_src {
+	ICE_MDD_SRC_NONE = 0,
+	ICE_MDD_SRC_TX_PQM,
+	ICE_MDD_SRC_TX_TCLAN,
+	ICE_MDD_SRC_TX_TDPU,
+	ICE_MDD_SRC_RX
 };
 
+struct ice_mdd_event {
+	struct list_head list;
+	enum ice_mdd_src src;
+	u8 pf_num;
+	u16 vf_num;
+	u8 event;
+	u16 queue;
+};
+
+struct ice_mdd_reporter {
+	struct devlink_health_reporter *reporter;
+	u16 count;
+	struct list_head event_list;
+};
+#endif /* HAVE_DEVLINK_HEALTH */
+
 struct ice_pf {
 	struct pci_dev *pdev;
 
 #if IS_ENABLED(CONFIG_NET_DEVLINK)
 #ifdef HAVE_DEVLINK_REGIONS
 	struct devlink_region *nvm_region;
+	struct devlink_region *sram_region;
 	struct devlink_region *devcaps_region;
 #endif /* HAVE_DEVLINK_REGIONS */
+	/* devlink port data */
+	struct devlink_port devlink_port;
+#ifdef HAVE_DEVLINK_HEALTH
+	struct ice_mdd_reporter mdd_reporter;
+#endif /* HAVE_DEVLINK_HEALTH */
 #endif /* CONFIG_NET_DEVLINK */
 
 	/* OS reserved IRQ details */
@@ -1013,18 +921,12 @@ struct ice_pf {
 
 	struct ice_vsi **vsi;		/* VSIs created by the driver */
 	struct ice_sw *first_sw;	/* first switch created by firmware */
+	u16 eswitch_mode;		/* current mode of eswitch */
 #ifdef CONFIG_DEBUG_FS
 	struct dentry *ice_debugfs_pf;
 #endif /* CONFIG_DEBUG_FS */
-	/* Virtchnl/SR-IOV config info */
-	struct ice_vf *vf;
-	u16 num_alloc_vfs;	/* actual number of VFs allocated */
-	u16 num_vfs_supported;		/* num VFs supported for this PF */
-	u16 num_qps_per_vf;
-	u16 num_msix_per_vf;
-	/* used to ratelimit the MDD event logging */
-	unsigned long last_printed_mdd_jiffies;
-	DECLARE_BITMAP(malvfs, ICE_MAX_VF_COUNT);
+	struct ice_vfs vfs;
+	DECLARE_BITMAP(features, ICE_F_MAX);
 	DECLARE_BITMAP(state, ICE_STATE_NBITS);
 	DECLARE_BITMAP(flags, ICE_PF_FLAGS_NBITS);
 	unsigned long *avail_txqs;	/* bitmap to track PF Tx queue usage */
@@ -1036,30 +938,29 @@ struct ice_pf {
 	struct mutex avail_q_mutex;	/* protects access to avail_[rx|tx]qs */
 	struct mutex sw_mutex;		/* lock for protecting VSI alloc flow */
 	struct mutex tc_mutex;		/* lock to protect TC changes */
+	struct mutex adev_mutex;	/* lock to protect aux device access */
 	u32 msg_enable;
-	u64 cached_systime;
-	u8 ptp_tx_ts_ena;
-	u8 ptp_one_pps_out_ena;
-	atomic_t ptp_phy_reset_lock;
-	struct workqueue_struct *ov_wq;
-	struct mutex ptp_ps_lock; /* protects access to PTP PHY start */
-	u8 ptp_link_up;
-	struct ice_cgu_info cgu_info;
+	struct ice_ptp ptp;
+	struct tty_driver *ice_gnss_tty_driver;
+	struct tty_port *gnss_tty_port[ICE_GNSS_TTY_MINOR_DEVICES];
+	struct gnss_serial *gnss_serial[ICE_GNSS_TTY_MINOR_DEVICES];
 	u16 num_rdma_msix;	/* Total MSIX vectors for RDMA driver */
 	u16 rdma_base_vector;
-	struct ice_peer_dev *rdma_peer;
-#ifdef HAVE_NETDEV_SB_DEV
+#ifdef HAVE_NDO_DFWD_OPS
 	/* MACVLAN specific variables */
 	DECLARE_BITMAP(avail_macvlan, ICE_MAX_MACVLANS);
 	struct list_head macvlan_list;
 	u16 num_macvlan;
 	u16 max_num_macvlan;
-#endif /* HAVE_NETDEV_SB_DEV */
+#endif /* HAVE_NDO_DFWD_OPS */
 
 	/* spinlock to protect the AdminQ wait list */
 	spinlock_t aq_wait_lock;
 	struct hlist_head aq_wait_list;
 	wait_queue_head_t aq_wait_queue;
+	bool fw_emp_reset_disabled;
+
+	wait_queue_head_t reset_wait_queue;
 
 	u32 hw_csum_rx_error;
 	u16 oicr_idx;		/* Other interrupt cause MSIX vector index */
@@ -1099,16 +1000,18 @@ struct ice_pf {
 	u64 rx_tcp_cso_err;
 	u64 rx_udp_cso_err;
 	u64 rx_sctp_cso_err;
-	u64 tx_vlano;
-	u64 rx_vlano;
+	u64 tx_q_vlano;
+	u64 rx_q_vlano;
+	u64 tx_ad_vlano;
+	u64 rx_ad_vlano;
 #endif
 	u16 dcbx_cap;
 	u32 tx_timeout_count;
 	unsigned long tx_timeout_last_recovery;
 	u32 tx_timeout_recovery_level;
 	char int_name[ICE_INT_NAME_STR_LEN];
-	struct ice_peer_dev_int **peers;
-	int peer_idx;
+	struct iidc_core_dev_info **cdev_infos;
+	int aux_idx;
 	u32 sw_int_count;
 #ifdef HAVE_TC_SETUP_CLSFLOWER
 	/* count of tc_flower filters specific to channel (aka where filter
@@ -1118,6 +1021,9 @@ struct ice_pf {
 	struct hlist_head tc_flower_fltr_list;
 #endif /* HAVE_TC_SETUP_CLSFLOWER */
 
+	u16 max_qps;
+	u16 max_adq_qps;
+
 	struct ice_dcf dcf;
 	__le64 nvm_phy_type_lo; /* NVM PHY type low */
 	__le64 nvm_phy_type_hi; /* NVM PHY type high */
@@ -1132,12 +1038,19 @@ struct ice_pf {
 	 */
 	spinlock_t tnl_lock;
 	struct list_head tnl_list;
+#ifdef HAVE_UDP_TUNNEL_NIC_INFO
+#ifdef HAVE_UDP_TUNNEL_NIC_SHARED
+	struct udp_tunnel_nic_shared udp_tunnel_shared;
+#endif /* HAVE_UDP_TUNNEL_NIC_SHARED */
+	struct udp_tunnel_nic_info udp_tunnel_nic;
+#endif /* HAVE_UDP_TUNNEL_NIC_INFO */
+	struct ice_switchdev_info switchdev;
 
 #define ICE_INVALID_AGG_NODE_ID		0
 #define ICE_PF_AGG_NODE_ID_START	1
 #define ICE_MAX_PF_AGG_NODES		32
 	struct ice_agg_node pf_agg_node[ICE_MAX_PF_AGG_NODES];
-#ifdef HAVE_NETDEV_SB_DEV
+#ifdef HAVE_NDO_DFWD_OPS
 #define ICE_MACVLAN_AGG_NODE_ID_START	(ICE_PF_AGG_NODE_ID_START + \
 					 ICE_MAX_PF_AGG_NODES)
 #define ICE_MAX_MACVLAN_AGG_NODES	32
@@ -1146,7 +1059,12 @@ struct ice_pf {
 #define ICE_VF_AGG_NODE_ID_START	65
 #define ICE_MAX_VF_AGG_NODES		32
 	struct ice_agg_node vf_agg_node[ICE_MAX_VF_AGG_NODES];
-
+	enum ice_cgu_state synce_dpll_state;
+	u8 synce_ref_pin;
+	enum ice_cgu_state ptp_dpll_state;
+	u8 ptp_ref_pin;
+	s64 ptp_dpll_phase_offset;
+	u8 n_quanta_prof_used;
 #ifdef HAVE_PF_RING
 	u16 instance; /* A unique number per ice_pf instance in the system */
 	struct {
@@ -1157,6 +1075,8 @@ struct ice_pf {
 #endif
 };
 
+extern struct workqueue_struct *ice_wq;
+
 struct ice_netdev_priv {
 	struct ice_vsi *vsi;
 #ifdef HAVE_TC_INDIR_BLOCK
@@ -1175,11 +1095,9 @@ struct ice_netdev_priv {
 	struct notifier_block netdevice_nb;
 #endif
 #endif /* HAVE_TC_INDIR_BLOCK */
+	struct ice_repr *repr;
 };
 
-extern struct ida ice_peer_index_ida;
-
-
 /**
  * ice_vector_ch_enabled
  * @qv: pointer to q_vector, can be NULL
@@ -1191,6 +1109,19 @@ static inline bool ice_vector_ch_enabled(struct ice_q_vector *qv)
 	return !!qv->ch; /* Enable it to run with TC */
 }
 
+/**
+ * ice_vector_ind_poller
+ * @qv: pointer to q_vector
+ *
+ * This function returns true if vector is channel enabled and
+ * independent pollers are enabled on the associated channel.
+ */
+static inline bool ice_vector_ind_poller(struct ice_q_vector *qv)
+{
+	return (ice_vector_ch_enabled(qv) && qv->ch->qps_per_poller &&
+		qv->ch->poller_timeout);
+}
+
 /**
  * ice_vector_busypoll_intr
  * @qv: pointer to q_vector
@@ -1204,45 +1135,6 @@ static inline bool ice_vector_busypoll_intr(struct ice_q_vector *qv)
 	      !(qv->state_flags & ICE_CHNL_IN_BP);
 }
 
-/**
- * ice_vector_ever_in_busypoll
- * @qv: pointer to q_vector
- *
- * This function returns true if vectors current OR previous state
- * is BUSY_POLL
- */
-static inline bool ice_vector_ever_in_busypoll(struct ice_q_vector *qv)
-{
-	return (qv->state_flags & ICE_CHNL_PREV_IN_BP) ||
-	       (qv->state_flags & ICE_CHNL_IN_BP);
-}
-
-/**
- * ice_vector_state_curr_prev_intr
- * @qv: pointer to q_vector
- *
- * This function returns true if vectors current AND previous state
- * is INTERRUPT
- */
-static inline bool ice_vector_state_curr_prev_intr(struct ice_q_vector *qv)
-{
-	return !(qv->state_flags & ICE_CHNL_PREV_IN_BP) &&
-	       !(qv->state_flags & ICE_CHNL_IN_BP);
-}
-
-/**
- * ice_vector_intr_busypoll
- * @qv: pointer to q_vector
- *
- * This function returns true if vector is transitioning from INTERRUPT
- * to BUSY_POLL based on current and previous state of vector
- */
-static inline bool ice_vector_intr_busypoll(struct ice_q_vector *qv)
-{
-	return !(qv->state_flags & ICE_CHNL_PREV_IN_BP) &&
-		(qv->state_flags & ICE_CHNL_IN_BP);
-}
-
 /**
  * ice_adq_trigger_sw_intr
  * @hw: ptr to HW
@@ -1290,57 +1182,6 @@ ice_sw_intr_cntr(struct ice_q_vector *q_vector, bool napi_codepath)
 }
 #endif /* ADQ_PERF_COUNTERS */
 
-/**
- * ice_force_wb - trigger force write-back by setting WB_ON_ITR bit
- * @hw: ptr to HW
- * @q_vector: pointer to q_vector
- *
- * This function is used to force write-backs by setting WB_ON_ITR bit
- * in DYN_CTLN register. WB_ON_ITR and INTENA are mutually exclusive bits.
- * Setting WB_ON_ITR bits means Tx and Rx descriptors are written back based
- * on ITR expiration irrespective of INTENA setting
- */
-static inline void
-ice_force_wb(struct ice_hw *hw, struct ice_q_vector *q_vector)
-{
-	if (q_vector->num_ring_rx || q_vector->num_ring_tx) {
-#ifdef ADQ_PERF_COUNTERS
-		q_vector->ch_stats.num_wb_on_itr_set++;
-#endif /* ADQ_PERF_COUNTERS */
-		wr32(hw, GLINT_DYN_CTL(q_vector->reg_idx),
-		     ICE_GLINT_DYN_CTL_WB_ON_ITR(0, ICE_RX_ITR));
-	}
-
-	/* needed to avoid triggering WB_ON_ITR again which typically
-	 * happens from ice_set_wb_on_itr function
-	 */
-	q_vector->itr_countdown = ICE_IN_WB_ON_ITR_MODE;
-}
-
-#ifdef HAVE_TC_SETUP_CLSFLOWER
-/**
- * ice_is_chnl_fltr - is this a valid channel filter
- * @f: Pointer to tc-flower filter
- *
- * Criteria to determine of given filter is valid channel filter
- * or not is based on its "destination". If destination is hw_tc (aka tc_class)
- * and it is non-zero, then it is valid channel (aka ADQ) filter
- */
-static inline bool ice_is_chnl_fltr(struct ice_tc_flower_fltr *f)
-{
-	return !!f->action.tc_class;
-}
-
-/**
- * ice_chnl_dmac_fltr_cnt - DMAC based CHNL filter count
- * @pf: Pointer to PF
- */
-static inline int ice_chnl_dmac_fltr_cnt(struct ice_pf *pf)
-{
-	return pf->num_dmac_chnl_fltrs;
-}
-#endif /* HAVE_TC_SETUP_CLSFLOWER */
-
 /**
  * ice_irq_dynamic_ena - Enable default interrupt generation settings
  * @hw: pointer to HW struct
@@ -1398,27 +1239,32 @@ static inline void ice_set_ring_xdp(struct ice_ring *ring)
  * Returns a pointer to xdp_umem structure if there is an UMEM present,
  * NULL otherwise.
  */
+#ifdef HAVE_NETDEV_BPF_XSK_POOL
+static inline struct xsk_buff_pool *ice_xsk_umem(struct ice_ring *ring)
+#else
 static inline struct xdp_umem *ice_xsk_umem(struct ice_ring *ring)
+#endif
 {
+	struct ice_vsi *vsi = ring->vsi;
 #ifndef HAVE_AF_XDP_NETDEV_UMEM
-	struct xdp_umem **umems = ring->vsi->xsk_umems;
+	struct xdp_umem **umems = vsi->xsk_umems;
 #endif /* !HAVE_AF_XDP_NETDEV_UMEM */
 	u16 qid = ring->q_index;
 
 	if (ice_ring_is_xdp(ring))
-		qid -= ring->vsi->num_xdp_txq;
+		qid -= vsi->num_xdp_txq;
 
 #ifndef HAVE_AF_XDP_NETDEV_UMEM
-	if (qid >= ring->vsi->num_xsk_umems || !umems || !umems[qid] ||
-	    !ice_is_xdp_ena_vsi(ring->vsi))
+	if (qid >= vsi->num_xsk_umems || !umems || !umems[qid] ||
+	    !ice_is_xdp_ena_vsi(vsi) || !test_bit(qid, vsi->af_xdp_zc_qps))
 		return NULL;
 
 	return umems[qid];
 #else
-	if (!ice_is_xdp_ena_vsi(ring->vsi))
+	if (!ice_is_xdp_ena_vsi(vsi) || !test_bit(qid, vsi->af_xdp_zc_qps))
 		return NULL;
 
-	return xdp_get_umem_from_qid(ring->vsi->netdev, qid);
+	return xsk_get_pool_from_qid(vsi->netdev, qid);
 #endif /* !HAVE_AF_XDP_NETDEV_UMEM */
 }
 #endif /* HAVE_AF_XDP_ZC_SUPPORT */
@@ -1431,10 +1277,7 @@ static inline struct xdp_umem *ice_xsk_umem(struct ice_ring *ring)
  */
 static inline struct ice_vsi *ice_get_main_vsi(struct ice_pf *pf)
 {
-	if (pf->vsi)
-		return pf->vsi[0];
-
-	return NULL;
+	return pf->vsi[0];
 }
 
 /**
@@ -1443,9 +1286,34 @@ static inline struct ice_vsi *ice_get_main_vsi(struct ice_pf *pf)
  */
 static inline struct ice_vsi *ice_get_netdev_priv_vsi(struct ice_netdev_priv *np)
 {
-	return np->vsi;
+	/* In case of port representor return source port VSI. */
+	if (np->repr)
+		return np->repr->src_vsi;
+	else
+		return np->vsi;
 }
 
+#if IS_ENABLED(CONFIG_NET_DEVLINK)
+/**
+ * ice_is_switchdev_running - check if switchdev is configured
+ * @pf: pointer to PF structure
+ *
+ * Returns true if eswitch mode is set to DEVLINK_ESWITCH_MODE_SWITCHDEV
+ * and switchdev is configured, false otherwise.
+ */
+static inline bool ice_is_switchdev_running(struct ice_pf *pf)
+{
+	return pf->switchdev.is_running;
+}
+
+#else
+static inline bool
+ice_is_switchdev_running(struct ice_pf __always_unused *pf)
+{
+	return false;
+}
+#endif /* IS_ENABLED(CONFIG_NET_DEVLINK) */
+
 /**
  * ice_get_ctrl_vsi - Get the control VSI
  * @pf: PF instance
@@ -1459,6 +1327,21 @@ static inline struct ice_vsi *ice_get_ctrl_vsi(struct ice_pf *pf)
 	return pf->vsi[pf->ctrl_vsi_idx];
 }
 
+/**
+ * ice_find_vsi - Find the VSI from VSI ID
+ * @pf: The PF pointer to search in
+ * @vsi_num: The VSI ID to search for
+ */
+static inline struct ice_vsi *ice_find_vsi(struct ice_pf *pf, u16 vsi_num)
+{
+	int i;
+
+	ice_for_each_vsi(pf, i)
+		if (pf->vsi[i] && pf->vsi[i]->vsi_num == vsi_num)
+			return  pf->vsi[i];
+	return NULL;
+}
+
 /**
  * ice_find_first_vsi_by_type - Find and return first VSI of a given type
  * @pf: PF to search for VSI
@@ -1521,17 +1404,9 @@ static inline bool ice_vsi_fd_ena(struct ice_vsi *vsi)
 	return !!test_bit(ICE_CHNL_FEATURE_FD_ENA, vsi->features);
 }
 
-/**
- * ice_vsi_inline_fd_ena
- * @vsi: pointer to VSI
- *
- * This function returns true if VSI is enabled for usage of flow-director
- * otherwise returns false. This is controlled thru' ethtool priv-flag
- * 'channel-inline-flow-director'
- */
-static inline bool ice_vsi_inline_fd_ena(struct ice_vsi *vsi)
+static inline bool ice_vsi_inline_fd_mark_ena(struct ice_vsi *vsi)
 {
-	return !!test_bit(ICE_CHNL_FEATURE_INLINE_FD_ENA, vsi->features);
+	return !!test_bit(ICE_CHNL_FEATURE_INLINE_FD_MARK_ENA, vsi->features);
 }
 
 /**
@@ -1658,12 +1533,12 @@ static inline bool ice_active_vmdqs(struct ice_pf *pf)
 	return !!ice_find_first_vsi_by_type(pf, ICE_VSI_VMDQ2);
 }
 
-#ifdef HAVE_NETDEV_SB_DEV
+#ifdef HAVE_NDO_DFWD_OPS
 static inline bool ice_is_offloaded_macvlan_ena(struct ice_pf *pf)
 {
 	return test_bit(ICE_FLAG_MACVLAN_ENA, pf->flags);
 }
-#endif /* HAVE_NETDEV_SB_DEV */
+#endif /* HAVE_NDO_DFWD_OPS */
 
 #ifdef CONFIG_DEBUG_FS
 void ice_debugfs_pf_init(struct ice_pf *pf);
@@ -1671,10 +1546,10 @@ void ice_debugfs_pf_exit(struct ice_pf *pf);
 void ice_debugfs_init(void);
 void ice_debugfs_exit(void);
 #else
-#define ice_debugfs_pf_init(pf) do {} while (0)
-#define ice_debugfs_pf_exit(pf) do {} while (0)
-#define ice_debugfs_init() do {} while (0)
-#define ice_debugfs_exit() do {} while (0)
+static inline void ice_debugfs_pf_init(struct ice_pf *pf) { }
+static inline void ice_debugfs_pf_exit(struct ice_pf *pf) { }
+static inline void ice_debugfs_init(void) { }
+static inline void ice_debugfs_exit(void) { }
 #endif /* CONFIG_DEBUG_FS */
 
 bool netif_is_ice(struct net_device *dev);
@@ -1682,6 +1557,7 @@ int ice_vsi_setup_tx_rings(struct ice_vsi *vsi);
 int ice_vsi_setup_rx_rings(struct ice_vsi *vsi);
 int ice_vsi_open_ctrl(struct ice_vsi *vsi);
 int ice_vsi_open(struct ice_vsi *vsi);
+void ice_set_ethtool_repr_ops(struct net_device *netdev);
 void ice_set_ethtool_ops(struct net_device *netdev);
 void ice_set_ethtool_recovery_ops(struct net_device *netdev);
 void ice_set_ethtool_safe_mode_ops(struct net_device *netdev);
@@ -1691,12 +1567,13 @@ int ice_vsi_recfg_qs(struct ice_vsi *vsi, int new_rx, int new_tx);
 void ice_update_pf_stats(struct ice_pf *pf);
 void ice_update_vsi_stats(struct ice_vsi *vsi);
 int ice_up(struct ice_vsi *vsi);
+void ice_fetch_u64_stats_per_ring(struct ice_ring *ring, u64 *pkts, u64 *bytes);
 int ice_down(struct ice_vsi *vsi);
 int ice_vsi_cfg(struct ice_vsi *vsi);
 struct ice_vsi *ice_lb_vsi_setup(struct ice_pf *pf, struct ice_port_info *pi);
-#ifdef HAVE_NETDEV_SB_DEV
+#ifdef HAVE_NDO_DFWD_OPS
 int ice_vsi_cfg_netdev_tc0(struct ice_vsi *vsi);
-#endif /* HAVE_NETDEV_SB_DEV */
+#endif /* HAVE_NDO_DFWD_OPS */
 #ifdef HAVE_XDP_SUPPORT
 int ice_prepare_xdp_rings(struct ice_vsi *vsi, struct bpf_prog *prog);
 int ice_destroy_xdp_rings(struct ice_vsi *vsi);
@@ -1718,28 +1595,17 @@ int ice_get_rss_key(struct ice_vsi *vsi, u8 *seed);
 void ice_fill_rss_lut(u8 *lut, u16 rss_table_size, u16 rss_size);
 int ice_schedule_reset(struct ice_pf *pf, enum ice_reset_req reset);
 void ice_print_link_msg(struct ice_vsi *vsi, bool isup);
-#if IS_ENABLED(CONFIG_MFD_CORE)
-int ice_init_peer_devices(struct ice_pf *pf);
+int ice_plug_aux_dev(struct iidc_core_dev_info *cdev_info, const char *name);
+void ice_unplug_aux_dev(struct iidc_core_dev_info *cdev_info);
+int ice_plug_aux_devs(struct ice_pf *pf);
+void ice_unplug_aux_devs(struct ice_pf *pf);
+int ice_init_aux_devices(struct ice_pf *pf);
 int
-ice_for_each_peer(struct ice_pf *pf, void *data,
-		  int (*fn)(struct ice_peer_dev_int *, void *));
+ice_for_each_aux(struct ice_pf *pf, void *data,
+		 int (*fn)(struct iidc_core_dev_info *, void *));
 #ifdef CONFIG_PM
-void ice_peer_refresh_msix(struct ice_pf *pf);
+void ice_cdev_info_refresh_msix(struct ice_pf *pf);
 #endif /* CONFIG_PM */
-#else /* !CONFIG_MFD_CORE */
-static inline int ice_init_peer_devices(struct ice_pf *pf) { return 0; }
-
-static inline int
-ice_for_each_peer(struct ice_pf *pf, void *data,
-		  int (*fn)(struct ice_peer_dev_int *, void *))
-{
-	return 0;
-}
-
-#ifdef CONFIG_PM
-#define ice_peer_refresh_msix(pf) do { } while (0)
-#endif /* CONFIG_PM */
-#endif /* !CONFIG_MFD_CORE */
 #ifdef HAVE_NETDEV_UPPER_INFO
 /**
  * ice_set_sriov_cap - enable SRIOV in PF flags
@@ -1766,8 +1632,9 @@ static inline void ice_clear_sriov_cap(struct ice_pf *pf)
  */
 static inline void ice_set_rdma_cap(struct ice_pf *pf)
 {
-	if (pf->hw.func_caps.common_cap.iwarp)
+	if (pf->hw.func_caps.common_cap.iwarp && pf->num_rdma_msix) {
 		set_bit(ICE_FLAG_IWARP_ENA, pf->flags);
+	}
 }
 
 /**
@@ -1780,11 +1647,20 @@ static inline void ice_clear_rdma_cap(struct ice_pf *pf)
 }
 
 #endif /* HAVE_NETDEV_UPPER_INFO */
+/** ice_chk_rdma_cap - check the status of RDMA if PF flags
+ * @pf: PF struct
+ */
+static inline bool ice_chk_rdma_cap(struct ice_pf *pf)
+{
+	return test_bit(ICE_FLAG_IWARP_ENA, pf->flags);
+}
+
 const char *ice_stat_str(enum ice_status stat_err);
 const char *ice_aq_str(enum ice_aq_err aq_err);
-bool ice_is_wol_supported(struct ice_pf *pf);
+bool ice_is_wol_supported(struct ice_hw *hw);
 int ice_aq_wait_for_event(struct ice_pf *pf, u16 opcode, unsigned long timeout,
 			  struct ice_rq_event_info *event);
+void ice_fdir_del_all_fltrs(struct ice_vsi *vsi);
 int
 ice_fdir_write_fltr(struct ice_pf *pf, struct ice_fdir_fltr *input, bool add,
 		    bool is_tun);
@@ -1816,6 +1692,7 @@ int
 ice_ntuple_update_list_entry(struct ice_pf *pf, struct ice_fdir_fltr *input,
 			     int fltr_idx);
 void ice_update_ring_dest_vsi(struct ice_vsi *vsi, u16 *dest_vsi, u32 *ring);
+void ice_ch_vsi_update_ring_vecs(struct ice_vsi *vsi);
 int ice_open(struct net_device *netdev);
 int ice_open_internal(struct net_device *netdev);
 int ice_stop(struct net_device *netdev);
@@ -1823,9 +1700,4 @@ void ice_service_task_schedule(struct ice_pf *pf);
 int
 ice_acl_add_rule_ethtool(struct ice_vsi *vsi, struct ethtool_rxnfc *cmd);
 int ice_init_acl(struct ice_pf *pf);
-#ifdef HAVE_TC_SETUP_CLSFLOWER
-int
-ice_add_tc_flower_adv_fltr(struct ice_vsi *vsi,
-			   struct ice_tc_flower_fltr *tc_fltr);
-#endif /* HAVE_TC_SETUP_CLSFLOWER */
 #endif /* _ICE_H_ */
