@@ -1,9 +1,10 @@
 // SPDX-License-Identifier: GPL-2.0
-/* Copyright (C) 2018-2019, Intel Corporation. */
+/* Copyright (C) 2018-2021, Intel Corporation. */
 
 #include "ice_base.h"
 #include "ice_lib.h"
 #include "ice_dcb_lib.h"
+#include "ice_sriov.h"
 
 #ifdef HAVE_PF_RING
 extern int enable_debug;
@@ -116,6 +117,9 @@ static int ice_vsi_alloc_q_vector(struct ice_vsi *vsi, u16 v_idx)
 	q_vector->v_idx = v_idx;
 	q_vector->tx.itr_setting = ICE_DFLT_TX_ITR;
 	q_vector->rx.itr_setting = ICE_DFLT_RX_ITR;
+	q_vector->tx.itr_mode = ITR_DYNAMIC;
+	q_vector->rx.itr_mode = ITR_DYNAMIC;
+
 	if (vsi->type == ICE_VSI_VF)
 		goto out;
 	/* only set affinity_mask if the CPU is online */
@@ -222,6 +226,31 @@ static u16 ice_calc_q_handle(struct ice_vsi *vsi, struct ice_ring *ring, u8 tc)
 	return ring->q_index - vsi->tc_cfg.tc_info[tc].qoffset;
 }
 
+/**
+ * ice_eswitch_calc_q_handle
+ * @ring: pointer to ring which unique index is needed
+ *
+ * To correctly work with many netdevs
+ * ring->q_index of Tx rings on switchdev VSI can repeat. Hardware ring setup
+ * requires unique q_index. Calculate it here by finding index in vsi->tx_rings
+ * of this ring.
+ *
+ * Return -1 when index wasn't found. Should never happen, because vsi is get
+ * from ring->vsi, so it has to be present in this vsi.
+ */
+static u16 ice_eswitch_calc_q_handle(struct ice_ring *ring)
+{
+	struct ice_vsi *vsi = ring->vsi;
+	int i;
+
+	ice_for_each_txq(vsi, i) {
+		if (vsi->tx_rings[i] == ring)
+			return i;
+	}
+
+	return -1;
+}
+
 /**
  * ice_cfg_xps_tx_ring - Configure XPS for a Tx ring
  * @ring: The Tx ring to configure
@@ -231,22 +260,27 @@ static u16 ice_calc_q_handle(struct ice_vsi *vsi, struct ice_ring *ring, u8 tc)
  */
 static void ice_cfg_xps_tx_ring(struct ice_ring *ring)
 {
+#ifndef HAVE_XPS_QOS_SUPPORT
 	struct ice_vsi *vsi = ring->vsi;
 
+#endif /* !HAVE_XPS_QOS_SUPPORT */
 	if (!ring->q_vector || !ring->netdev)
 		return;
 
+#ifndef HAVE_XPS_QOS_SUPPORT
 	/* Single TC mode enable XPS
-	 * If there are more than 1 TC the netdev_set_num_tc() in newer kernels
-	 * resets XPS settings
+	 * If there is more than 1 TC, netdev_set_num_tc() resets XPS settings
 	 */
 	if (vsi->tc_cfg.numtc > 1)
 		return;
+#endif /* !HAVE_XPS_QOS_SUPPORT */
 
-	if (!test_and_set_bit(ICE_TX_XPS_INIT_DONE, ring->xps_state))
-		netif_set_xps_queue(ring->netdev,
-				    &ring->q_vector->affinity_mask,
-				    ring->q_index);
+	/* We only initialize XPS once, so as not to overwrite user settings */
+	if (test_and_set_bit(ICE_TX_XPS_INIT_DONE, ring->xps_state))
+		return;
+
+	netif_set_xps_queue(ring->netdev, &ring->q_vector->affinity_mask,
+			    ring->q_index);
 }
 
 /**
@@ -292,11 +326,13 @@ ice_setup_tx_ctx(struct ice_ring *ring, struct ice_tlan_ctx *tlan_ctx, u16 pf_q)
 		break;
 	case ICE_VSI_VF:
 		/* Firmware expects vmvf_num to be absolute VF ID */
-		tlan_ctx->vmvf_num = hw->func_caps.vf_base_id + vsi->vf_id;
+		tlan_ctx->vmvf_num = hw->func_caps.vf_base_id + vsi->vf->vf_id;
 		tlan_ctx->vmvf_type = ICE_TLAN_CTX_VMVF_TYPE_VF;
 		break;
+	case ICE_VSI_ADI:
 	case ICE_VSI_OFFLOAD_MACVLAN:
 	case ICE_VSI_VMDQ2:
+	case ICE_VSI_SWITCHDEV_CTRL:
 		tlan_ctx->vmvf_type = ICE_TLAN_CTX_VMVF_TYPE_VMQ;
 		break;
 	default:
@@ -309,6 +345,16 @@ ice_setup_tx_ctx(struct ice_ring *ring, struct ice_tlan_ctx *tlan_ctx, u16 pf_q)
 	else
 		tlan_ctx->src_vsi = ice_get_hw_vsi_num(hw, vsi->idx);
 
+	/* Restrict Tx timestamps to the PF VSI */
+	switch (vsi->type) {
+	case ICE_VSI_PF:
+		tlan_ctx->tsyn_ena = 1;
+		break;
+	default:
+		break;
+	}
+
+	tlan_ctx->quanta_prof_idx = ring->quanta_prof_id;
 	tlan_ctx->tso_ena = ICE_TX_LEGACY;
 	tlan_ctx->tso_qnum = pf_q;
 
@@ -317,7 +363,6 @@ ice_setup_tx_ctx(struct ice_ring *ring, struct ice_tlan_ctx *tlan_ctx, u16 pf_q)
 	 * 1: Legacy Host Interface
 	 */
 	tlan_ctx->legacy_int = ICE_TX_LEGACY;
-	tlan_ctx->tsyn_ena = 1;
 }
 
 /**
@@ -370,10 +415,35 @@ static int ice_setup_rx_ctx(struct ice_ring *ring)
 	/* Strip the Ethernet CRC bytes before the packet is posted to host
 	 * memory.
 	 */
-	rlan_ctx.crcstrip = 1;
+	rlan_ctx.crcstrip = ring->rx_crc_strip_dis ? 0 : 1;
 
-	/* L2TSEL flag defines the reported L2 Tags in the receive descriptor */
-	rlan_ctx.l2tsel = 1;
+	/* L2TSEL flag defines the reported L2 Tags in the receive descriptor
+	 * and it needs to remain 1 for non-DVM capable configurations to not
+	 * break backward compatibility for VF drivers. Setting this field to 0
+	 * will cause the single/outer VLAN tag to be stripped to the L2TAG2_2ND
+	 * field in the Rx descriptor. Setting it to 1 allows the VLAN tag to
+	 * be stripped in L2TAG1 of the Rx descriptor, which is where VFs will
+	 * check for the tag
+	 */
+	if (ice_is_dvm_ena(hw)) {
+		if (vsi->type == ICE_VSI_VF) {
+			struct ice_vf *vf = vsi->vf;
+
+			if (vf && ice_vf_is_port_vlan_ena(vf)) {
+				rlan_ctx.l2tsel = 1;
+			} else if (!vf) {
+				WARN(1, "VF VSI %u has NULL VF pointer",
+				     vsi->vsi_num);
+				rlan_ctx.l2tsel = 0;
+			} else {
+				rlan_ctx.l2tsel = 0;
+			}
+		} else {
+			rlan_ctx.l2tsel = 0;
+		}
+	} else {
+		rlan_ctx.l2tsel = 1;
+	}
 
 	rlan_ctx.dtype = ICE_RX_DTYPE_NO_SPLIT;
 	rlan_ctx.hsplit_0 = ICE_RLAN_RX_HSPLIT_0_NO_SPLIT;
@@ -390,7 +460,7 @@ static int ice_setup_rx_ctx(struct ice_ring *ring)
 	 * multiple buffers, thus letting us skip that
 	 * handling in the fast-path.
 	 */
-	if (ring->xsk_umem)
+	if (ring->xsk_pool)
 		chain_len = 1;
 #endif /* HAVE_AF_XDP_ZC_SUPPORT */
 	/* Max packet size for this queue - must not be set to a larger value
@@ -408,22 +478,25 @@ static int ice_setup_rx_ctx(struct ice_ring *ring)
 	 * setting to 0x03 to ensure profile is programming if prev context is
 	 * of same priority
 	 */
-	if (vsi->type != ICE_VSI_VF)
-		ice_write_qrxflxp_cntxt(hw, pf_q, rxdid, 0x3, true);
-	else
+	switch (vsi->type) {
+	case ICE_VSI_ADI:
+	case ICE_VSI_VF:
 		ice_write_qrxflxp_cntxt(hw, pf_q, ICE_RXDID_LEGACY_1, 0x3,
 					false);
+		break;
+	default:
+		ice_write_qrxflxp_cntxt(hw, pf_q, rxdid, 0x3, true);
+	}
 
 	/* Absolute queue number out of 2K needs to be passed */
 	err = ice_write_rxq_ctx(hw, &rlan_ctx, pf_q);
 	if (err) {
-		dev_err(ice_pf_to_dev(vsi->back),
-			"Failed to set LAN Rx queue context for absolute Rx queue %d error: %d\n",
+		dev_err(ice_pf_to_dev(vsi->back), "Failed to set LAN Rx queue context for absolute Rx queue %d error: %d\n",
 			pf_q, err);
 		return -EIO;
 	}
 
-	if (vsi->type == ICE_VSI_VF)
+	if (vsi->type == ICE_VSI_VF || vsi->type == ICE_VSI_ADI)
 		return 0;
 
 	/* configure Rx buffer alignment */
@@ -453,19 +526,21 @@ int ice_vsi_cfg_rxq(struct ice_ring *ring)
 
 	ring->rx_buf_len = ring->vsi->rx_buf_len;
 
+#ifdef HAVE_XDP_BUFF_RXQ
 #ifdef HAVE_AF_XDP_ZC_SUPPORT
 	if (ring->vsi->type == ICE_VSI_PF) {
 		if (!xdp_rxq_info_is_reg(&ring->xdp_rxq))
 			/* coverity[check_return] */
 			xdp_rxq_info_reg(&ring->xdp_rxq, ring->netdev,
-					 ring->q_index);
+					 ring->q_index, ring->q_vector->napi.napi_id);
 
-		ring->xsk_umem = ice_xsk_umem(ring);
-		if (ring->xsk_umem) {
+		ring->xsk_pool = ice_xsk_umem(ring);
+		if (ring->xsk_pool) {
 			xdp_rxq_info_unreg_mem_model(&ring->xdp_rxq);
 
-			ring->rx_buf_len = ring->xsk_umem->chunk_size_nohr -
-					   XDP_PACKET_HEADROOM;
+			ring->rx_buf_len =
+				xsk_pool_get_rx_frame_size(ring->xsk_pool);
+#ifndef HAVE_MEM_TYPE_XSK_BUFF_POOL
 			ring->zca.free = ice_zca_free;
 			err = xdp_rxq_info_reg_mem_model(&ring->xdp_rxq,
 							 MEM_TYPE_ZERO_COPY,
@@ -475,13 +550,26 @@ int ice_vsi_cfg_rxq(struct ice_ring *ring)
 
 			dev_info(dev, "Registered XDP mem model MEM_TYPE_ZERO_COPY on Rx ring %d\n",
 				 ring->q_index);
+#else
+			err = xdp_rxq_info_reg_mem_model(&ring->xdp_rxq,
+							 MEM_TYPE_XSK_BUFF_POOL,
+							 NULL);
+			if (err)
+				return err;
+			xsk_pool_set_rxq_info(ring->xsk_pool, &ring->xdp_rxq);
+
+			dev_info(dev, "Registered XDP mem model MEM_TYPE_XSK_BUFF_POOL on Rx ring %d\n",
+				 ring->q_index);
+#endif
 		} else {
+#ifndef HAVE_MEM_TYPE_XSK_BUFF_POOL
 			ring->zca.free = NULL;
+#endif
 			if (!xdp_rxq_info_is_reg(&ring->xdp_rxq))
 				/* coverity[check_return] */
 				xdp_rxq_info_reg(&ring->xdp_rxq,
 						 ring->netdev,
-						 ring->q_index);
+						 ring->q_index, ring->q_vector->napi.napi_id);
 
 			err = xdp_rxq_info_reg_mem_model(&ring->xdp_rxq,
 							 MEM_TYPE_PAGE_SHARED,
@@ -495,7 +583,7 @@ int ice_vsi_cfg_rxq(struct ice_ring *ring)
 		if (!xdp_rxq_info_is_reg(&ring->xdp_rxq))
 			/* coverity[check_return] */
 			xdp_rxq_info_reg(&ring->xdp_rxq, ring->netdev,
-					 ring->q_index);
+					 ring->q_index, ring->q_vector->napi.napi_id);
 
 		err = xdp_rxq_info_reg_mem_model(&ring->xdp_rxq,
 						 MEM_TYPE_PAGE_SHARED, NULL);
@@ -503,18 +591,20 @@ int ice_vsi_cfg_rxq(struct ice_ring *ring)
 			return err;
 	}
 #endif /* HAVE_AF_XDP_ZC_SUPPORT */
+#endif /* HAVE_XDP_BUFF_RXQ */
 
 	err = ice_setup_rx_ctx(ring);
 	if (err) {
-		dev_err(dev, "ice_setup_rx_ctx failed for RxQ %d, err %d\n",
-			ring->q_index, err);
+		ice_dev_err_errno(dev, err,
+				  "ice_setup_rx_ctx failed for RxQ %d",
+				  ring->q_index);
 		return err;
 	}
 
 #ifdef HAVE_AF_XDP_ZC_SUPPORT
-	if (ring->xsk_umem) {
+	if (ring->xsk_pool) {
 #ifdef HAVE_XSK_UMEM_HAS_ADDRS
-		if (!xsk_umem_has_addrs_rq(ring->xsk_umem, num_bufs)) {
+		if (!xsk_umem_has_addrs_rq(ring->xsk_pool, num_bufs)) {
 			dev_warn(dev, "UMEM does not provide enough addresses to fill %d buffers on Rx ring %d\n",
 				 num_bufs, ring->q_index);
 			dev_warn(dev, "Change Rx ring/fill queue size to avoid performance issues\n");
@@ -523,7 +613,11 @@ int ice_vsi_cfg_rxq(struct ice_ring *ring)
 		}
 #endif /* HAVE_XSK_UMEM_HAS_ADDRS */
 
+#ifndef HAVE_MEM_TYPE_XSK_BUFF_POOL
 		err = ice_alloc_rx_bufs_slow_zc(ring, num_bufs);
+#else
+		err = ice_alloc_rx_bufs_zc(ring, ICE_DESC_UNUSED(ring));
+#endif
 		if (err) {
 			u16 pf_q = ring->vsi->rxq_map[ring->q_index];
 
@@ -650,8 +744,9 @@ err_out:
 	while (v_idx--)
 		ice_free_q_vector(vsi, v_idx);
 
-	dev_err(dev, "Failed to allocate %d q_vector for VSI %d, ret=%d\n",
-		vsi->num_q_vectors, vsi->vsi_num, err);
+	ice_dev_err_errno(dev, err,
+			  "Failed to allocate %d q_vector for VSI %d",
+			  vsi->num_q_vectors, vsi->vsi_num);
 	vsi->num_q_vectors = 0;
 	return err;
 }
@@ -690,9 +785,14 @@ void ice_vsi_map_rings_to_vectors(struct ice_vsi *vsi)
 		for (q_id = q_base; q_id < (q_base + tx_rings_per_v); q_id++) {
 			struct ice_ring *tx_ring = vsi->tx_rings[q_id];
 
-			tx_ring->q_vector = q_vector;
-			tx_ring->next = q_vector->tx.ring;
-			q_vector->tx.ring = tx_ring;
+			if (tx_ring) {
+				tx_ring->q_vector = q_vector;
+				tx_ring->next = q_vector->tx.ring;
+				q_vector->tx.ring = tx_ring;
+			} else {
+				dev_err(ice_pf_to_dev(vsi->back), "NULL Tx ring found\n");
+				break;
+			}
 		}
 		tx_rings_rem -= tx_rings_per_v;
 
@@ -707,9 +807,14 @@ void ice_vsi_map_rings_to_vectors(struct ice_vsi *vsi)
 		for (q_id = q_base; q_id < (q_base + rx_rings_per_v); q_id++) {
 			struct ice_ring *rx_ring = vsi->rx_rings[q_id];
 
-			rx_ring->q_vector = q_vector;
-			rx_ring->next = q_vector->rx.ring;
-			q_vector->rx.ring = rx_ring;
+			if (rx_ring) {
+				rx_ring->q_vector = q_vector;
+				rx_ring->next = q_vector->rx.ring;
+				q_vector->rx.ring = rx_ring;
+			} else {
+				dev_err(ice_pf_to_dev(vsi->back), "NULL Rx ring found\n");
+				break;
+			}
 		}
 		rx_rings_rem -= rx_rings_per_v;
 	}
@@ -730,17 +835,17 @@ void ice_vsi_free_q_vectors(struct ice_vsi *vsi)
 /**
  * ice_vsi_cfg_txq - Configure single Tx queue
  * @vsi: the VSI that queue belongs to
- * @ring: Tx ring to be configured
+ * @tx_ring: Tx ring to be configured
  * @qg_buf: queue group buffer
  */
 int
-ice_vsi_cfg_txq(struct ice_vsi *vsi, struct ice_ring *ring,
+ice_vsi_cfg_txq(struct ice_vsi *vsi, struct ice_ring *tx_ring,
 		struct ice_aqc_add_tx_qgrp *qg_buf)
 {
 	u8 buf_len = struct_size(qg_buf, txqs, 1);
 	struct ice_tlan_ctx tlan_ctx = { 0 };
+	struct ice_channel *ch = tx_ring->ch;
 	struct ice_aqc_add_txqs_perq *txq;
-	struct ice_channel *ch = ring->ch;
 	struct ice_pf *pf = vsi->back;
 	struct ice_hw *hw = &pf->hw;
 	enum ice_status status;
@@ -748,10 +853,10 @@ ice_vsi_cfg_txq(struct ice_vsi *vsi, struct ice_ring *ring,
 	u8 tc;
 
 	/* Configure XPS */
-	ice_cfg_xps_tx_ring(ring);
+	ice_cfg_xps_tx_ring(tx_ring);
 
-	pf_q = ring->reg_idx;
-	ice_setup_tx_ctx(ring, &tlan_ctx, pf_q);
+	pf_q = tx_ring->reg_idx;
+	ice_setup_tx_ctx(tx_ring, &tlan_ctx, pf_q);
 	/* copy context contents into the qg_buf */
 	qg_buf->txqs[0].txq_id = cpu_to_le16(pf_q);
 	ice_set_ctx(hw, (u8 *)&tlan_ctx, qg_buf->txqs[0].txq_ctx,
@@ -760,23 +865,28 @@ ice_vsi_cfg_txq(struct ice_vsi *vsi, struct ice_ring *ring,
 	/* init queue specific tail reg. It is referred as
 	 * transmit comm scheduler queue doorbell.
 	 */
-	ring->tail = hw->hw_addr + QTX_COMM_DBELL(pf_q);
-
+	tx_ring->tail = hw->hw_addr + QTX_COMM_DBELL(pf_q);
 	if (IS_ENABLED(CONFIG_DCB))
-		tc = ring->dcb_tc;
+		tc = tx_ring->dcb_tc;
 	else
 		tc = 0;
 
 	/* Add unique software queue handle of the Tx queue per
 	 * TC into the VSI Tx ring
 	 */
-	ring->q_handle = ice_calc_q_handle(vsi, ring, tc);
+	if (vsi->type == ICE_VSI_SWITCHDEV_CTRL)
+		tx_ring->q_handle = ice_eswitch_calc_q_handle(tx_ring);
+	else
+		tx_ring->q_handle = ice_calc_q_handle(vsi, tx_ring, tc);
 
-	status = (ch ?
-		  ice_ena_vsi_txq(vsi->port_info, ch->ch_vsi->idx, 0,
-				  ring->q_handle, 1, qg_buf, buf_len, NULL) :
-		  ice_ena_vsi_txq(vsi->port_info, vsi->idx, tc,
-				  ring->q_handle, 1, qg_buf, buf_len, NULL));
+	if (ch)
+		status = ice_ena_vsi_txq(vsi->port_info, ch->ch_vsi->idx, 0,
+					 tx_ring->q_handle, 1, qg_buf, buf_len,
+					 NULL);
+	else
+		status = ice_ena_vsi_txq(vsi->port_info, vsi->idx, tc,
+					 tx_ring->q_handle, 1, qg_buf, buf_len,
+					 NULL);
 	if (status) {
 		dev_err(ice_pf_to_dev(pf), "Failed to set LAN Tx queue context, error: %s\n",
 			ice_stat_str(status));
@@ -789,7 +899,7 @@ ice_vsi_cfg_txq(struct ice_vsi *vsi, struct ice_ring *ring,
 	 */
 	txq = &qg_buf->txqs[0];
 	if (pf_q == le16_to_cpu(txq->txq_id))
-		ring->txq_teid = le32_to_cpu(txq->q_teid);
+		tx_ring->txq_teid = le32_to_cpu(txq->q_teid);
 
 	return 0;
 }
@@ -806,25 +916,13 @@ void ice_cfg_itr(struct ice_hw *hw, struct ice_q_vector *q_vector)
 {
 	ice_cfg_itr_gran(hw);
 
-	if (q_vector->num_ring_rx) {
-		struct ice_ring_container *rc = &q_vector->rx;
+	if (q_vector->num_ring_rx)
+		ice_write_itr(&q_vector->rx, q_vector->rx.itr_setting);
 
-		rc->target_itr = ITR_TO_REG(rc->itr_setting);
-		rc->next_update = jiffies + 1;
-		rc->current_itr = rc->target_itr;
-		wr32(hw, GLINT_ITR(rc->itr_idx, q_vector->reg_idx),
-		     ITR_REG_ALIGN(rc->current_itr) >> ICE_ITR_GRAN_S);
-	}
+	if (q_vector->num_ring_tx)
+		ice_write_itr(&q_vector->tx, q_vector->tx.itr_setting);
 
-	if (q_vector->num_ring_tx) {
-		struct ice_ring_container *rc = &q_vector->tx;
-
-		rc->target_itr = ITR_TO_REG(rc->itr_setting);
-		rc->next_update = jiffies + 1;
-		rc->current_itr = rc->target_itr;
-		wr32(hw, GLINT_ITR(rc->itr_idx, q_vector->reg_idx),
-		     ITR_REG_ALIGN(rc->current_itr) >> ICE_ITR_GRAN_S);
-	}
+	ice_write_intrl(q_vector, q_vector->intrl);
 }
 
 /**
@@ -949,7 +1047,7 @@ ice_vsi_stop_tx_ring(struct ice_vsi *vsi, enum ice_disq_rst_src rst_src,
 	} else if (status == ICE_ERR_DOES_NOT_EXIST) {
 		dev_dbg(ice_pf_to_dev(vsi->back), "LAN Tx queues do not exist, nothing to disable\n");
 	} else if (status) {
-		dev_err(ice_pf_to_dev(vsi->back), "Failed to disable LAN Tx queues, error: %s\n",
+		dev_dbg(ice_pf_to_dev(vsi->back), "Failed to disable LAN Tx queues, error: %s\n",
 			ice_stat_str(status));
 		return -ENODEV;
 	}
