@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: GPL-2.0
-/* Copyright (C) 2018-2019, Intel Corporation. */
+/* Copyright (C) 2018-2021, Intel Corporation. */
 
 #include "ice_dcb_lib.h"
 #include "ice_dcb_nl.h"
@@ -184,16 +184,18 @@ void ice_vsi_set_dcb_tc_cfg(struct ice_vsi *vsi)
 
 	switch (vsi->type) {
 	case ICE_VSI_PF:
+	case ICE_VSI_VF:
 		vsi->tc_cfg.ena_tc = ice_dcb_get_ena_tc(cfg);
 		vsi->tc_cfg.numtc = ice_dcb_get_num_tc(cfg);
 		break;
 #ifdef HAVE_TC_CB_AND_SETUP_QDISC_MQPRIO
 	case ICE_VSI_CHNL:
 #endif /* HAVE_TC_CB_AND_SETUP_QDISC_MQPRIO */
-#ifdef HAVE_NETDEV_SB_DEV
+#ifdef HAVE_NDO_DFWD_OPS
 	case ICE_VSI_OFFLOAD_MACVLAN:
-#endif /* HAVE_NETDEV_SB_DEV */
+#endif /* HAVE_NDO_DFWD_OPS */
 	case ICE_VSI_VMDQ2:
+	case ICE_VSI_SWITCHDEV_CTRL:
 		vsi->tc_cfg.ena_tc = BIT(ice_get_first_droptc(vsi));
 		vsi->tc_cfg.numtc = 1;
 		break;
@@ -223,8 +225,7 @@ u8 ice_dcb_get_tc(struct ice_vsi *vsi, int queue_index)
  */
 void ice_vsi_cfg_dcb_rings(struct ice_vsi *vsi)
 {
-	u16 qoffset;
-	u16 qcount;
+	u16 qoffset, qcount;
 	int i, n;
 
 	if (!test_bit(ICE_FLAG_DCB_ENA, vsi->back->flags)) {
@@ -253,7 +254,7 @@ void ice_vsi_cfg_dcb_rings(struct ice_vsi *vsi)
 			vsi->rx_rings[i]->dcb_tc = n;
 	}
 
-#ifdef HAVE_NETDEV_SB_DEV
+#ifdef HAVE_NDO_DFWD_OPS
 	/* when DCB is configured TC for MACVLAN queues should be
 	 * the first drop TC of the main VSI
 	 */
@@ -265,7 +266,7 @@ void ice_vsi_cfg_dcb_rings(struct ice_vsi *vsi)
 		ice_for_each_alloc_rxq(vsi, i)
 			vsi->rx_rings[i]->dcb_tc = first_droptc;
 	}
-#endif /* HAVE_NETDEV_SB_DEV */
+#endif /* HAVE_NDO_DFWD_OPS */
 
 #ifdef HAVE_TC_CB_AND_SETUP_QDISC_MQPRIO
 	if (vsi->type == ICE_VSI_PF) {
@@ -289,30 +290,6 @@ void ice_vsi_cfg_dcb_rings(struct ice_vsi *vsi)
 #endif /* HAVE_TC_CB_AND_SETUP_QDISC_MQPRIO */
 }
 
-/**
- * ice_peer_prep_tc_change - Pre-notify RDMA Peer in blocking call of TC change
- * @peer_dev_int: ptr to peer device internal struct
- * @data: ptr to opaque data
- */
-static int
-ice_peer_prep_tc_change(struct ice_peer_dev_int *peer_dev_int,
-			void __always_unused *data)
-{
-	struct ice_peer_dev *peer_dev;
-
-	peer_dev = ice_get_peer_dev(peer_dev_int);
-	if (!ice_validate_peer_dev(peer_dev))
-		return 0;
-
-	if (!test_bit(ICE_PEER_DEV_STATE_OPENED, peer_dev_int->state))
-		return 0;
-
-	if (peer_dev->peer_ops && peer_dev->peer_ops->prep_tc_change)
-		peer_dev->peer_ops->prep_tc_change(peer_dev);
-
-	return 0;
-}
-
 /**
  * ice_dcb_ena_dis_vsi - disable certain VSIs for DCB config/reconfig
  * @pf: pointer to the PF instance
@@ -338,6 +315,7 @@ static void ice_dcb_ena_dis_vsi(struct ice_pf *pf, bool ena, bool locked)
 		case ICE_VSI_CHNL:
 		case ICE_VSI_OFFLOAD_MACVLAN:
 		case ICE_VSI_VMDQ2:
+		case ICE_VSI_SWITCHDEV_CTRL:
 		case ICE_VSI_PF:
 			if (ena)
 				ice_ena_vsi(vsi, locked);
@@ -352,7 +330,7 @@ static void ice_dcb_ena_dis_vsi(struct ice_pf *pf, bool ena, bool locked)
 
 /**
  * ice_dcb_bwchk - check if ETS bandwidth input parameters are correct
- * @pf: pointer to PF struct
+ * @pf: pointer to the PF struct
  * @dcbcfg: pointer to DCB config structure
  */
 int ice_dcb_bwchk(struct ice_pf *pf, struct ice_dcbx_cfg *dcbcfg)
@@ -380,8 +358,7 @@ int ice_dcb_bwchk(struct ice_pf *pf, struct ice_dcbx_cfg *dcbcfg)
 	if (!total_bw) {
 		etscfg->tcbwtable[0] = ICE_TC_MAX_BW;
 	} else if (total_bw != ICE_TC_MAX_BW) {
-		dev_err(ice_pf_to_dev(pf),
-			"Invalid config, total bandwidth must equal 100\n");
+		dev_err(ice_pf_to_dev(pf), "Invalid config, total bandwidth must equal 100\n");
 		return -EINVAL;
 	}
 
@@ -400,6 +377,7 @@ int ice_pf_dcb_cfg(struct ice_pf *pf, struct ice_dcbx_cfg *new_cfg, bool locked)
 	struct ice_dcbx_cfg *old_cfg, *curr_cfg;
 	struct device *dev = ice_pf_to_dev(pf);
 	int ret = ICE_DCB_NO_HW_CHG;
+	struct iidc_event *event;
 
 	curr_cfg = &pf->hw.port_info->qos_cfg.local_dcbx_cfg;
 
@@ -430,8 +408,16 @@ int ice_pf_dcb_cfg(struct ice_pf *pf, struct ice_dcbx_cfg *new_cfg, bool locked)
 		return -ENOMEM;
 
 	dev_info(dev, "Commit DCB Configuration to the hardware\n");
-	/* Notify capable peers about impending change to TCs */
-	ice_for_each_peer(pf, NULL, ice_peer_prep_tc_change);
+	/* Notify capable aux drivers about impending change to TCs */
+	event = kzalloc(sizeof(*event), GFP_KERNEL);
+	if (!event) {
+		kfree(old_cfg);
+		return -ENOMEM;
+	}
+
+	set_bit(IIDC_EVENT_BEFORE_TC_CHANGE, event->type);
+	ice_send_event_to_auxs(pf, event);
+	kfree(event);
 
 	/* avoid race conditions by holding the lock while disabling and
 	 * re-enabling the VSI
@@ -654,7 +640,7 @@ static int ice_dcb_init_cfg(struct ice_pf *pf, bool locked)
  * @ets_willing: configure ETS willing
  * @locked: was this function called with RTNL held
  */
-static int ice_dcb_sw_dflt_cfg(struct ice_pf *pf, bool ets_willing, bool locked)
+int ice_dcb_sw_dflt_cfg(struct ice_pf *pf, bool ets_willing, bool locked)
 {
 	struct ice_aqc_port_ets_elem buf = { 0 };
 	struct ice_dcbx_cfg *dcbcfg;
@@ -738,7 +724,7 @@ static int ice_dcb_noncontig_cfg(struct ice_pf *pf)
 	/* Configure SW DCB default with ETS non-willing */
 	ret = ice_dcb_sw_dflt_cfg(pf, false, true);
 	if (ret) {
-		dev_err(dev, "Failed to set local DCB config %d\n", ret);
+		ice_dev_err_errno(dev, ret, "Failed to set local DCB config");
 		return ret;
 	}
 
@@ -762,10 +748,11 @@ static int ice_dcb_noncontig_cfg(struct ice_pf *pf)
 void ice_pf_dcb_recfg(struct ice_pf *pf)
 {
 	struct ice_dcbx_cfg *dcbcfg = &pf->hw.port_info->qos_cfg.local_dcbx_cfg;
+	struct iidc_core_dev_info *cdev_info;
+	struct iidc_event *event;
 	u8 tc_map = 0;
 	int v, ret;
 
-
 	/* Update each VSI */
 	ice_for_each_vsi(pf, v) {
 		struct ice_vsi *vsi = pf->vsi[v];
@@ -774,6 +761,8 @@ void ice_pf_dcb_recfg(struct ice_pf *pf)
 			continue;
 
 		if (vsi->type == ICE_VSI_PF) {
+			if (ice_dcb_get_num_tc(dcbcfg) > vsi->alloc_txq)
+				dev_warn(ice_pf_to_dev(vsi->back), "More TCs defined than queues/rings allocated.\n");
 			tc_map = ice_dcb_get_ena_tc(dcbcfg);
 
 			/* If DCBX request non-contiguous TC, then configure
@@ -783,11 +772,11 @@ void ice_pf_dcb_recfg(struct ice_pf *pf)
 				tc_map = ICE_DFLT_TRAFFIC_CLASS;
 				ice_dcb_noncontig_cfg(pf);
 			}
-#if defined(HAVE_TC_CB_AND_SETUP_QDISC_MQPRIO) && defined(HAVE_NETDEV_SB_DEV)
+#if defined(HAVE_TC_CB_AND_SETUP_QDISC_MQPRIO) && defined(HAVE_NDO_DFWD_OPS)
 		} else if (vsi->type == ICE_VSI_CHNL ||
 			   vsi->type == ICE_VSI_OFFLOAD_MACVLAN) {
 			tc_map = BIT(ice_get_first_droptc(vsi));
-# endif /* HAVE_TC_CB_AND_SETUP_QDISC_MQPRIO && HAVE_NETDEV_SB_DEV */
+# endif /* HAVE_TC_CB_AND_SETUP_QDISC_MQPRIO && HAVE_NDO_DFWD_OPS */
 		} else {
 			tc_map = ICE_DFLT_TRAFFIC_CLASS;
 		}
@@ -798,20 +787,32 @@ void ice_pf_dcb_recfg(struct ice_pf *pf)
 				vsi->idx);
 			continue;
 		}
-		/* no need to proceed with remaining cfg if it is CHNL VSI */
-		if (vsi->type == ICE_VSI_CHNL)
+		/* no need to proceed with remaining cfg if it is CHNL
+		 * or switchdev VSI
+		 */
+		if (vsi->type == ICE_VSI_CHNL ||
+		    vsi->type == ICE_VSI_SWITCHDEV_CTRL)
 			continue;
 
 		ice_vsi_map_rings_to_vectors(vsi);
 		if (vsi->type == ICE_VSI_PF)
 			ice_dcbnl_set_all(vsi);
 	}
-	/* If the RDMA peer is registered, update that peer's initial_qos_info struct.
-	 * The peer is closed during this process, so when it is opened, it will access
-	 * the initial_qos_info element to configure itself.
+	/* Notify the aux drivers that TC change is finished
 	 */
-	if (pf->rdma_peer)
-		ice_setup_dcb_qos_info(pf, &pf->rdma_peer->initial_qos_info);
+	cdev_info = ice_find_cdev_info_by_id(pf, IIDC_RDMA_ID);
+	if (cdev_info) {
+		ice_setup_dcb_qos_info(pf, &cdev_info->qos_info);
+
+		event = kzalloc(sizeof(*event), GFP_KERNEL);
+		if (!event)
+			return;
+
+		set_bit(IIDC_EVENT_AFTER_TC_CHANGE, event->type);
+		event->info.port_qos = cdev_info->qos_info;
+		ice_send_event_to_auxs(pf, event);
+		kfree(event);
+	}
 }
 
 /**
@@ -824,28 +825,36 @@ int ice_init_pf_dcb(struct ice_pf *pf, bool locked)
 	struct device *dev = ice_pf_to_dev(pf);
 	struct ice_port_info *port_info;
 	struct ice_hw *hw = &pf->hw;
+	enum ice_status status;
 	int err;
 
 	port_info = hw->port_info;
 
-	err = ice_init_dcb(hw, false);
-	if (err && !port_info->qos_cfg.is_sw_lldp) {
-		dev_err(dev, "Error initializing DCB %d\n", err);
+	status = ice_init_dcb(hw, false);
+	if (status && !port_info->qos_cfg.is_sw_lldp) {
+		dev_err(dev, "Error initializing DCB %s\n",
+			ice_stat_str(status));
+		err = ice_status_to_errno(status);
 		goto dcb_init_err;
 	}
 
 	dev_info(dev, "DCB is enabled in the hardware, max number of TCs supported on this port are %d\n",
 		 pf->hw.func_caps.common_cap.maxtc);
-	if (err) {
+	if (port_info->qos_cfg.is_sw_lldp) {
 		struct ice_vsi *pf_vsi;
 
 		/* FW LLDP is disabled, activate SW DCBX/LLDP mode */
 		dev_info(dev, "FW LLDP is disabled, DCBx/LLDP in SW mode.\n");
 		clear_bit(ICE_FLAG_FW_LLDP_AGENT, pf->flags);
+		err = ice_aq_set_pfc_mode(&pf->hw, ICE_AQC_PFC_VLAN_BASED_PFC,
+					  NULL);
+		if (err)
+			dev_info(dev, "Failed to set VLAN PFC mode\n");
+
 		err = ice_dcb_sw_dflt_cfg(pf, true, locked);
 		if (err) {
-			dev_err(dev, "Failed to set local DCB config %d\n",
-				err);
+			ice_dev_err_errno(dev, err,
+					  "Failed to set local DCB config");
 			err = -EIO;
 			goto dcb_init_err;
 		}
@@ -875,7 +884,7 @@ int ice_init_pf_dcb(struct ice_pf *pf, bool locked)
 	if (err)
 		goto dcb_init_err;
 
-	return err;
+	return 0;
 
 dcb_init_err:
 	dev_err(dev, "DCB init failed\n");
@@ -939,7 +948,8 @@ ice_tx_prepare_vlan_flags_dcb(struct ice_ring *tx_ring,
 		return;
 
 	/* Insert 802.1p priority into VLAN header */
-	if ((first->tx_flags & ICE_TX_FLAGS_HW_VLAN) ||
+	if ((first->tx_flags & ICE_TX_FLAGS_HW_VLAN ||
+	     first->tx_flags & ICE_TX_FLAGS_HW_OUTER_SINGLE_VLAN) ||
 	    skb->priority != TC_PRIO_CONTROL) {
 		first->tx_flags &= ~ICE_TX_FLAGS_VLAN_PR_M;
 		/* Mask the lower 3 bits to set the 802.1p priority */
@@ -948,7 +958,10 @@ ice_tx_prepare_vlan_flags_dcb(struct ice_ring *tx_ring,
 		/* if this is not already set it means a VLAN 0 + priority needs
 		 * to be offloaded
 		 */
-		first->tx_flags |= ICE_TX_FLAGS_HW_VLAN;
+		if (tx_ring->flags & ICE_TX_FLAGS_VLAN_TAG_LOC_L2TAG2)
+			first->tx_flags |= ICE_TX_FLAGS_HW_OUTER_SINGLE_VLAN;
+		else
+			first->tx_flags |= ICE_TX_FLAGS_HW_VLAN;
 	}
 }
 
@@ -957,19 +970,28 @@ ice_tx_prepare_vlan_flags_dcb(struct ice_ring *tx_ring,
  * @pf: ptr to ice_pf
  * @qos_info: QoS param instance
  */
-void ice_setup_dcb_qos_info(struct ice_pf *pf, struct ice_qos_params *qos_info)
+void ice_setup_dcb_qos_info(struct ice_pf *pf, struct iidc_qos_params *qos_info)
 {
+	struct iidc_core_dev_info *cdev_info;
 	struct ice_dcbx_cfg *dcbx_cfg;
 	unsigned int i;
 	u32 up2tc;
 
+	if (!pf || !qos_info)
+		return;
+
+	cdev_info = ice_find_cdev_info_by_id(pf, IIDC_RDMA_ID);
+
+	if (!cdev_info)
+		return;
+
 	dcbx_cfg = &pf->hw.port_info->qos_cfg.local_dcbx_cfg;
 	up2tc = rd32(&pf->hw, PRTDCB_TUP2TC);
 	qos_info->num_apps = dcbx_cfg->numapps;
 
 	qos_info->num_tc = ice_dcb_get_num_tc(dcbx_cfg);
 
-	for (i = 0; i < ICE_IDC_MAX_USER_PRIORITY; i++)
+	for (i = 0; i < IIDC_MAX_USER_PRIORITY; i++)
 		qos_info->up2tc[i] = (up2tc >> (i * 3)) & 0x7;
 
 	for (i = 0; i < IEEE_8021QAZ_MAX_TCS; i++)
@@ -981,6 +1003,10 @@ void ice_setup_dcb_qos_info(struct ice_pf *pf, struct ice_qos_params *qos_info)
 		qos_info->apps[i].prot_id = dcbx_cfg->app[i].prot_id;
 		qos_info->apps[i].selector = dcbx_cfg->app[i].selector;
 	}
+
+	qos_info->pfc_mode = dcbx_cfg->pfc_mode;
+	for (i = 0; i < ICE_DSCP_NUM_VAL; i++)
+		qos_info->dscp_map[i] = dcbx_cfg->dscp_map[i];
 }
 
 /**
@@ -994,11 +1020,13 @@ ice_dcb_process_lldp_set_mib_change(struct ice_pf *pf,
 {
 	struct ice_aqc_port_ets_elem buf = { 0 };
 	struct device *dev = ice_pf_to_dev(pf);
+	struct iidc_core_dev_info *cdev_info;
 	struct ice_aqc_lldp_get_mib *mib;
 	struct ice_dcbx_cfg tmp_dcbx_cfg;
 	bool need_reconfig = false;
 	struct ice_port_info *pi;
 	u8 mib_type;
+	u32 numtc;
 	int ret;
 
 	/* Not DCB capable or capability disabled */
@@ -1033,6 +1061,7 @@ ice_dcb_process_lldp_set_mib_change(struct ice_pf *pf,
 		}
 	}
 
+	/* That a DCB change has happened is now determined */
 	mutex_lock(&pf->tc_mutex);
 
 	/* store the old configuration */
@@ -1065,7 +1094,8 @@ ice_dcb_process_lldp_set_mib_change(struct ice_pf *pf,
 		goto out;
 
 	/* Enable DCB tagging only when more than one TC */
-	if (ice_dcb_get_num_tc(&pi->qos_cfg.local_dcbx_cfg) > 1) {
+	numtc = ice_dcb_get_num_tc(&pi->qos_cfg.local_dcbx_cfg);
+	if (numtc > 1) {
 		dev_dbg(dev, "DCB tagging enabled (num TC > 1)\n");
 		set_bit(ICE_FLAG_DCB_ENA, pf->flags);
 	} else {
@@ -1073,6 +1103,25 @@ ice_dcb_process_lldp_set_mib_change(struct ice_pf *pf,
 		clear_bit(ICE_FLAG_DCB_ENA, pf->flags);
 	}
 
+	if (numtc > pf->hw.func_caps.common_cap.maxtc)
+		dev_warn(dev, "%d TCs more than supported max of %d\n", numtc,
+			 pf->hw.func_caps.common_cap.maxtc);
+
+	cdev_info = ice_find_cdev_info_by_id(pf, IIDC_RDMA_ID);
+	if (cdev_info) {
+		struct iidc_event *ievent;
+
+		/* can't fail the LAN flow based on a failure to notify
+		 * the RDMA driver
+		 */
+		ievent = kzalloc(sizeof(*ievent), GFP_KERNEL);
+		if (ievent) {
+			set_bit(IIDC_EVENT_BEFORE_TC_CHANGE, ievent->type);
+			ice_send_event_to_auxs(pf, ievent);
+			kfree(ievent);
+		}
+	}
+
 	rtnl_lock();
 	/* disable VSIs affected by DCB changes */
 	ice_dcb_ena_dis_vsi(pf, false, true);
@@ -1094,4 +1143,5 @@ out:
 	mutex_unlock(&pf->tc_mutex);
 }
 
-#endif
+#endif /* HAVE_PF_RING */
+
