@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: GPL-2.0
-/* Copyright (C) 2018-2019, Intel Corporation. */
+/* Copyright (C) 2018-2021, Intel Corporation. */
 
 #include "ice.h"
 
@@ -11,6 +11,8 @@ static const enum ice_adminq_opc aqc_permitted_tbl[] = {
 	ice_aqc_opc_list_func_caps,
 	ice_aqc_opc_list_dev_caps,
 
+	ice_aqc_opc_get_vlan_mode_parameters,
+
 	/* Package Configuration Admin Commands */
 	ice_aqc_opc_update_pkg,
 	ice_aqc_opc_get_pkg_info_list,
@@ -52,6 +54,9 @@ static const enum ice_adminq_opc aqc_permitted_tbl[] = {
 	ice_aqc_opc_query_acl_entry,
 	ice_aqc_opc_query_acl_actpair,
 	ice_aqc_opc_query_acl_counter,
+
+	/* QoS */
+	ice_aqc_opc_query_port_ets,
 };
 
 /**
@@ -85,6 +90,88 @@ bool ice_dcf_is_acl_aq_cmd(struct ice_aq_desc *desc)
 	return false;
 }
 
+/**
+ * ice_dcf_is_udp_tunnel_aq_cmd - check if the AdminQ command is UDP tunnel
+ * command
+ * @desc: descriptor describing the command
+ * @aq_buf: AdminQ buffer
+ */
+bool ice_dcf_is_udp_tunnel_aq_cmd(struct ice_aq_desc *desc, u8 *aq_buf)
+{
+	struct ice_buf_hdr *pkg_buf;
+
+	if (!aq_buf)
+		return false;
+
+	if (le16_to_cpu(desc->opcode) != ice_aqc_opc_update_pkg)
+		return false;
+
+	pkg_buf = (struct ice_buf_hdr *)aq_buf;
+	/* section count for udp tunnel command is always 2 */
+	if (le16_to_cpu(pkg_buf->section_count) != 2)
+		return false;
+
+	if (le32_to_cpu(pkg_buf->section_entry[0].type) ==
+	    ICE_SID_RXPARSER_BOOST_TCAM ||
+	    le32_to_cpu(pkg_buf->section_entry[0].type) ==
+	    ICE_SID_TXPARSER_BOOST_TCAM)
+		return true;
+
+	return false;
+}
+
+/**
+ * ice_is_vf_adq_enabled - Check if any VF has ADQ enabled
+ * @pf: pointer to the PF structure
+ * @vf_id: on true return, the first VF ID that we found had ADQ enabled
+ *
+ * Return true if any VF has ADQ enabled. Return false otherwise.
+ */
+static bool ice_is_vf_adq_enabled(struct ice_pf *pf, u16 *vf_id)
+{
+	bool adq_enabled = false;
+	struct ice_vf *vf;
+	unsigned int bkt;
+
+	rcu_read_lock();
+	ice_for_each_vf_rcu(pf, bkt, vf) {
+		if (vf->adq_enabled) {
+			*vf_id = vf->vf_id;
+			adq_enabled = true;
+			break;
+		}
+	}
+	rcu_read_unlock();
+
+	return adq_enabled;
+}
+
+/**
+ * ice_vf_chnl_fltrs_enabled - Check if a VF has TC filters enabled
+ * @pf: pointer to the PF structure
+ * @vf_id: on true return, the first VF ID that we found had TC filters
+ *
+ * Return true if any VF has TC filters. Return false otherwise.
+ */
+static bool ice_vf_chnl_fltrs_enabled(struct ice_pf *pf, u16 *vf_id)
+{
+	bool chnl_fltrs_enabled = false;
+	struct ice_vf *vf;
+	unsigned int bkt;
+
+	rcu_read_lock();
+	ice_for_each_vf_rcu(pf, bkt, vf) {
+		if (vf->num_dmac_chnl_fltrs) {
+			*vf_id = vf->vf_id;
+			chnl_fltrs_enabled = true;
+			break;
+		}
+	}
+	rcu_read_unlock();
+
+	return chnl_fltrs_enabled;
+}
+
 /**
  * ice_check_dcf_allowed - check if DCF is allowed based on various checks
  * @vf: pointer to the VF to check
@@ -116,12 +203,11 @@ bool ice_check_dcf_allowed(struct ice_vf *vf)
 		return false;
 	}
 #endif /* NETIF_F_HW_TC */
-	ice_for_each_vf(pf, i) {
-		if (pf->vf[i].adq_enabled) {
-			dev_err(dev, "ADQ on VF %d is currently enabled. Device Control Functionality cannot be enabled.\n",
-				pf->vf[i].vf_id);
-			return false;
-		}
+
+	if (ice_is_vf_adq_enabled(pf, &i)) {
+		dev_err(dev, "ADQ on VF %d is currently enabled. Device Control Functionality cannot be enabled.\n",
+			i);
+		return false;
 	}
 
 #ifdef HAVE_TC_SETUP_CLSFLOWER
@@ -130,20 +216,19 @@ bool ice_check_dcf_allowed(struct ice_vf *vf)
 		return false;
 	}
 #endif /* HAVE_TC_SETUP_CLSFLOWER */
-	ice_for_each_vf(pf, i) {
-		if (pf->vf[i].num_dmac_chnl_fltrs) {
-			dev_err(dev, "TC filters on VF %d are currently in use. Device Control Functionality cannot be enabled.\n",
-				pf->vf[i].vf_id);
-			return false;
-		}
+
+	if (ice_vf_chnl_fltrs_enabled(pf, &i)) {
+		dev_err(dev, "TC filters on VF %d are currently in use. Device Control Functionality cannot be enabled.\n",
+			i);
+		return false;
 	}
 
-#ifdef HAVE_NETDEV_SB_DEV
+#ifdef HAVE_NDO_DFWD_OPS
 	if (ice_is_offloaded_macvlan_ena(pf)) {
 		dev_err(dev, "L2 Forwarding Offload is currently enabled. Device Control Functionality cannot be enabled.\n");
 		return false;
 	}
-#endif /* HAVE_NETDEV_SB_DEV */
+#endif /* HAVE_NDO_DFWD_OPS */
 
 	sw = pf->hw.switch_info;
 	for (i = 0; i < ICE_MAX_NUM_RECIPES; i++) {
@@ -165,7 +250,7 @@ bool ice_is_dcf_enabled(struct ice_pf *pf)
 }
 
 /**
- * ice_vf_is_dcf - helper to check if the assigned VF is a DCF
+ * ice_is_vf_dcf - helper to check if the assigned VF is a DCF
  * @vf: the assigned VF to be checked
  */
 bool ice_is_vf_dcf(struct ice_vf *vf)
@@ -373,15 +458,15 @@ void ice_rm_dcf_sw_vsi_rule(struct ice_pf *pf, u16 hw_vsi_id)
 						       s_entry->vsi_list_info,
 						       hw_vsi_id);
 			if (ret && ret != -ENOENT)
-				dev_err(ice_pf_to_dev(pf),
-					"Failed to remove VSI %u from VSI list : %d\n",
-					hw_vsi_id, ret);
+				ice_dev_err_errno(ice_pf_to_dev(pf), ret,
+						  "Failed to remove VSI %u from VSI list",
+						  hw_vsi_id);
 		} else if (s_entry->fwd_id.hw_vsi_id == hw_vsi_id) {
 			ret = ice_dcf_rm_sw_rule_to_vsi(pf, s_entry);
 			if (ret)
-				dev_err(ice_pf_to_dev(pf),
-					"Failed to remove VSI %u switch rule : %d\n",
-					hw_vsi_id, ret);
+				ice_dev_err_errno(ice_pf_to_dev(pf), ret,
+						  "Failed to remove VSI %u switch rule",
+						  hw_vsi_id);
 		}
 }
 
@@ -413,16 +498,16 @@ void ice_rm_all_dcf_sw_rules(struct ice_pf *pf)
 			rule_id = sw_rule->rule_id;
 			ret = ice_dcf_rm_sw_rule_to_vsi_list(pf, sw_rule);
 			if (ret)
-				dev_err(ice_pf_to_dev(pf),
-					"Failed to remove switch rule 0x%04x with list id %u : %d\n",
-					rule_id, list_id, ret);
+				ice_dev_err_errno(ice_pf_to_dev(pf), ret,
+						  "Failed to remove switch rule 0x%04x with list id %u",
+						  rule_id, list_id);
 		} else {
 			rule_id = sw_rule->rule_id;
 			ret = ice_dcf_rm_sw_rule_to_vsi(pf, sw_rule);
 			if (ret)
-				dev_err(ice_pf_to_dev(pf),
-					"Failed to remove switch rule 0x%04x : %d\n",
-					rule_id, ret);
+				ice_dev_err_errno(ice_pf_to_dev(pf), ret,
+						  "Failed to remove switch rule 0x%04x",
+						  rule_id);
 		}
 
 	/* clears rule filter management data if AdminQ command has error */
@@ -441,18 +526,45 @@ void ice_rm_all_dcf_sw_rules(struct ice_pf *pf)
 }
 
 /**
- * ice_dis_dcf_acl_cap - disable DCF ACL capability for the PF
+ * ice_clear_dcf_acl_cfg - clear DCF ACL configuration for the PF
  * @pf: pointer to the PF info
  */
-void ice_dis_dcf_acl_cap(struct ice_pf *pf)
+void ice_clear_dcf_acl_cfg(struct ice_pf *pf)
 {
-	if (pf->hw.dcf_acl_enabled) {
-		pf->hw.dcf_acl_enabled = false;
+	if (pf->hw.dcf_caps & DCF_ACL_CAP) {
 		ice_acl_destroy_tbl(&pf->hw);
 		ice_init_acl(pf);
 	}
 }
 
+/**
+ * ice_dcf_is_acl_capable - check if DCF ACL capability enabled
+ * @hw: pointer to the hardware info
+ */
+bool ice_dcf_is_acl_capable(struct ice_hw *hw)
+{
+	return hw->dcf_caps & DCF_ACL_CAP;
+}
+
+/**
+ * ice_clear_dcf_udp_tunnel_cfg - clear DCF UDP tunnel configuration for the PF
+ * @pf: pointer to the PF info
+ */
+void ice_clear_dcf_udp_tunnel_cfg(struct ice_pf *pf)
+{
+	if (pf->hw.dcf_caps & DCF_UDP_TUNNEL_CAP)
+		ice_destroy_tunnel(&pf->hw, 0, true);
+}
+
+/**
+ * ice_dcf_is_udp_tunnel_capable - check if DCF UDP tunnel capability enabled
+ * @hw: pointer to the hardware info
+ */
+bool ice_dcf_is_udp_tunnel_capable(struct ice_hw *hw)
+{
+	return hw->dcf_caps & DCF_UDP_TUNNEL_CAP;
+}
+
 /**
  * ice_dcf_find_vsi_list_info - find the VSI list by ID.
  * @pf: pointer to the PF info
@@ -833,6 +945,64 @@ ice_dcf_handle_free_res_rsp(struct ice_pf *pf, u8 *aq_buf)
 	return status;
 }
 
+/**
+ * ice_dcf_handle_udp_tunnel_rsp - handle the update package response
+ * @pf: pointer to the PF info
+ * @aq_desc: descriptor describing the command
+ * @aq_buf: pointer to the package update command buffer
+ */
+static enum virtchnl_status_code
+ice_dcf_handle_udp_tunnel_rsp(struct ice_pf *pf, struct ice_aq_desc *aq_desc,
+			      u8 *aq_buf)
+{
+	struct ice_boost_tcam_section *sect;
+	struct ice_buf_hdr *pkg_buf;
+	struct ice_hw *hw = &pf->hw;
+	u16 port_key, inv_port_key;
+	u16 offset;
+	u16 addr;
+	u8 count;
+	u16 i, j;
+
+	mutex_lock(&hw->tnl_lock);
+	pkg_buf = (struct ice_buf_hdr *)aq_buf;
+	offset = le16_to_cpu(pkg_buf->section_entry[0].offset);
+	sect = (struct ice_boost_tcam_section *)(((u8 *)pkg_buf) + offset);
+	count = le16_to_cpu(sect->count);
+
+	for (i = 0; i < hw->tnl.count && i < ICE_TUNNEL_MAX_ENTRIES; i++)
+		for (j = 0; j < count; j++) {
+			addr = le16_to_cpu(sect->tcam[j].addr);
+			inv_port_key =
+			    le16_to_cpu(sect->tcam[j].key.key.hv_dst_port_key);
+			port_key =
+			    le16_to_cpu(sect->tcam[j].key.key2.hv_dst_port_key);
+			if (hw->tnl.tbl[i].valid &&
+			    hw->tnl.tbl[i].boost_addr == addr) {
+				/* It's tunnel destroy command if the key and
+				 * inverse key is the same.
+				 */
+				if (port_key == inv_port_key) {
+					hw->tnl.tbl[i].in_use = false;
+					hw->tnl.tbl[i].port = 0;
+					hw->tnl.tbl[i].ref = 0;
+				} else {
+					hw->tnl.tbl[i].port = port_key;
+					hw->tnl.tbl[i].in_use = true;
+					hw->tnl.tbl[i].ref = 1;
+				}
+			}
+		}
+
+	if (ice_is_tunnel_empty(&pf->hw))
+		hw->dcf_caps &= ~DCF_UDP_TUNNEL_CAP;
+	else
+		hw->dcf_caps |= DCF_UDP_TUNNEL_CAP;
+	mutex_unlock(&hw->tnl_lock);
+
+	return VIRTCHNL_STATUS_SUCCESS;
+}
+
 /**
  * ice_dcf_post_aq_send_cmd - get the data from firmware successful response
  * @pf: pointer to the PF info
@@ -849,16 +1019,28 @@ ice_dcf_post_aq_send_cmd(struct ice_pf *pf, struct ice_aq_desc *aq_desc,
 	if (!aq_buf)
 		return VIRTCHNL_STATUS_SUCCESS;
 
-	if (opc == ice_aqc_opc_add_sw_rules)
+	switch (opc) {
+	case ice_aqc_opc_add_sw_rules:
 		status = ice_dcf_handle_add_sw_rule_rsp(pf, aq_buf);
-	else if (opc == ice_aqc_opc_update_sw_rules)
+		break;
+	case ice_aqc_opc_update_sw_rules:
 		status = ice_dcf_handle_updt_sw_rule_rsp(pf, aq_buf);
-	else if (opc == ice_aqc_opc_remove_sw_rules)
+		break;
+	case ice_aqc_opc_remove_sw_rules:
 		status = ice_dcf_handle_rm_sw_rule_rsp(pf, aq_buf);
-	else if (opc == ice_aqc_opc_alloc_res)
+		break;
+	case ice_aqc_opc_alloc_res:
 		status = ice_dcf_handle_alloc_res_rsp(pf, aq_buf);
-	else if (opc == ice_aqc_opc_free_res)
+		break;
+	case ice_aqc_opc_free_res:
 		status = ice_dcf_handle_free_res_rsp(pf, aq_buf);
+		break;
+	case ice_aqc_opc_update_pkg:
+		if (ice_dcf_is_udp_tunnel_aq_cmd(aq_desc, aq_buf))
+			status = ice_dcf_handle_udp_tunnel_rsp(pf, aq_desc,
+							       aq_buf);
+		break;
+	}
 
 	return status;
 }
