@@ -1,5 +1,5 @@
 /* SPDX-License-Identifier: GPL-2.0 */
-/* Copyright (C) 2018-2019, Intel Corporation. */
+/* Copyright (C) 2018-2021, Intel Corporation. */
 
 #ifndef _ICE_TXRX_H_
 #define _ICE_TXRX_H_
@@ -123,6 +123,7 @@ static inline int ice_skb_pad(void)
 #define ICE_TX_FLAGS_IPV4	BIT(5)
 #define ICE_TX_FLAGS_IPV6	BIT(6)
 #define ICE_TX_FLAGS_TUNNEL	BIT(7)
+#define ICE_TX_FLAGS_HW_OUTER_SINGLE_VLAN	BIT(8)
 #define ICE_TX_FLAGS_VLAN_M	0xffff0000
 #define ICE_TX_FLAGS_VLAN_PR_M	0xe0000000
 #define ICE_TX_FLAGS_VLAN_PR_S	29
@@ -153,7 +154,6 @@ struct ice_tx_buf {
 	u32 tx_flags;
 	DEFINE_DMA_UNMAP_LEN(len);
 	DEFINE_DMA_UNMAP_ADDR(dma);
-	int ptp_ts_idx;
 };
 
 struct ice_tx_offload_params {
@@ -178,8 +178,13 @@ struct ice_rx_buf {
 		};
 #ifdef HAVE_AF_XDP_ZC_SUPPORT
 		struct {
-			void *addr;
+			union {
+				struct xdp_buff *xdp;
+				void *addr;
+			};
+#ifndef HAVE_MEM_TYPE_XSK_BUFF_POOL
 			u64 handle;
+#endif
 		};
 #endif /* HAVE_AF_XDP_ZC_SUPPORT */
 	};
@@ -188,6 +193,7 @@ struct ice_rx_buf {
 #ifdef ADQ_PERF_COUNTERS
 struct ice_ch_tx_q_stats {
 	u64 num_atr_setup; /* How many times, ATR is setup */
+	u64 num_mark_atr_setup; /* How many times, ATR is setup via skb->mark */
 	u64 num_atr_evict; /* How many times, ATR is teardown */
 	u64 num_atr_bailouts; /* How many times, bailout from ATR setup */
 };
@@ -237,7 +243,6 @@ struct ice_rxq_stats {
 	u64 non_eop_descs;
 	u64 alloc_page_failed;
 	u64 alloc_buf_failed;
-	u64 gro_dropped; /* GRO returned dropped */
 #ifdef ICE_ADD_PROBES
 	u64 page_reuse;
 #endif /* ICE_ADD_PROBES */
@@ -272,23 +277,20 @@ enum ice_rx_dtype {
 #define ICE_TX_ITR	ICE_IDX_ITR1
 #define ICE_ITR_8K	124
 #define ICE_ITR_20K	50
-#define ICE_ITR_MAX	8160
-#define ICE_DFLT_TX_ITR	(ICE_ITR_20K | ICE_ITR_DYNAMIC)
-#define ICE_DFLT_RX_ITR	(ICE_ITR_20K | ICE_ITR_DYNAMIC)
-#define ICE_ITR_DYNAMIC	0x8000  /* used as flag for itr_setting */
-#define ITR_IS_DYNAMIC(setting) (!!((setting) & ICE_ITR_DYNAMIC))
-#define ITR_TO_REG(setting)	((setting) & ~ICE_ITR_DYNAMIC)
+#define ICE_ITR_MAX	8160 /* 0x1FE0 */
+#define ICE_DFLT_TX_ITR	ICE_ITR_20K
+#define ICE_DFLT_RX_ITR	ICE_ITR_20K
+enum ice_dynamic_itr {
+	ITR_STATIC = 0,
+	ITR_DYNAMIC = 1
+};
+
+#define ITR_IS_DYNAMIC(rc) ((rc)->itr_mode == ITR_DYNAMIC)
 #define ICE_ITR_GRAN_S		1	/* ITR granularity is always 2us */
 #define ICE_ITR_GRAN_US		BIT(ICE_ITR_GRAN_S)
 #define ICE_ITR_MASK		0x1FFE	/* ITR register value alignment mask */
 #define ITR_REG_ALIGN(setting)	((setting) & ICE_ITR_MASK)
 
-#define ICE_ITR_ADAPTIVE_MIN_INC	0x0002
-#define ICE_ITR_ADAPTIVE_MIN_USECS	0x0002
-#define ICE_ITR_ADAPTIVE_MAX_USECS	0x00FA
-#define ICE_ITR_ADAPTIVE_LATENCY	0x8000
-#define ICE_ITR_ADAPTIVE_BULK		0x0000
-
 #define ICE_DFLT_INTRL	0
 #define ICE_MAX_INTRL	236
 
@@ -308,9 +310,24 @@ enum ice_rx_dtype {
 #define ICE_TX_ADVANCED	0
 #define ICE_TX_LEGACY	1
 
+#ifdef HAVE_XDP_SUPPORT
+#ifdef ICE_ADD_PROBES
+struct ice_xdp_stats {
+	u64 xdp_rx_pkts;
+	u64 xdp_rx_bytes;
+	u64 xdp_pass;
+	u64 xdp_drop;
+	u64 xdp_tx;
+	u64 xdp_tx_fail;
+	u64 xdp_unknown;
+	u64 xdp_redirect;
+	u64 xdp_redirect_fail;
+};
+#endif
+#endif
+
 /* descriptor ring, associated with a VSI */
 struct ice_ring {
-	/* CL1 - 1st cacheline starts here */
 	struct ice_ring *next;		/* pointer to next ring in q_vector */
 	void *desc;			/* Descriptor ring memory */
 	struct device *dev;		/* Used for DMA mapping */
@@ -322,10 +339,18 @@ struct ice_ring {
 		struct ice_tx_buf *tx_buf;
 		struct ice_rx_buf *rx_buf;
 	};
-	/* CL2 - 2nd cacheline starts here */
+	/* --- cacheline 1 boundary (64 bytes) --- */
 	u16 q_index;			/* Queue number of ring */
 	u16 q_handle;			/* Queue handle per TC */
 
+#ifdef HAVE_XDP_SUPPORT
+#define ICE_TX_FLAGS_RING_XDP			BIT(0)
+#endif /* HAVE_XDP_SUPPORT */
+#define ICE_RX_FLAGS_RING_BUILD_SKB		BIT(1)
+#define ICE_TXRX_FLAGS_VLAN_TAG_LOC_L2TAG1	BIT(2)
+#define ICE_TX_FLAGS_VLAN_TAG_LOC_L2TAG2	BIT(3)
+	u8 flags;
+
 	u16 count;			/* Number of descriptors */
 	u16 reg_idx;			/* HW register index of the ring */
 
@@ -350,37 +375,48 @@ struct ice_ring {
 		struct ice_rxq_stats rx_stats;
 	};
 
+	/* --- cacheline 2 boundary (128 bytes) was 8 bytes ago --- */
 	struct rcu_head rcu;		/* to avoid race on free */
 	DECLARE_BITMAP(xps_state, ICE_TX_NBITS);	/* XPS Config State */
 	struct ice_channel *ch;
 #ifdef HAVE_XDP_SUPPORT
 	struct bpf_prog *xdp_prog;
+#ifdef ICE_ADD_PROBES
+	struct ice_xdp_stats xdp_stats;
+#endif
 #ifdef HAVE_AF_XDP_ZC_SUPPORT
-	struct xdp_umem *xsk_umem;
+#ifdef HAVE_NETDEV_BPF_XSK_POOL
+	struct xsk_buff_pool *xsk_pool;
+#else
+	struct xdp_umem *xsk_pool;
+#endif /* HAVE_NETDEV_BPF_XSK_POOL */
+#ifndef HAVE_MEM_TYPE_XSK_BUFF_POOL
 	struct zero_copy_allocator zca;
+#endif
 #endif /* HAVE_AF_XDP_ZC_SUPPORT */
-	/* CL3 - 3rd cacheline starts here */
 #ifdef HAVE_XDP_BUFF_RXQ
+	/* --- cacheline 3 boundary (192 bytes) --- */
 	struct xdp_rxq_info xdp_rxq;
 #endif /* HAVE_XDP_BUFF_RXQ */
 #endif /* HAVE_XDP_SUPPORT */
 
-	/* CLX - the below items are only accessed infrequently and should be
-	 * in their own cache line if possible
-	 */
-#ifdef HAVE_XDP_SUPPORT
-#define ICE_TX_FLAGS_RING_XDP		BIT(0)
-#endif /* HAVE_XDP_SUPPORT */
-#define ICE_RX_FLAGS_RING_BUILD_SKB	BIT(1)
-	u8 flags;
+	/* --- cacheline 4 boundary (256 bytes) --- */
 	dma_addr_t dma;			/* physical address of ring */
 	unsigned int size;		/* length of descriptor ring in bytes */
 	u32 txq_teid;			/* Added Tx queue TEID */
-	u16 rx_buf_len;
-	u8 dcb_tc;			/* Traffic class of ring */
-	u64 cached_systime;
-	u8 ptp_rx:1;
 	u32 ch_inline_fd_cnt_index;
+	u8 rx_crc_strip_dis;
+	u8 dcb_tc;			/* Traffic class of ring */
+	u16 rx_buf_len;
+	u16 quanta_prof_id;
+	struct ice_ptp_tx *tx_tstamps;
+	u64 cached_phctime;
+	u8 ptp_rx:1;
+	u8 ptp_tx:1;
+
+	/* cacheline - the below items are only accessed infrequently and
+	 * should be in their own cache line if possible
+	 */
 #ifdef ADQ_PERF_COUNTERS
 	struct ice_ch_q_stats ch_q_stats;
 #endif /* ADQ_PERF_COUNTERS */
@@ -430,17 +466,19 @@ static inline bool ice_ring_is_xdp(struct ice_ring *ring)
 struct ice_ring_container {
 	/* head of linked-list of rings */
 	struct ice_ring *ring;
-	unsigned long next_update;	/* jiffies value of next queue update */
-	unsigned int total_bytes;	/* total bytes processed this int */
-	unsigned int total_pkts;	/* total packets processed this int */
+	struct dim dim;		/* data for net_dim algorithm */
 	u16 itr_idx;		/* index in the interrupt vector */
-	u16 target_itr;		/* value in usecs divided by the hw->itr_gran */
-	u16 current_itr;	/* value in usecs divided by the hw->itr_gran */
-	/* high bit set means dynamic ITR, rest is used to store user
-	 * readable ITR value in usecs and must be converted before programming
-	 * to a register.
+	/* this matches the maximum number of ITR bits, but in usec
+	 * values, so it is shifted left one bit (bit zero is ignored)
 	 */
-	u16 itr_setting;
+	union {
+		struct {
+			u16 itr_setting:13;
+			u16 itr_reserved:2;
+			u16 itr_mode:1;
+		};
+		u16 itr_settings;
+	};
 };
 
 struct ice_coalesce_stored {
@@ -466,16 +504,38 @@ static inline unsigned int ice_rx_pg_order(struct ice_ring *ring)
 
 #define ice_rx_pg_size(_ring) (PAGE_SIZE << ice_rx_pg_order(_ring))
 
-
 union ice_32b_rx_flex_desc;
 
 bool ice_alloc_rx_bufs(struct ice_ring *rxr, u16 cleaned_count);
 netdev_tx_t ice_start_xmit(struct sk_buff *skb, struct net_device *netdev);
+#ifndef HAVE_NDO_SELECT_QUEUE_SB_DEV
+#if defined(HAVE_NDO_SELECT_QUEUE_ACCEL) || defined(HAVE_NDO_SELECT_QUEUE_ACCEL_FALLBACK)
+#ifndef HAVE_NDO_SELECT_QUEUE_FALLBACK_REMOVED
+u16 ice_select_queue(struct net_device *dev, struct sk_buff *skb,
+		     void *accel_priv, select_queue_fallback_t fallback);
+#else /* HAVE_NDO_SELECT_QUEUE_FALLBACK_REMOVED */
+u16 ice_select_queue(struct net_device *dev, struct sk_buff *skb,
+		     void *accel_priv);
+#endif /* HAVE_NDO_SELECT_QUEUE_FALLBACK_REMOVED */
+#else /* HAVE_NDO_SELECT_QUEUE_ACCEL || HAVE_NDO_SELECT_QUEUE_ACCEL_FALLBACK */
+u16 ice_select_queue(struct net_device *dev, struct sk_buff *skb);
+#endif /* HAVE_NDO_SELECT_QUEUE_ACCEL || HAVE_NDO_SELECT_QUEUE_ACCEL_FALLBACK */
+#else /* HAVE_NDO_SELECT_QUEUE_SB_DEV */
+#ifdef HAVE_NDO_SELECT_QUEUE_FALLBACK_REMOVED
+u16 ice_select_queue(struct net_device *dev, struct sk_buff *skb,
+		     struct net_device *sb_dev);
+#else /* HAVE_NDO_SELECT_QUEUE_FALLBACK_REMOVED */
+u16 ice_select_queue(struct net_device *dev, struct sk_buff *skb,
+		     struct net_device *sb_dev,
+		     select_queue_fallback_t fallback);
+#endif /* HAVE_NDO_SELECT_QUEUE_FALLBACK_REMOVED */
+#endif /* HAVE_NDO_SELECT_QUEUE_SB_DEV */
+int ice_setup_tstamp_ring(struct ice_ring *tstamp_ring);
+void ice_free_tx_ring(struct ice_ring *tx_ring);
 void ice_clean_tx_ring(struct ice_ring *tx_ring);
 void ice_clean_rx_ring(struct ice_ring *rx_ring);
 int ice_setup_tx_ring(struct ice_ring *tx_ring);
 int ice_setup_rx_ring(struct ice_ring *rx_ring);
-void ice_free_tx_ring(struct ice_ring *tx_ring);
 void ice_free_rx_ring(struct ice_ring *rx_ring);
 int ice_napi_poll(struct napi_struct *napi, int budget);
 int
