@@ -1,17 +1,19 @@
 // SPDX-License-Identifier: GPL-2.0
-/* Copyright (C) 2018-2019, Intel Corporation. */
+/* Copyright (C) 2018-2021, Intel Corporation. */
 
 #include <linux/bpf_trace.h>
 #include <net/xdp_sock.h>
 #include <net/xdp.h>
+#include <net/busy_poll.h>
 #include "ice.h"
+#include "ice_lib.h"
 #include "ice_base.h"
 #include "ice_type.h"
 #include "ice_xsk.h"
 #include "ice_txrx.h"
 #include "ice_txrx_lib.h"
+#include "ice_irq.h"
 #ifdef HAVE_AF_XDP_ZC_SUPPORT
-#include "ice_lib.h"
 
 /**
  * ice_qp_reset_stats - Resets all stats for rings of given index
@@ -39,6 +41,7 @@ static void ice_qp_clean_rings(struct ice_vsi *vsi, u16 q_idx)
 	ice_clean_tx_ring(vsi->tx_rings[q_idx]);
 	if (ice_is_xdp_ena_vsi(vsi))
 		ice_clean_tx_ring(vsi->xdp_rings[q_idx]);
+
 	ice_clean_rx_ring(vsi->rx_rings[q_idx]);
 }
 
@@ -89,8 +92,9 @@ ice_qvec_dis_irq(struct ice_vsi *vsi, struct ice_ring *rx_ring,
 		u16 v_idx = q_vector->v_idx;
 
 		wr32(hw, GLINT_DYN_CTL(q_vector->reg_idx), 0);
+
 		ice_flush(hw);
-		synchronize_irq(pf->msix_entries[v_idx + base].vector);
+		synchronize_irq(ice_get_irq_num(pf, v_idx + base));
 	}
 }
 
@@ -109,9 +113,6 @@ ice_qvec_cfg_msix(struct ice_vsi *vsi, struct ice_q_vector *q_vector)
 
 	ice_cfg_itr(hw, q_vector);
 
-	wr32(hw, GLINT_RATE(reg_idx),
-	     ice_intrl_usec_to_reg(q_vector->intrl, hw->intrl_gran));
-
 	ice_for_each_ring(ring, q_vector->tx)
 		ice_cfg_txq_interrupt(vsi, ring->reg_idx, reg_idx,
 				      q_vector->tx.itr_idx);
@@ -160,7 +161,7 @@ static int ice_qp_dis(struct ice_vsi *vsi, u16 q_idx)
 	rx_ring = vsi->rx_rings[q_idx];
 	q_vector = rx_ring->q_vector;
 
-	while (test_and_set_bit(__ICE_CFG_BUSY, vsi->state)) {
+	while (test_and_set_bit(ICE_CFG_BUSY, vsi->state)) {
 		timeout--;
 		if (!timeout)
 			return -EBUSY;
@@ -207,17 +208,18 @@ static int ice_qp_ena(struct ice_vsi *vsi, u16 q_idx)
 	struct ice_aqc_add_tx_qgrp *qg_buf;
 	struct ice_ring *tx_ring, *rx_ring;
 	struct ice_q_vector *q_vector;
+	u16 size;
 	int err;
 
 	if (q_idx >= vsi->num_rxq || q_idx >= vsi->num_txq)
 		return -EINVAL;
 
-	qg_buf = kzalloc(sizeof(*qg_buf), GFP_KERNEL);
+	size = struct_size(qg_buf, txqs, 1);
+	qg_buf = kzalloc(size, GFP_KERNEL);
 	if (!qg_buf)
 		return -ENOMEM;
 
 	qg_buf->num_txqs = 1;
-
 	tx_ring = vsi->tx_rings[q_idx];
 	rx_ring = vsi->rx_rings[q_idx];
 	q_vector = rx_ring->q_vector;
@@ -229,16 +231,16 @@ static int ice_qp_ena(struct ice_vsi *vsi, u16 q_idx)
 	if (ice_is_xdp_ena_vsi(vsi)) {
 		struct ice_ring *xdp_ring = vsi->xdp_rings[q_idx];
 
-		memset(qg_buf, 0, sizeof(*qg_buf));
+		memset(qg_buf, 0, size);
 		qg_buf->num_txqs = 1;
 		err = ice_vsi_cfg_txq(vsi, xdp_ring, qg_buf);
 		if (err)
 			goto free_buf;
 		ice_set_ring_xdp(xdp_ring);
-		xdp_ring->xsk_umem = ice_xsk_umem(xdp_ring);
+		xdp_ring->xsk_pool = ice_xsk_umem(xdp_ring);
 	}
 
-	err = ice_setup_rx_ctx(rx_ring);
+	err = ice_vsi_cfg_rxq(rx_ring);
 	if (err)
 		goto free_buf;
 
@@ -248,7 +250,7 @@ static int ice_qp_ena(struct ice_vsi *vsi, u16 q_idx)
 	if (err)
 		goto free_buf;
 
-	clear_bit(__ICE_CFG_BUSY, vsi->state);
+	clear_bit(ICE_CFG_BUSY, vsi->state);
 	ice_qvec_toggle_napi(vsi, q_vector, true);
 	ice_qvec_ena_irq(vsi, q_vector);
 
@@ -258,6 +260,7 @@ free_buf:
 	return err;
 }
 
+#ifndef HAVE_AF_XDP_NETDEV_UMEM
 /**
  * ice_xsk_alloc_umems - allocate a UMEM region for an XDP socket
  * @vsi: VSI to allocate the UMEM on
@@ -268,9 +271,13 @@ static int ice_xsk_alloc_umems(struct ice_vsi *vsi)
 {
 	if (vsi->xsk_umems)
 		return 0;
-
+#ifdef HAVE_NETDEV_BPF_XSK_POOL
 	vsi->xsk_umems = kcalloc(vsi->num_xsk_umems, sizeof(*vsi->xsk_umems),
 				 GFP_KERNEL);
+#else
+	vsi->xsk_umems = kcalloc(vsi->num_xsk_umems, sizeof(*vsi->xsk_umems),
+				 GFP_KERNEL);
+#endif /* HAVE_NETDEV_BPF_XSK_POOL */
 
 	if (!vsi->xsk_umems) {
 		vsi->num_xsk_umems = 0;
@@ -280,28 +287,6 @@ static int ice_xsk_alloc_umems(struct ice_vsi *vsi)
 	return 0;
 }
 
-/**
- * ice_xsk_add_umem - add a UMEM region for XDP sockets
- * @vsi: VSI to which the UMEM will be added
- * @umem: pointer to a requested UMEM region
- * @qid: queue ID
- *
- * Returns 0 on success, negative on error
- */
-static int ice_xsk_add_umem(struct ice_vsi *vsi, struct xdp_umem *umem, u16 qid)
-{
-	int err;
-
-	err = ice_xsk_alloc_umems(vsi);
-	if (err)
-		return err;
-
-	vsi->xsk_umems[qid] = umem;
-	vsi->num_xsk_umems_used++;
-
-	return 0;
-}
-
 /**
  * ice_xsk_remove_umem - Remove an UMEM for a certain ring/qid
  * @vsi: VSI from which the VSI will be removed
@@ -318,7 +303,9 @@ static void ice_xsk_remove_umem(struct ice_vsi *vsi, u16 qid)
 		vsi->num_xsk_umems = 0;
 	}
 }
+#endif /* !HAVE_AF_XDP_NETDEV_UMEM */
 
+#ifndef HAVE_MEM_TYPE_XSK_BUFF_POOL
 /**
  * ice_xsk_umem_dma_map - DMA map UMEM region for XDP sockets
  * @vsi: VSI to map the UMEM region
@@ -339,7 +326,7 @@ static int ice_xsk_umem_dma_map(struct ice_vsi *vsi, struct xdp_umem *umem)
 						    DMA_BIDIRECTIONAL,
 						    ICE_RX_DMA_ATTR);
 		if (dma_mapping_error(dev, dma)) {
-			dev_dbg(dev, "XSK UMEM DMA mapping error on page num %d",
+			dev_dbg(dev, "XSK UMEM DMA mapping error on page num %d/n",
 				i);
 			goto out_unmap;
 		}
@@ -378,6 +365,7 @@ static void ice_xsk_umem_dma_unmap(struct ice_vsi *vsi, struct xdp_umem *umem)
 		umem->pages[i].dma = 0;
 	}
 }
+#endif
 
 /**
  * ice_xsk_umem_disable - disable a UMEM region
@@ -388,12 +376,35 @@ static void ice_xsk_umem_dma_unmap(struct ice_vsi *vsi, struct xdp_umem *umem)
  */
 static int ice_xsk_umem_disable(struct ice_vsi *vsi, u16 qid)
 {
-	if (!vsi->xsk_umems || qid >= vsi->num_xsk_umems ||
-	    !vsi->xsk_umems[qid])
+#ifdef HAVE_AF_XDP_NETDEV_UMEM
+#ifdef HAVE_NETDEV_BPF_XSK_POOL
+	struct xsk_buff_pool *umem = xsk_get_pool_from_qid(vsi->netdev, qid);
+#else
+	struct xdp_umem *umem = xsk_get_pool_from_qid(vsi->netdev, qid);
+#endif
+#else
+	struct xdp_umem *umem;
+
+	if (!vsi->xsk_umems || qid >= vsi->num_xsk_umems)
 		return -EINVAL;
 
-	ice_xsk_umem_dma_unmap(vsi, vsi->xsk_umems[qid]);
+	umem = vsi->xsk_umems[qid];
+#endif
+
+	if (!umem)
+		return -EINVAL;
+
+	clear_bit(qid, vsi->af_xdp_zc_qps);
+
+#ifndef HAVE_MEM_TYPE_XSK_BUFF_POOL
+	ice_xsk_umem_dma_unmap(vsi, umem);
+#else
+	xsk_pool_dma_unmap(umem, ICE_RX_DMA_ATTR);
+#endif
+
+#ifndef HAVE_AF_XDP_NETDEV_UMEM
 	ice_xsk_remove_umem(vsi, qid);
+#endif
 
 	return 0;
 }
@@ -407,22 +418,42 @@ static int ice_xsk_umem_disable(struct ice_vsi *vsi, u16 qid)
  * Returns 0 on success, negative on failure
  */
 static int
+#ifdef HAVE_NETDEV_BPF_XSK_POOL
+ice_xsk_umem_enable(struct ice_vsi *vsi, struct xsk_buff_pool *umem, u16 qid)
+#else
 ice_xsk_umem_enable(struct ice_vsi *vsi, struct xdp_umem *umem, u16 qid)
+#endif /* HAVE_NETDEV_BPF_XSK_POOL */
 {
+#ifndef HAVE_MEM_TYPE_XSK_BUFF_POOL
 	struct xdp_umem_fq_reuse *reuseq;
+#endif
 	int err;
 
 	if (vsi->type != ICE_VSI_PF)
 		return -EINVAL;
 
+#ifndef HAVE_AF_XDP_NETDEV_UMEM
 	if (!vsi->num_xsk_umems)
 		vsi->num_xsk_umems = min_t(u16, vsi->num_rxq, vsi->num_txq);
 	if (qid >= vsi->num_xsk_umems)
 		return -EINVAL;
 
+	err = ice_xsk_alloc_umems(vsi);
+	if (err)
+		return err;
+
 	if (vsi->xsk_umems && vsi->xsk_umems[qid])
 		return -EBUSY;
 
+	vsi->xsk_umems[qid] = umem;
+	vsi->num_xsk_umems_used++;
+#else
+	if (qid >= vsi->netdev->real_num_rx_queues ||
+	    qid >= vsi->netdev->real_num_tx_queues)
+		return -EINVAL;
+#endif /* !HAVE_AF_XDP_NETDEV_UMEM */
+
+#ifndef HAVE_MEM_TYPE_XSK_BUFF_POOL
 	reuseq = xsk_reuseq_prepare(vsi->rx_rings[0]->count);
 	if (!reuseq)
 		return -ENOMEM;
@@ -430,12 +461,15 @@ ice_xsk_umem_enable(struct ice_vsi *vsi, struct xdp_umem *umem, u16 qid)
 	xsk_reuseq_free(xsk_reuseq_swap(umem, reuseq));
 
 	err = ice_xsk_umem_dma_map(vsi, umem);
+#else
+	err = xsk_pool_dma_map(umem, ice_pf_to_dev(vsi->back),
+			       ICE_RX_DMA_ATTR);
+#endif
+
 	if (err)
 		return err;
 
-	err = ice_xsk_add_umem(vsi, umem, qid);
-	if (err)
-		return err;
+	set_bit(qid, vsi->af_xdp_zc_qps);
 
 	return 0;
 }
@@ -448,7 +482,11 @@ ice_xsk_umem_enable(struct ice_vsi *vsi, struct xdp_umem *umem, u16 qid)
  *
  * Returns 0 on success, negative on failure
  */
+#ifdef HAVE_NETDEV_BPF_XSK_POOL
+int ice_xsk_umem_setup(struct ice_vsi *vsi, struct xsk_buff_pool *umem, u16 qid)
+#else
 int ice_xsk_umem_setup(struct ice_vsi *vsi, struct xdp_umem *umem, u16 qid)
+#endif
 {
 	bool if_running, umem_present = !!umem;
 	int ret = 0, umem_failure = 0;
@@ -495,6 +533,7 @@ xsk_umem_if_up:
  */
 int ice_xsk_umem_query(struct ice_vsi *vsi, struct xdp_umem **umem, u16 qid)
 {
+#ifndef HAVE_AF_XDP_NETDEV_UMEM
 	if (vsi->type != ICE_VSI_PF)
 		return -EINVAL;
 
@@ -509,10 +548,25 @@ int ice_xsk_umem_query(struct ice_vsi *vsi, struct xdp_umem **umem, u16 qid)
 	}
 
 	*umem = NULL;
+#else
+	struct net_device *netdev = vsi->netdev;
+	struct xdp_umem *queried_umem;
+
+	if (vsi->type != ICE_VSI_PF)
+		return -EINVAL;
+
+	queried_umem = xsk_get_pool_from_qid(netdev, qid);
+	if (!queried_umem)
+		return -EINVAL;
+
+	*umem = queried_umem;
+#endif /* !HAVE_AF_XDP_NETDEV_UMEM */
+
 	return 0;
 }
 #endif /* NO_XDP_QUERY_XSK_UMEM */
 
+#ifndef HAVE_MEM_TYPE_XSK_BUFF_POOL
 /**
  * ice_zca_free - Callback for MEM_TYPE_ZERO_COPY allocations
  * @zca: zero-cpoy allocator
@@ -527,7 +581,7 @@ void ice_zca_free(struct zero_copy_allocator *zca, unsigned long handle)
 	u16 nta;
 
 	rx_ring = container_of(zca, struct ice_ring, zca);
-	umem = rx_ring->xsk_umem;
+	umem = rx_ring->xsk_pool;
 	hr = umem->headroom + XDP_PACKET_HEADROOM;
 
 #ifndef HAVE_XDP_UMEM_PROPS
@@ -566,12 +620,14 @@ void ice_zca_free(struct zero_copy_allocator *zca, unsigned long handle)
 static __always_inline bool
 ice_alloc_buf_fast_zc(struct ice_ring *rx_ring, struct ice_rx_buf *rx_buf)
 {
-	struct xdp_umem *umem = rx_ring->xsk_umem;
+	struct xdp_umem *umem = rx_ring->xsk_pool;
 	void *addr = rx_buf->addr;
 	u64 handle, hr;
 
 	if (addr) {
-		rx_ring->rx_stats.page_reuse_count++;
+#ifdef ICE_ADD_PROBES
+		rx_ring->rx_stats.page_reuse++;
+#endif /* ICE_ADD_PROBES */
 		return true;
 	}
 
@@ -607,7 +663,7 @@ ice_alloc_buf_fast_zc(struct ice_ring *rx_ring, struct ice_rx_buf *rx_buf)
 static __always_inline bool
 ice_alloc_buf_slow_zc(struct ice_ring *rx_ring, struct ice_rx_buf *rx_buf)
 {
-	struct xdp_umem *umem = rx_ring->xsk_umem;
+	struct xdp_umem *umem = rx_ring->xsk_pool;
 	u64 handle, headroom;
 
 	if (!xsk_umem_peek_addr_rq(umem, &handle)) {
@@ -629,12 +685,13 @@ ice_alloc_buf_slow_zc(struct ice_ring *rx_ring, struct ice_rx_buf *rx_buf)
 	xsk_umem_release_addr_rq(umem);
 	return true;
 }
+#endif /* !HAVE_MEM_TYPE_XSK_BUFF_POOL */
 
-/*
+#ifdef HAVE_MEM_TYPE_XSK_BUFF_POOL
+/**
  * ice_alloc_rx_bufs_zc - allocate a number of Rx buffers
  * @rx_ring: Rx ring
  * @count: The number of buffers to allocate
- * @alloc: the function pointer to call for allocation
  *
  * This function allocates a number of Rx buffers from the fill ring
  * or the internal recycle mechanism and places them on the Rx ring.
@@ -643,14 +700,20 @@ ice_alloc_buf_slow_zc(struct ice_ring *rx_ring, struct ice_rx_buf *rx_buf)
  * NOTE: this function header description doesn't do kdoc style
  *       because of the function pointer creating problems.
  */
+bool ice_alloc_rx_bufs_zc(struct ice_ring *rx_ring, int count)
+#else
 static bool
 ice_alloc_rx_bufs_zc(struct ice_ring *rx_ring, int count,
 		     bool (*alloc)(struct ice_ring *, struct ice_rx_buf *))
+#endif
 {
 	union ice_32b_rx_flex_desc *rx_desc;
 	u16 ntu = rx_ring->next_to_use;
 	struct ice_rx_buf *rx_buf;
 	bool ret = false;
+#ifdef HAVE_MEM_TYPE_XSK_BUFF_POOL
+	dma_addr_t dma;
+#endif
 
 	if (!count)
 		return false;
@@ -659,6 +722,7 @@ ice_alloc_rx_bufs_zc(struct ice_ring *rx_ring, int count,
 	rx_buf = &rx_ring->rx_buf[ntu];
 
 	do {
+#ifndef HAVE_MEM_TYPE_XSK_BUFF_POOL
 		if (!alloc(rx_ring, rx_buf)) {
 			ret = true;
 			break;
@@ -669,6 +733,16 @@ ice_alloc_rx_bufs_zc(struct ice_ring *rx_ring, int count,
 						 DMA_BIDIRECTIONAL);
 
 		rx_desc->read.pkt_addr = cpu_to_le64(rx_buf->dma);
+#else
+		rx_buf->xdp = xsk_buff_alloc(rx_ring->xsk_pool);
+		if (!rx_buf->xdp) {
+			ret = true;
+			break;
+		}
+
+		dma = xsk_buff_xdp_get_dma(rx_buf->xdp);
+		rx_desc->read.pkt_addr = cpu_to_le64(dma);
+#endif
 		rx_desc->wb.status_error0 = 0;
 
 		rx_desc++;
@@ -688,6 +762,7 @@ ice_alloc_rx_bufs_zc(struct ice_ring *rx_ring, int count,
 	return ret;
 }
 
+#ifndef HAVE_MEM_TYPE_XSK_BUFF_POOL
 /**
  * ice_alloc_rx_bufs_fast_zc - allocate zero copy bufs in the hot path
  * @rx_ring: Rx ring
@@ -713,6 +788,7 @@ bool ice_alloc_rx_bufs_slow_zc(struct ice_ring *rx_ring, u16 count)
 	return ice_alloc_rx_bufs_zc(rx_ring, count,
 				    ice_alloc_buf_slow_zc);
 }
+#endif
 
 /**
  * ice_bump_ntc - Bump the next_to_clean counter of an Rx ring
@@ -727,6 +803,7 @@ static void ice_bump_ntc(struct ice_ring *rx_ring)
 	prefetch(ICE_RX_DESC(rx_ring, ntc));
 }
 
+#ifndef HAVE_MEM_TYPE_XSK_BUFF_POOL
 /**
  * ice_get_rx_buf_zc - Fetch the current Rx buffer
  * @rx_ring: Rx ring
@@ -761,11 +838,11 @@ static void
 ice_reuse_rx_buf_zc(struct ice_ring *rx_ring, struct ice_rx_buf *old_buf)
 {
 #ifndef HAVE_XDP_UMEM_PROPS
-	unsigned long mask = (unsigned long)rx_ring->xsk_umem->chunk_mask;
+	unsigned long mask = (unsigned long)rx_ring->xsk_pool->chunk_mask;
 #else
-	unsigned long mask = (unsigned long)rx_ring->xsk_umem->props.chunk_mask;
+	unsigned long mask = (unsigned long)rx_ring->xsk_pool->props.chunk_mask;
 #endif /* NO_XDP_UMEM_PROPS */
-	u64 hr = rx_ring->xsk_umem->headroom + XDP_PACKET_HEADROOM;
+	u64 hr = rx_ring->xsk_pool->headroom + XDP_PACKET_HEADROOM;
 	u16 nta = rx_ring->next_to_alloc;
 	struct ice_rx_buf *new_buf;
 
@@ -779,16 +856,17 @@ ice_reuse_rx_buf_zc(struct ice_ring *rx_ring, struct ice_rx_buf *old_buf)
 	new_buf->addr += hr;
 
 	new_buf->handle = old_buf->handle & mask;
-	new_buf->handle += rx_ring->xsk_umem->headroom;
+	new_buf->handle += rx_ring->xsk_pool->headroom;
 
 	old_buf->addr = NULL;
 }
+#endif
 
 /**
  * ice_construct_skb_zc - Create an sk_buff from zero-copy buffer
  * @rx_ring: Rx ring
  * @rx_buf: zero-copy Rx buffer
- * @xdp: XDP buffer
+ * @xdp: xdp buffer
  *
  * This function allocates a new skb from a zero-copy Rx buffer.
  *
@@ -811,10 +889,16 @@ ice_construct_skb_zc(struct ice_ring *rx_ring, struct ice_rx_buf *rx_buf,
 
 	skb_reserve(skb, xdp->data - xdp->data_hard_start);
 	memcpy(__skb_put(skb, datasize), xdp->data, datasize);
+
 	if (metasize)
 		skb_metadata_set(skb, metasize);
 
+#ifndef HAVE_MEM_TYPE_XSK_BUFF_POOL
 	ice_reuse_rx_buf_zc(rx_ring, rx_buf);
+#else
+	xsk_buff_free(rx_buf->xdp);
+	rx_buf->xdp = NULL;
+#endif
 
 	return skb;
 }
@@ -842,7 +926,9 @@ ice_run_xdp_zc(struct ice_ring *rx_ring, struct xdp_buff *xdp)
 	}
 
 	act = bpf_prog_run_xdp(xdp_prog, xdp);
+#ifndef HAVE_MEM_TYPE_XSK_BUFF_POOL
 	xdp->handle += xdp->data - xdp->data_hard_start;
+#endif
 	switch (act) {
 	case XDP_PASS:
 		break;
@@ -855,11 +941,11 @@ ice_run_xdp_zc(struct ice_ring *rx_ring, struct xdp_buff *xdp)
 		result = !err ? ICE_XDP_REDIR : ICE_XDP_CONSUMED;
 		break;
 	default:
-		bpf_warn_invalid_xdp_action(act);
-		/* fallthrough -- not supported action */
+		bpf_warn_invalid_xdp_action(rx_ring->netdev, xdp_prog, act);
+		fallthrough; /* not supported action */
 	case XDP_ABORTED:
 		trace_xdp_exception(rx_ring->netdev, xdp_prog, act);
-		/* fallthrough -- handle aborts by dropping frame */
+		fallthrough; /* handle aborts by dropping frame */
 	case XDP_DROP:
 		result = ICE_XDP_CONSUMED;
 		break;
@@ -881,11 +967,13 @@ int ice_clean_rx_irq_zc(struct ice_ring *rx_ring, int budget)
 	unsigned int total_rx_bytes = 0, total_rx_packets = 0;
 	u16 cleaned_count = ICE_DESC_UNUSED(rx_ring);
 	unsigned int xdp_xmit = 0;
-	struct xdp_buff xdp;
 	bool failure = false;
+#ifndef HAVE_MEM_TYPE_XSK_BUFF_POOL
+	struct xdp_buff xdp;
 
 	xdp.rxq = &rx_ring->xdp_rxq;
 
+#endif
 	while (likely(total_rx_packets < (unsigned int)budget)) {
 		union ice_32b_rx_flex_desc *rx_desc;
 		unsigned int size, xdp_res = 0;
@@ -896,15 +984,19 @@ int ice_clean_rx_irq_zc(struct ice_ring *rx_ring, int budget)
 		u16 rx_ptype;
 
 		if (cleaned_count >= ICE_RX_BUF_WRITE) {
+#ifndef HAVE_MEM_TYPE_XSK_BUFF_POOL
 			failure |= ice_alloc_rx_bufs_fast_zc(rx_ring,
 							     cleaned_count);
+#else
+			failure |= ice_alloc_rx_bufs_zc(rx_ring, cleaned_count);
+#endif
 			cleaned_count = 0;
 		}
 
 		rx_desc = ICE_RX_DESC(rx_ring, rx_ring->next_to_clean);
 
 		stat_err_bits = BIT(ICE_RX_FLEX_DESC_STATUS0_DD_S);
-		if (!ice_test_staterr(rx_desc, stat_err_bits))
+		if (!ice_test_staterr(rx_desc->wb.status_error0, stat_err_bits))
 			break;
 
 		/* This memory barrier is needed to keep us from reading
@@ -918,6 +1010,7 @@ int ice_clean_rx_irq_zc(struct ice_ring *rx_ring, int budget)
 		if (!size)
 			break;
 
+#ifndef HAVE_MEM_TYPE_XSK_BUFF_POOL
 		rx_buf = ice_get_rx_buf_zc(rx_ring, size);
 		if (!rx_buf->addr)
 			break;
@@ -929,14 +1022,31 @@ int ice_clean_rx_irq_zc(struct ice_ring *rx_ring, int budget)
 		xdp.handle = rx_buf->handle;
 
 		xdp_res = ice_run_xdp_zc(rx_ring, &xdp);
+#else
+		rx_buf = &rx_ring->rx_buf[rx_ring->next_to_clean];
+		if (!rx_buf->xdp)
+			break;
+
+		rx_buf->xdp->data_end = rx_buf->xdp->data + size;
+		xsk_buff_dma_sync_for_cpu(rx_buf->xdp, rx_ring->xsk_pool);
+
+		xdp_res = ice_run_xdp_zc(rx_ring, rx_buf->xdp);
+#endif
 		if (xdp_res) {
+#ifndef HAVE_MEM_TYPE_XSK_BUFF_POOL
 			if (xdp_res & (ICE_XDP_TX | ICE_XDP_REDIR)) {
 				xdp_xmit |= xdp_res;
 				rx_buf->addr = NULL;
 			} else {
 				ice_reuse_rx_buf_zc(rx_ring, rx_buf);
 			}
-
+#else
+			if (xdp_res & (ICE_XDP_TX | ICE_XDP_REDIR))
+				xdp_xmit |= xdp_res;
+			else
+				xsk_buff_free(rx_buf->xdp);
+			rx_buf->xdp = NULL;
+#endif
 			total_rx_bytes += size;
 			total_rx_packets++;
 			cleaned_count++;
@@ -946,7 +1056,11 @@ int ice_clean_rx_irq_zc(struct ice_ring *rx_ring, int budget)
 		}
 
 		/* XDP_PASS path */
+#ifndef HAVE_MEM_TYPE_XSK_BUFF_POOL
 		skb = ice_construct_skb_zc(rx_ring, rx_buf, &xdp);
+#else
+		skb = ice_construct_skb_zc(rx_ring, rx_buf, rx_buf->xdp);
+#endif
 		if (!skb) {
 			rx_ring->rx_stats.alloc_buf_failed++;
 			break;
@@ -963,9 +1077,7 @@ int ice_clean_rx_irq_zc(struct ice_ring *rx_ring, int budget)
 		total_rx_bytes += skb->len;
 		total_rx_packets++;
 
-		stat_err_bits = BIT(ICE_RX_FLEX_DESC_STATUS0_L2TAG1P_S);
-		if (ice_test_staterr(rx_desc, stat_err_bits))
-			vlan_tag = le16_to_cpu(rx_desc->wb.l2tag1);
+		vlan_tag = ice_get_vlan_tag_from_rx_desc(rx_desc);
 
 		rx_ptype = le16_to_cpu(rx_desc->wb.ptype_flex_flags0) &
 				       ICE_RX_FLEX_DESC_PTYPE_M;
@@ -977,6 +1089,19 @@ int ice_clean_rx_irq_zc(struct ice_ring *rx_ring, int budget)
 	ice_finalize_xdp_rx(rx_ring, xdp_xmit);
 	ice_update_rx_ring_stats(rx_ring, total_rx_packets, total_rx_bytes);
 
+#ifdef HAVE_NDO_XSK_WAKEUP
+	if (xsk_uses_need_wakeup(rx_ring->xsk_pool)) {
+		if (failure || rx_ring->next_to_clean == rx_ring->next_to_use ||
+		    (ice_ring_ch_enabled(rx_ring) &&
+		     !ice_vsi_pkt_inspect_opt_ena(rx_ring->vsi)))
+			xsk_set_rx_need_wakeup(rx_ring->xsk_pool);
+		else
+			xsk_clear_rx_need_wakeup(rx_ring->xsk_pool);
+
+		return (int)total_rx_packets;
+	}
+
+#endif /* HAVE_NDO_XSK_WAKEUP */
 	return failure ? budget : (int)total_rx_packets;
 }
 
@@ -989,8 +1114,9 @@ int ice_clean_rx_irq_zc(struct ice_ring *rx_ring, int budget)
  */
 static bool ice_xmit_zc(struct ice_ring *xdp_ring, int budget)
 {
+	unsigned int sent_frames = 0, total_bytes = 0;
 	struct ice_tx_desc *tx_desc = NULL;
-	bool work_done = true;
+	u16 ntu = xdp_ring->next_to_use;
 #ifdef XSK_UMEM_RETURNS_XDP_DESC
 	struct xdp_desc desc;
 #endif /* XSK_UMEM_RETURNS_XDP_DESC */
@@ -1002,26 +1128,25 @@ static bool ice_xmit_zc(struct ice_ring *xdp_ring, int budget)
 	while (likely(budget-- > 0)) {
 		struct ice_tx_buf *tx_buf;
 
-		if (unlikely(!ICE_DESC_UNUSED(xdp_ring))) {
-			xdp_ring->tx_stats.tx_busy++;
-			work_done = false;
-			break;
-		}
-
-		tx_buf = &xdp_ring->tx_buf[xdp_ring->next_to_use];
+		tx_buf = &xdp_ring->tx_buf[ntu];
 
 #ifdef XSK_UMEM_RETURNS_XDP_DESC
-		if (!xsk_umem_consume_tx(xdp_ring->xsk_umem, &desc))
+		if (!xsk_tx_peek_desc(xdp_ring->xsk_pool, &desc))
 			break;
 
-		dma = xdp_umem_get_dma(xdp_ring->xsk_umem, desc.addr);
+#ifndef HAVE_MEM_TYPE_XSK_BUFF_POOL
+		dma = xdp_umem_get_dma(xdp_ring->xsk_pool, desc.addr);
 
 		dma_sync_single_for_device(xdp_ring->dev, dma, desc.len,
 					   DMA_BIDIRECTIONAL);
-
+#else
+		dma = xsk_buff_raw_get_dma(xdp_ring->xsk_pool, desc.addr);
+		xsk_buff_raw_dma_sync_for_device(xdp_ring->xsk_pool, dma,
+						 desc.len);
+#endif
 		tx_buf->bytecount = desc.len;
 #else
-		if (!xsk_umem_consume_tx(xdp_ring->xsk_umem, &dma, &len))
+		if (!xsk_tx_peek_desc(xdp_ring->xsk_pool, &dma, &len))
 			break;
 
 		dma_sync_single_for_device(xdp_ring->dev, dma, len,
@@ -1030,26 +1155,34 @@ static bool ice_xmit_zc(struct ice_ring *xdp_ring, int budget)
 		tx_buf->bytecount = len;
 #endif /* XSK_UMEM_RETURNS_XDP_DESC */
 
-		tx_desc = ICE_TX_DESC(xdp_ring, xdp_ring->next_to_use);
+		tx_desc = ICE_TX_DESC(xdp_ring, ntu);
 		tx_desc->buf_addr = cpu_to_le64(dma);
 		tx_desc->cmd_type_offset_bsz =
 #ifdef XSK_UMEM_RETURNS_XDP_DESC
-			ice_build_ctob(ICE_TXD_LAST_DESC_CMD, 0, desc.len, 0);
+			ice_build_ctob(ICE_TX_DESC_CMD_EOP, 0, desc.len, 0);
 #else
-			ice_build_ctob(ICE_TXD_LAST_DESC_CMD, 0, len, 0);
+			ice_build_ctob(ICE_TX_DESC_CMD_EOP, 0, len, 0);
 #endif /* XSK_UMEM_RETURNS_XDP_DESC */
 
-		xdp_ring->next_to_use++;
-		if (xdp_ring->next_to_use == xdp_ring->count)
-			xdp_ring->next_to_use = 0;
+		xdp_ring->next_rs_idx = ntu;
+		ntu++;
+		if (ntu == xdp_ring->count)
+			ntu = 0;
+		sent_frames++;
+		total_bytes += tx_buf->bytecount;
 	}
 
 	if (tx_desc) {
+		xdp_ring->next_to_use = ntu;
+		/* Set RS bit for the last frame and bump tail ptr */
+		tx_desc->cmd_type_offset_bsz |=
+			cpu_to_le64(ICE_TX_DESC_CMD_RS << ICE_TXD_QW1_CMD_S);
 		ice_xdp_ring_update_tail(xdp_ring);
-		xsk_umem_consume_tx_done(xdp_ring->xsk_umem);
+		xsk_tx_release(xdp_ring->xsk_pool);
+		ice_update_tx_ring_stats(xdp_ring, sent_frames, total_bytes);
 	}
 
-	return budget > 0 && work_done;
+	return budget > 0;
 }
 
 /**
@@ -1061,6 +1194,7 @@ static void
 ice_clean_xdp_tx_buf(struct ice_ring *xdp_ring, struct ice_tx_buf *tx_buf)
 {
 	xdp_return_frame((struct xdp_frame *)tx_buf->raw_buf);
+	xdp_ring->xdp_tx_active--;
 	dma_unmap_single(xdp_ring->dev, dma_unmap_addr(tx_buf, dma),
 			 dma_unmap_len(tx_buf, len), DMA_TO_DEVICE);
 	dma_unmap_len_set(tx_buf, len, 0);
@@ -1069,30 +1203,38 @@ ice_clean_xdp_tx_buf(struct ice_ring *xdp_ring, struct ice_tx_buf *tx_buf)
 /**
  * ice_clean_tx_irq_zc - Completes AF_XDP entries, and cleans XDP entries
  * @xdp_ring: XDP Tx ring
- * @budget: NAPI budget
  *
  * Returns true if cleanup/tranmission is done.
  */
-bool ice_clean_tx_irq_zc(struct ice_ring *xdp_ring, int budget)
+bool ice_clean_tx_irq_zc(struct ice_ring *xdp_ring)
 {
-	int total_packets = 0, total_bytes = 0;
-	s16 ntc = xdp_ring->next_to_clean;
-	struct ice_tx_desc *tx_desc;
+	u16 next_rs_idx = xdp_ring->next_rs_idx;
+	u16 ntc = xdp_ring->next_to_clean;
+	u16 frames_ready = 0, send_budget;
+	struct ice_tx_desc *next_rs_desc;
 	struct ice_tx_buf *tx_buf;
-	bool xmit_done = true;
 	u32 xsk_frames = 0;
+	u16 i;
 
-	tx_desc = ICE_TX_DESC(xdp_ring, ntc);
-	tx_buf = &xdp_ring->tx_buf[ntc];
-	ntc -= xdp_ring->count;
+	next_rs_desc = ICE_TX_DESC(xdp_ring, next_rs_idx);
+	if (next_rs_desc->cmd_type_offset_bsz &
+	    cpu_to_le64(ICE_TX_DESC_DTYPE_DESC_DONE)) {
+		if (next_rs_idx >= ntc)
+			frames_ready = next_rs_idx - ntc;
+		else
+			frames_ready = next_rs_idx + xdp_ring->count - ntc;
+	}
 
-	do {
-		if (!(tx_desc->cmd_type_offset_bsz &
-		      cpu_to_le64(ICE_TX_DESC_DTYPE_DESC_DONE)))
-			break;
+	if (!frames_ready)
+		goto out_xmit;
 
-		total_bytes += tx_buf->bytecount;
-		total_packets++;
+	if (likely(!xdp_ring->xdp_tx_active)) {
+		xsk_frames = frames_ready;
+		goto skip;
+	}
+
+	for (i = 0; i < frames_ready; i++) {
+		tx_buf = &xdp_ring->tx_buf[ntc];
 
 		if (tx_buf->raw_buf) {
 			ice_clean_xdp_tx_buf(xdp_ring, tx_buf);
@@ -1101,31 +1243,27 @@ bool ice_clean_tx_irq_zc(struct ice_ring *xdp_ring, int budget)
 			xsk_frames++;
 		}
 
-		tx_desc->cmd_type_offset_bsz = 0;
-		tx_buf++;
-		tx_desc++;
-		ntc++;
+		++ntc;
+		if (ntc >= xdp_ring->count)
+			ntc = 0;
+	}
 
-		if (unlikely(!ntc)) {
-			ntc -= xdp_ring->count;
-			tx_buf = xdp_ring->tx_buf;
-			tx_desc = ICE_TX_DESC(xdp_ring, 0);
-		}
-
-		prefetch(tx_desc);
-
-	} while (likely(--budget));
-
-	ntc += xdp_ring->count;
-	xdp_ring->next_to_clean = ntc;
+skip:
+	xdp_ring->next_to_clean += frames_ready;
+	if (unlikely(xdp_ring->next_to_clean >= xdp_ring->count))
+		xdp_ring->next_to_clean -= xdp_ring->count;
 
 	if (xsk_frames)
-		xsk_umem_complete_tx(xdp_ring->xsk_umem, xsk_frames);
+		xsk_tx_completed(xdp_ring->xsk_pool, xsk_frames);
 
-	ice_update_tx_ring_stats(xdp_ring, total_packets, total_bytes);
-	xmit_done = ice_xmit_zc(xdp_ring, ICE_DFLT_IRQ_WORK);
-
-	return budget > 0 && xmit_done;
+out_xmit:
+#ifdef HAVE_NDO_XSK_WAKEUP
+	if (xsk_uses_need_wakeup(xdp_ring->xsk_pool))
+		xsk_set_tx_need_wakeup(xdp_ring->xsk_pool);
+#endif /* HAVE_NDO_XSK_WAKEUP */
+	send_budget = ICE_DESC_UNUSED(xdp_ring);
+	send_budget = min_t(u16, send_budget, xdp_ring->count >> 2);
+	return ice_xmit_zc(xdp_ring, send_budget);
 }
 
 #ifdef HAVE_NDO_XSK_WAKEUP
@@ -1149,7 +1287,7 @@ int ice_xsk_async_xmit(struct net_device *netdev, u32 queue_id)
 	struct ice_vsi *vsi = np->vsi;
 	struct ice_ring *ring;
 
-	if (test_bit(__ICE_DOWN, vsi->state))
+	if (test_bit(ICE_VSI_DOWN, vsi->state))
 		return -ENETDOWN;
 
 	if (!ice_is_xdp_ena_vsi(vsi))
@@ -1158,7 +1296,7 @@ int ice_xsk_async_xmit(struct net_device *netdev, u32 queue_id)
 	if (queue_id >= vsi->num_txq)
 		return -ENXIO;
 
-	if (!vsi->xdp_rings[queue_id]->xsk_umem)
+	if (!vsi->xdp_rings[queue_id]->xsk_pool)
 		return -ENXIO;
 
 	ring = vsi->xdp_rings[queue_id];
@@ -1170,8 +1308,17 @@ int ice_xsk_async_xmit(struct net_device *netdev, u32 queue_id)
 	 * honored.
 	 */
 	q_vector = ring->q_vector;
-	if (!napi_if_scheduled_mark_missed(&q_vector->napi))
-		ice_trigger_sw_intr(&vsi->back->hw, q_vector);
+	if (!napi_if_scheduled_mark_missed(&q_vector->napi)) {
+#if IS_ENABLED(CONFIG_NET_RX_BUSY_POLL)
+		if (ice_ring_ch_enabled(vsi->rx_rings[queue_id]) &&
+		    !ice_vsi_pkt_inspect_opt_ena(vsi))
+#define ICE_BUSY_POLL_BUDGET 8
+			napi_busy_loop(q_vector->napi.napi_id, NULL, NULL,
+				       false, ICE_BUSY_POLL_BUDGET);
+		else
+#endif
+			ice_trigger_sw_intr(&vsi->back->hw, q_vector);
+	}
 
 	return 0;
 }
@@ -1186,6 +1333,7 @@ bool ice_xsk_any_rx_ring_ena(struct ice_vsi *vsi)
 {
 	int i;
 
+#ifndef HAVE_AF_XDP_NETDEV_UMEM
 	if (!vsi->xsk_umems)
 		return false;
 
@@ -1193,6 +1341,12 @@ bool ice_xsk_any_rx_ring_ena(struct ice_vsi *vsi)
 		if (vsi->xsk_umems[i])
 			return true;
 	}
+#else
+	ice_for_each_rxq(vsi, i) {
+		if (xsk_get_pool_from_qid(vsi->netdev, i))
+			return true;
+	}
+#endif /* HAVE_AF_XDP_NETDEV_UMEM */
 
 	return false;
 }
@@ -1211,7 +1365,9 @@ void ice_xsk_clean_rx_ring(struct ice_ring *rx_ring)
 		if (!rx_buf->addr)
 			continue;
 
-		xsk_umem_fq_reuse(rx_ring->xsk_umem, rx_buf->handle);
+#ifndef HAVE_MEM_TYPE_XSK_BUFF_POOL
+		xsk_umem_fq_reuse(rx_ring->xsk_pool, rx_buf->handle);
+#endif
 		rx_buf->addr = NULL;
 	}
 }
@@ -1241,6 +1397,6 @@ void ice_xsk_clean_xdp_ring(struct ice_ring *xdp_ring)
 	}
 
 	if (xsk_frames)
-		xsk_umem_complete_tx(xdp_ring->xsk_umem, xsk_frames);
+		xsk_tx_completed(xdp_ring->xsk_pool, xsk_frames);
 }
 #endif /* HAVE_AF_XDP_ZC_SUPPORT */
