@@ -1,26 +1,5 @@
-/*******************************************************************************
-
-  Intel(R) Gigabit Ethernet Linux Driver
-  Copyright(c) 2007 - 2018 Intel Corporation.
-
-  This program is free software; you can redistribute it and/or modify it
-  under the terms and conditions of the GNU General Public License,
-  version 2, as published by the Free Software Foundation.
-
-  This program is distributed in the hope it will be useful, but WITHOUT
-  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-  more details.
-
-  The full GNU General Public License is included in this distribution in
-  the file called "COPYING".
-
-  Contact Information:
-  Linux NICS <linux.nics@intel.com>
-  e1000-devel Mailing List <e1000-devel@lists.sourceforge.net>
-  Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497
-
-*******************************************************************************/
+/* SPDX-License-Identifier: GPL-2.0 */
+/* Copyright(c) 2007 - 2022 Intel Corporation. */
 
 /* Linux PRO/1000 Ethernet Driver main header file */
 
@@ -125,10 +104,10 @@ struct igb_adapter;
 #define MAX_Q_VECTORS                     10
 
 /* Transmit and receive queues */
-#define IGB_MAX_RX_QUEUES                 16
+#define IGB_MAX_RX_QUEUES                  8
 #define IGB_MAX_RX_QUEUES_82575            4
 #define IGB_MAX_RX_QUEUES_I211             2
-#define IGB_MAX_TX_QUEUES                 16
+#define IGB_MAX_TX_QUEUES                  8
 
 #define IGB_MAX_VF_MC_ENTRIES             30
 #define IGB_MAX_VF_FUNCTIONS               8
@@ -151,6 +130,7 @@ struct vf_data_storage {
 	u32 uta_table_copy[IGB_MAX_UTA_ENTRIES];
 	u32 flags;
 	unsigned long last_nack;
+	bool trusted;
 #ifdef IFLA_VF_MAX
 	u16 pf_vlan; /* When set, guest VLAN config not allowed. */
 	u16 pf_qos;
@@ -161,6 +141,13 @@ struct vf_data_storage {
 #endif
 };
 
+struct vf_mac_filter {
+	struct list_head l;
+	int vf;
+	bool free;
+	u8 vf_mac[ETH_ALEN];
+};
+
 #define IGB_VF_FLAG_CTS            0x00000001 /* VF is clear to send data */
 #define IGB_VF_FLAG_UNI_PROMISC    0x00000002 /* VF has unicast promisc */
 #define IGB_VF_FLAG_MULTI_PROMISC  0x00000004 /* VF has multicast promisc */
@@ -225,9 +212,9 @@ struct vf_data_storage {
 #define IGB_EEPROM_APME         0x0400
 #define AUTO_ALL_MODES          0
 
-#ifndef IGB_MASTER_SLAVE
-/* Switch to override PHY master/slave setting */
-#define IGB_MASTER_SLAVE	e1000_ms_hw_default
+#ifndef IGB_PRIMARY_SECONDARY
+/* Switch to override PHY primary/secondary setting */
+#define IGB_PRIMARY_SECONDARY	e1000_ms_hw_default
 #endif
 
 #define IGB_MNG_VLAN_NONE -1
@@ -301,7 +288,7 @@ enum igb_tx_flags {
  * maintain a power of two alignment we have to limit ourselves to 32K.
  */
 #define IGB_MAX_TXD_PWR		15
-#define IGB_MAX_DATA_PER_TXD	BIT(IGB_MAX_TXD_PWR)
+#define IGB_MAX_DATA_PER_TXD	(1u << IGB_MAX_TXD_PWR)
 
 /* Tx Descriptors needed, worst case */
 #define TXD_USE_COUNT(S)	DIV_ROUND_UP((S), IGB_MAX_DATA_PER_TXD)
@@ -473,6 +460,8 @@ struct igb_mac_addr {
 #define IGB_MAC_STATE_DEFAULT	0x1
 #define IGB_MAC_STATE_MODIFIED	0x2
 #define IGB_MAC_STATE_IN_USE	0x4
+#define IGB_MAC_STATE_SRC_ADDR  0x8
+#define IGB_MAC_STATE_QUEUE_STEERING 0x10
 
 #define IGB_TXD_DCMD (E1000_ADVTXD_DCMD_EOP | E1000_ADVTXD_DCMD_RS)
 
@@ -509,12 +498,10 @@ static inline u16 igb_desc_unused(const struct igb_ring *ring)
 	return ((ntc > ntu) ? 0 : ring->count) + ntc - ntu - 1;
 }
 
-#ifdef CONFIG_BQL
 static inline struct netdev_queue *txring_txq(const struct igb_ring *tx_ring)
 {
 	return netdev_get_tx_queue(tx_ring->netdev, tx_ring->queue_index);
 }
-#endif /* CONFIG_BQL */
 
 struct igb_therm_proc_data {
 	struct e1000_hw *hw;
@@ -532,17 +519,47 @@ struct hwmon_attr {
 	struct e1000_hw *hw;
 	struct e1000_thermal_diode_data *sensor;
 	char name[12];
-	};
+};
 
 struct hwmon_buff {
 	struct device *device;
 	struct hwmon_attr *hwmon_list;
 	unsigned int n_hwmon;
-	};
+};
 #endif /* IGB_HWMON */
 #ifdef ETHTOOL_GRXFHINDIR
 #define IGB_RETA_SIZE	128
 #endif /* ETHTOOL_GRXFHINDIR */
+enum igb_filter_match_flags {
+	IGB_FILTER_FLAG_NONE = 0x0,
+	IGB_FILTER_FLAG_ETHER_TYPE = 0x1,
+	IGB_FILTER_FLAG_VLAN_TCI   = 0x2,
+	IGB_FILTER_FLAG_SRC_MAC_ADDR = 0x4,
+	IGB_FILTER_FLAG_DST_MAC_ADDR = 0x8,
+};
+
+#define IGB_MAX_RXNFC_FILTERS 16
+
+/* RX network flow classification data structure */
+struct igb_nfc_input {
+	/* Byte layout in order, all values with MSB first:
+	 * match_flags - 1 byte
+	 * etype - 2 bytes
+	 * vlan_tci - 2 bytes
+	 */
+	u8 match_flags;
+	__be16 etype;
+	__be16 vlan_tci;
+	u8 src_addr[ETH_ALEN];
+	u8 dst_addr[ETH_ALEN];
+};
+
+struct igb_nfc_filter {
+	struct hlist_node nfc_node;
+	struct igb_nfc_input filter;
+	u16 sw_idx;
+	u16 action;
+};
 
 /* board specific private data structure */
 struct igb_adapter {
@@ -583,6 +600,7 @@ struct igb_adapter {
 	u8 port_num;
 
 	u8 __iomem *io_addr; /* for iounmap */
+	u8 __iomem *flash_addr;
 
 	/* Interrupt Throttle Rate */
 	u32 rx_itr_setting;
@@ -623,6 +641,7 @@ struct igb_adapter {
 
 	int msg_enable;
 
+	struct vf_mac_filter vf_macs;
 	struct igb_q_vector *q_vector[MAX_Q_VECTORS];
 	u32 eims_enable_mask;
 	u32 eims_other;
@@ -697,6 +716,13 @@ struct igb_adapter {
 
 	int copper_tries;
 	u16 eee_advert;
+
+	/* RX network flow classification support */
+	struct hlist_head nfc_filter_list;
+	unsigned int nfc_filter_count;
+	/* lock for RX network flow classification filter */
+	spinlock_t nfc_lock;
+
 #ifdef ETHTOOL_GRXFHINDIR
 	u32 rss_indir_tbl_init;
 	u8 rss_indir_tbl[IGB_RETA_SIZE];
@@ -741,7 +767,8 @@ struct igb_vmdq_adapter {
 #define IGB_FLAG_NEED_LINK_UPDATE	BIT(12)
 #define IGB_FLAG_LOOPBACK_ENABLE	BIT(13)
 #define IGB_FLAG_MEDIA_RESET		BIT(14)
-#define IGB_FLAG_MAS_ENABLE		BIT(15)
+#define IGB_FLAG_VLAN_PROMISC		BIT(15)
+#define IGB_FLAG_MAS_ENABLE		BIT(16)
 
 /* Media Auto Sense */
 #define IGB_MAS_ENABLE_0		0X0001
@@ -777,15 +804,16 @@ struct igb_vmdq_adapter {
 #define FW_HDR_LEN           0x4
 #define FW_CMD_DRV_INFO      0xDD
 #define FW_CMD_DRV_INFO_LEN  0x5
+#define FW_CMD_DRV_INFO_LEN_NEW 0x9
 #define FW_CMD_RESERVED      0X0
 #define FW_RESP_SUCCESS      0x1
-#define FW_UNUSED_VER        0x0
 #define FW_MAX_RETRIES       3
 #define FW_STATUS_SUCCESS    0x1
-#define FW_FAMILY_DRV_VER    0Xffffffff
+#define FW_FAMILY_DRV_VER    0XFFFFFFFF
 
 #define IGB_MAX_LINK_TRIES   20
 
+#pragma pack(push, 1)
 struct e1000_fw_hdr {
 	u8 cmd;
 	u8 buf_len;
@@ -796,16 +824,26 @@ struct e1000_fw_hdr {
 	u8 checksum;
 };
 
-#pragma pack(push, 1)
 struct e1000_fw_drv_info {
 	struct e1000_fw_hdr hdr;
 	u8 port_num;
-	u32 drv_version;
+	u32 family_drv_version;
+	u32 actual_drv_version;
 	u16 pad; /* end spacing to ensure length is mult. of dword */
 	u8  pad2; /* end spacing to ensure length is mult. of dword2 */
 };
 #pragma pack(pop)
 
+enum host_cmd_id_status {
+	HCI_STATUS_OK = 1,
+	HCI_ILLEGAL_CMD_ID,
+	HCI_UNSUPPORTED_CMD,
+	HCI_ILLEGAL_PAYLOAD_LENGTH,
+	HCI_CHECKSUM_FAILED,
+	HCI_DATA_ERROR,
+	HCI_INVALID_PARAMETER
+};
+
 enum e1000_state_t {
 	__IGB_TESTING,
 	__IGB_RESETTING,
@@ -816,63 +854,63 @@ enum e1000_state_t {
 extern char igb_driver_name[];
 extern char igb_driver_version[];
 
-extern int igb_open(struct net_device *netdev);
-extern int igb_close(struct net_device *netdev);
-extern int igb_up(struct igb_adapter *);
-extern void igb_down(struct igb_adapter *);
-extern void igb_reinit_locked(struct igb_adapter *);
-extern void igb_reset(struct igb_adapter *);
-extern int igb_reinit_queues(struct igb_adapter *);
+int igb_open(struct net_device *netdev);
+int igb_close(struct net_device *netdev);
+void igb_up(struct igb_adapter *adapter);
+void igb_down(struct igb_adapter *adapter);
+void igb_reinit_locked(struct igb_adapter *adapter);
+void igb_reset(struct igb_adapter *adapter);
+int igb_reinit_queues(struct igb_adapter *adapter);
 #ifdef ETHTOOL_SRXFHINDIR
-extern void igb_write_rss_indir_tbl(struct igb_adapter *);
+void igb_write_rss_indir_tbl(struct igb_adapter *adapter);
 #endif
-extern int igb_set_spd_dplx(struct igb_adapter *, u16);
-extern int igb_setup_tx_resources(struct igb_ring *);
-extern int igb_setup_rx_resources(struct igb_ring *);
-extern void igb_free_tx_resources(struct igb_ring *);
-extern void igb_free_rx_resources(struct igb_ring *);
-extern void igb_configure_tx_ring(struct igb_adapter *, struct igb_ring *);
-extern void igb_configure_rx_ring(struct igb_adapter *, struct igb_ring *);
-extern void igb_setup_tctl(struct igb_adapter *);
-extern void igb_setup_rctl(struct igb_adapter *);
-extern netdev_tx_t igb_xmit_frame_ring(struct sk_buff *, struct igb_ring *);
-extern void igb_unmap_and_free_tx_resource(struct igb_ring *,
-					   struct igb_tx_buffer *);
-extern void igb_alloc_rx_buffers(struct igb_ring *, u16);
-extern void igb_clean_rx_ring(struct igb_ring *);
-extern int igb_setup_queues(struct igb_adapter *adapter);
-extern void igb_update_stats(struct igb_adapter *);
-extern bool igb_has_link(struct igb_adapter *adapter);
-extern void igb_set_ethtool_ops(struct net_device *);
-extern void igb_check_options(struct igb_adapter *);
-extern void igb_power_up_link(struct igb_adapter *);
+int igb_set_spd_dplx(struct igb_adapter *, u16);
+int igb_setup_tx_resources(struct igb_ring *tx_ring);
+int igb_setup_rx_resources(struct igb_ring *rx_ring);
+void igb_free_tx_resources(struct igb_ring *tx_ring);
+void igb_free_rx_resources(struct igb_ring *rx_ring);
+void igb_configure_tx_ring(struct igb_adapter *adapter,
+			   struct igb_ring *tx_ring);
+void igb_configure_rx_ring(struct igb_adapter *adapter,
+			   struct igb_ring *rx_ring);
+void igb_setup_tctl(struct igb_adapter *adapter);
+void igb_setup_rctl(struct igb_adapter *adapter);
+netdev_tx_t igb_xmit_frame_ring(struct sk_buff *skb, struct igb_ring *tx_ring);
+void igb_unmap_and_free_tx_resource(struct igb_ring *ring,
+				    struct igb_tx_buffer *tx_buffer);
+void igb_alloc_rx_buffers(struct igb_ring *rx_ring, u16 cleaned_count);
+void igb_clean_rx_ring(struct igb_ring *rx_ring);
+int igb_setup_queues(struct igb_adapter *adapter);
+void igb_update_stats(struct igb_adapter *adapter);
+bool igb_has_link(struct igb_adapter *adapter);
+void igb_set_ethtool_ops(struct net_device *netdev);
+void igb_check_options(struct igb_adapter *adapter);
+void igb_power_up_link(struct igb_adapter *adapter);
 #ifdef HAVE_PTP_1588_CLOCK
-extern void igb_ptp_init(struct igb_adapter *adapter);
-extern void igb_ptp_stop(struct igb_adapter *adapter);
-extern void igb_ptp_reset(struct igb_adapter *adapter);
-extern void igb_ptp_tx_work(struct work_struct *work);
-extern void igb_ptp_rx_hang(struct igb_adapter *adapter);
-extern void igb_ptp_tx_hwtstamp(struct igb_adapter *adapter);
-extern void igb_ptp_rx_rgtstamp(struct igb_q_vector *q_vector,
-				struct sk_buff *skb);
-extern void igb_ptp_rx_pktstamp(struct igb_q_vector *q_vector,
-				unsigned char *va,
-				struct sk_buff *skb);
-extern int igb_ptp_hwtstamp_ioctl(struct net_device *netdev,
-				  struct ifreq *ifr, int cmd);
+void igb_ptp_init(struct igb_adapter *adapter);
+void igb_ptp_stop(struct igb_adapter *adapter);
+void igb_ptp_reset(struct igb_adapter *adapter);
+void igb_ptp_tx_work(struct work_struct *work);
+void igb_ptp_rx_hang(struct igb_adapter *adapter);
+void igb_ptp_tx_hwtstamp(struct igb_adapter *adapter);
+void igb_ptp_rx_rgtstamp(struct igb_q_vector *q_vector,
+			 struct sk_buff *skb);
+void igb_ptp_rx_pktstamp(struct igb_q_vector *q_vector,
+			 unsigned char *va,
+			 struct sk_buff *skb);
+int igb_ptp_hwtstamp_ioctl(struct net_device *netdev,
+			   struct ifreq *ifr, int cmd);
 #endif /* HAVE_PTP_1588_CLOCK */
 #ifdef ETHTOOL_OPS_COMPAT
-extern int ethtool_ioctl(struct ifreq *);
+int ethtool_ioctl(struct ifreq *);
 #endif
-extern int igb_write_mc_addr_list(struct net_device *netdev);
-extern int igb_add_mac_filter(struct igb_adapter *adapter, u8 *addr, u16 queue);
-extern int igb_del_mac_filter(struct igb_adapter *adapter, u8 *addr, u16 queue);
-extern int igb_available_rars(struct igb_adapter *adapter);
-extern s32 igb_vlvf_set(struct igb_adapter *, u32, bool, u32);
-extern void igb_configure_vt_default_pool(struct igb_adapter *adapter);
-extern void igb_enable_vlan_tags(struct igb_adapter *adapter);
+int igb_write_mc_addr_list(struct net_device *netdev);
+int igb_available_rars(struct igb_adapter *adapter);
+s32 igb_vlvf_set(struct igb_adapter *adapter, u32 vid, bool add, u32 vf);
+void igb_configure_vt_default_pool(struct igb_adapter *adapter);
+void igb_enable_vlan_tags(struct igb_adapter *adapter);
 #ifndef HAVE_VLAN_RX_REGISTER
-extern void igb_vlan_mode(struct net_device *, u32);
+void igb_vlan_mode(struct net_device *, u32);
 #endif
 
 #define E1000_PCS_CFG_IGN_SD	1
@@ -891,4 +929,11 @@ void igb_procfs_topdir_exit(void);
 #endif /* IGB_PROCFS */
 #endif /* IGB_HWMON */
 
+int igb_add_filter(struct igb_adapter *adapter, struct igb_nfc_filter *input);
+int igb_del_filter(struct igb_adapter *adapter, struct igb_nfc_filter *input);
+
+int igb_add_mac_steering_filter(struct igb_adapter *adapter,
+				const u8 *addr, u8 queue, u8 flags);
+int igb_del_mac_steering_filter(struct igb_adapter *adapter,
+				const u8 *addr, u8 queue, u8 flags);
 #endif /* _IGB_H_ */
