@@ -2,7 +2,7 @@
  *
  * Definitions for packet ring
  *
- * 2004-2021 - ntop.org
+ * 2004-23 - ntop
  *
  */
 
@@ -23,11 +23,11 @@
 #endif /* __KERNEL__ */
 
 /* Versioning */
-#define RING_VERSION                "8.0.0"
-#define RING_VERSION_NUM           0x080000
+#define RING_VERSION                "8.4.0"
+#define RING_VERSION_NUM           0x080400
 
 /* Increment whenever we change slot or packet header layout (e.g. we add/move a field) */
-#define RING_FLOWSLOT_VERSION          18
+#define RING_FLOWSLOT_VERSION          20
 
 #define RING_MAGIC
 #define RING_MAGIC_VALUE             0x88
@@ -69,10 +69,13 @@
 #define SO_REHASH_RSS_PACKET             119
 #define SO_SET_FILTERING_SAMPLING_RATE   120
 #define SO_SET_POLL_WATERMARK_TIMEOUT    121
+#define SO_SET_DEV_TIME			 122
+#define SO_ADJ_DEV_TIME			 123
 #define SO_SHUTDOWN_RING                 124
 #define SO_PURGE_IDLE_RULES              125 /* inactivity (sec) */
 #define SO_SET_SOCKET_MODE               126
 #define SO_USE_SHORT_PKT_HEADER          127
+#define SO_CONTROL_DEV_QUEUE		 128
 #define SO_ENABLE_RX_PACKET_BOUNCE       131
 #define SO_SET_APPL_STATS                133
 #define SO_SET_STACK_INJECTION_MODE      134 /* stack injection/interception from userspace */
@@ -103,8 +106,8 @@
 #define SO_GET_DEVICE_IFINDEX            185
 #define SO_GET_APPL_STATS_FILE_NAME      186
 #define SO_GET_LINK_STATUS               187
-
-/* Other *sockopt */
+#define SO_GET_DEV_TX_TIME		 188
+#define SO_GET_DEV_STATS		 189
 #define SO_SELECT_ZC_DEVICE              190
 
 /* Error codes */
@@ -143,13 +146,13 @@
 typedef long __kernel_time_t;
 
 struct timeval {
-	__kernel_time_t		tv_sec;		/* seconds */
-	__kernel_suseconds_t	tv_usec;	/* microseconds */
+  __kernel_time_t		tv_sec;		/* seconds */
+  __kernel_suseconds_t	tv_usec;	/* microseconds */
 };
 
 struct timespec {
-	__kernel_time_t	tv_sec;			/* seconds */
-	long		tv_nsec;		/* nanoseconds */
+  __kernel_time_t	tv_sec;			/* seconds */
+  long		tv_nsec;		/* nanoseconds */
 };
 
 struct timespec ns_to_timespec(const s64 nsec);
@@ -166,9 +169,9 @@ struct timeval ns_to_timeval(const s64 nsec);
   please do not change them to unsigned
 */
 struct pkt_offset {
-  /* This 'eth_offset' offset *must* be added to all offsets below 
+  /* This 'eth_offset' offset *must* be added to all offsets below
    * ONLY if you are inside the kernel. Ignore it in user-space. */
-  int16_t eth_offset; 
+  int16_t eth_offset;
 
   int16_t vlan_offset;
   int16_t l3_offset;
@@ -219,8 +222,8 @@ struct eth_vlan_hdr {
 
 struct kcompact_ipv6_hdr {
   u_int32_t         flow_lbl:24,
-		    priority:4,
-                    version:4;
+    priority:4,
+    version:4;
   u_int16_t         payload_len;
   u_int8_t          nexthdr;
   u_int8_t          hop_limit;
@@ -268,7 +271,7 @@ struct gtp_v1_hdr {
 } __attribute__((__packed__));
 
 /* Optional: GTP_FLAGS_EXTENSION | GTP_FLAGS_SEQ_NUM | GTP_FLAGS_NPDU_NUM */
-struct gtp_v1_opt_hdr { 
+struct gtp_v1_opt_hdr {
   u_int16_t seq_num;
   u_int8_t  npdu_num;
   u_int8_t  next_ext_hdr;
@@ -286,15 +289,17 @@ struct gtp_v1_ext_hdr {
 
 #define NO_TUNNEL_ID     0xFFFFFFFF
 
-/* GPRS Tunneling Protocol */
+/* Tunneling Protocols (e.g. GTP) */
 typedef struct {
   u_int32_t tunnel_id; /* GTP/GRE tunnelId or NO_TUNNEL_ID for no filtering */
+  u_int8_t  tunneled_dmac[ETH_ALEN], tunneled_smac[ETH_ALEN];  /* MAC src/dst addresses */
+  u_int16_t tunneled_eth_type;         /* Ethernet type */
   u_int8_t tunneled_ip_version; /* Layer 4 protocol */
   u_int8_t tunneled_proto; /* Layer 4 protocol */
-  ip_addr tunneled_ip_src, tunneled_ip_dst;  
+  ip_addr tunneled_ip_src, tunneled_ip_dst;
   u_int16_t tunneled_l4_src_port, tunneled_l4_dst_port;
 } __attribute__((packed))
- tunnel_info;
+tunnel_info;
 
 #define MOBILE_IP_PORT           434
 
@@ -303,6 +308,15 @@ struct mobile_ip_hdr {
   u_int16_t reserved;
 } __attribute__((packed));
 
+#define VXLAN_IP_PORT           4789
+
+struct vxlan_hdr {
+  u_int8_t flags[2];
+  u_int16_t gdp;
+  u_int8_t vni[3];
+  u_int8_t res;
+} __attribute__((packed));
+
 typedef enum {
   long_pkt_header = 0, /* it includes PF_RING-extensions over the original pcap header */
   short_pkt_header     /* Short pcap-like header */
@@ -365,6 +379,10 @@ struct pfring_extended_pkthdr {
     struct sk_buff *reserved; /* Kernel only pointer */
   } tx;
 
+  struct {
+    u_int32_t pid /* process id */;
+  } process;
+
   /* NOTE: leave it as last field of the memset on parse_pkt() will fail */
   struct pkt_parsing_info parsed_pkt; /* packet parsing info */
 } __attribute__((packed));
@@ -386,7 +404,7 @@ struct pfring_pkthdr {
 #ifdef __KERNEL__
 typedef struct {
   u_int32_t num_elements, top_element_id;
-  rwlock_t list_lock;
+  spinlock_t list_lock;
   void *list_elements[MAX_NUM_LIST_ELEMENTS];
 } lockless_list;
 
@@ -438,6 +456,8 @@ filtering_rule_extended_fields;
 
 /* ************************************************* */
 
+#define FILTERING_RULE_AUTO_RULE_ID 0xFFFF
+
 typedef enum {
   forward_packet_and_stop_rule_evaluation = 0,
   dont_forward_packet_and_stop_rule_evaluation,
@@ -464,7 +484,8 @@ typedef enum {
 typedef enum {
   send_and_recv_mode = 0,
   send_only_mode,
-  recv_only_mode
+  recv_only_mode,
+  management_only_mode
 } socket_mode;
 
 typedef struct {
@@ -474,8 +495,8 @@ typedef struct {
 filtering_internals;
 
 typedef struct {
-#define FILTERING_RULE_AUTO_RULE_ID 0xFFFF
-  u_int16_t rule_id;                 /* Rules are processed in order from lowest to higest id */
+  /* FILTERING_RULE_AUTO_RULE_ID to auto generate a rule ID */
+  u_int16_t rule_id;                 /* Rules are processed in order from lowest to highest id */
 
   rule_action_behaviour rule_action; /* What to do in case of match */
   u_int8_t balance_id, balance_pool; /* If balance_pool > 0, then pass the packet above only if the
@@ -515,11 +536,11 @@ intel_82599_perfect_filter_hw_rule;
   Rules are defined per port. Each redirector device
   has 4 ports (numbeder 0..3):
 
-         0   +--------------+   2   +--------------+
+  0   +--------------+   2   +--------------+
   LAN  <===> |              | <===> |   1/10G      |
-             |  Redirector  |       |   Ethernet   |
+  |  Redirector  |       |   Ethernet   |
   LAN  <===> |    Switch    | <===> |   Adapter    |
-         1   +--------------+   3   +--------------+
+  1   +--------------+   3   +--------------+
 
   Drop Rule
   Discard incoming packets matching the filter
@@ -583,27 +604,38 @@ typedef struct {
 } __attribute__((packed))
 accolade_hw_rule;
 
+typedef enum {
+  default_pass,
+  default_drop
+} generic_default_action_type;
+
 typedef enum {
   flow_drop_rule,
-  flow_mark_rule
+  flow_mark_rule,
+  flow_pass_rule,
+  flow_steer_rule
 } generic_flow_rule_action_type;
 
-typedef struct { 
+typedef struct {
   generic_flow_rule_action_type action;
   u_int32_t flow_id; /* flow id from flow metadata */
   u_int32_t thread; /* id of the thread setting the rule */
 } __attribute__((packed))
 generic_flow_id_hw_rule;
 
-typedef struct { 
+typedef struct {
   generic_flow_rule_action_type action;
   ip_addr src_ip;
   ip_addr dst_ip;
+  ip_addr src_ip_mask;
+  ip_addr dst_ip_mask;
   u_int16_t src_port;
   u_int16_t dst_port;
+  u_int16_t vlan_id;
   u_int8_t ip_version;
   u_int8_t protocol;
   u_int8_t interface; /* from extended_hdr.if_index */
+  u_int8_t queue_id; /* with action == flow_steer_rule */
 } __attribute__((packed))
 generic_flow_tuple_hw_rule;
 
@@ -619,15 +651,22 @@ typedef enum {
 
 typedef struct {
   hw_filtering_rule_type rule_family_type;
+
+  /* FILTERING_RULE_AUTO_RULE_ID to auto generate a rule ID
+   * Supported by Accolade and Mellanox */
   u_int16_t rule_id;
 
+  /* Rule priority (when supported by the adapter)
+   * Supported by Mellanox (0..2) */
+  u_int16_t priority;
+
   union {
-    intel_82599_five_tuple_filter_hw_rule five_tuple_rule;
-    intel_82599_perfect_filter_hw_rule perfect_rule;
-    silicom_redirector_hw_rule redirector_rule;
+    intel_82599_five_tuple_filter_hw_rule five_tuple_rule; /* Intel ixgbe */
+    intel_82599_perfect_filter_hw_rule perfect_rule; /* Intel ixgbe */
+    silicom_redirector_hw_rule redirector_rule; /* Silicom Redirector (Intel) */
     generic_flow_id_hw_rule flow_id_rule;
-    generic_flow_tuple_hw_rule flow_tuple_rule;
-    accolade_hw_rule accolade_rule;
+    generic_flow_tuple_hw_rule flow_tuple_rule; /* Mellanox */
+    accolade_hw_rule accolade_rule; /* Accolade */
   } rule_family;
 } __attribute__((packed))
 hw_filtering_rule;
@@ -643,23 +682,23 @@ hw_filtering_rule;
 #if defined(I82599_HW_FILTERING_SUPPORT) && (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,40))
 #define	FLOW_EXT 0x80000000
 union _kcompat_ethtool_flow_union {
-	struct ethtool_tcpip4_spec		tcp_ip4_spec;
-	struct ethtool_usrip4_spec		usr_ip4_spec;
-	__u8					hdata[60];
+  struct ethtool_tcpip4_spec		tcp_ip4_spec;
+  struct ethtool_usrip4_spec		usr_ip4_spec;
+  __u8					hdata[60];
 };
 struct _kcompat_ethtool_flow_ext {
-	__be16	vlan_etype;
-	__be16	vlan_tci;
-	__be32	data[2];
+  __be16	vlan_etype;
+  __be16	vlan_tci;
+  __be32	data[2];
 };
 struct _kcompat_ethtool_rx_flow_spec {
-	__u32		flow_type;
-	union _kcompat_ethtool_flow_union h_u;
-	struct _kcompat_ethtool_flow_ext h_ext;
-	union _kcompat_ethtool_flow_union m_u;
-	struct _kcompat_ethtool_flow_ext m_ext;
-	__u64		ring_cookie;
-	__u32		location;
+  __u32		flow_type;
+  union _kcompat_ethtool_flow_union h_u;
+  struct _kcompat_ethtool_flow_ext h_ext;
+  union _kcompat_ethtool_flow_union m_u;
+  struct _kcompat_ethtool_flow_ext m_ext;
+  __u64		ring_cookie;
+  __u32		location;
 };
 #define ethtool_rx_flow_spec _kcompat_ethtool_rx_flow_spec
 #endif /* defined(I82599_HW_FILTERING_SUPPORT) && (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,40)) */
@@ -678,7 +717,7 @@ struct pfring_timespec {
   u_int32_t tv_nsec;
 } __attribute__((packed));
 
-typedef struct { 
+typedef struct {
   u_int32_t flow_id;
 
   u_int8_t ip_version;
@@ -701,7 +740,7 @@ typedef struct {
   u_int32_t rev_packets;
   u_int64_t fwd_bytes;
   u_int64_t rev_bytes;
-  
+
   struct pfring_timespec fwd_ts_first;
   struct pfring_timespec fwd_ts_last;
   struct pfring_timespec rev_ts_first;
@@ -850,11 +889,30 @@ struct ring_sock {
 
 /* *********************************** */
 
-typedef int (*zc_dev_wait_packet)(void *adapter, int mode);
-typedef int (*zc_dev_notify)(void *rx_adapter_ptr, void *tx_adapter_ptr, u_int8_t device_in_use);
+/* ZC driver API - data structures */
+
+typedef int (*zc_dev_wait_packet)(void *rx_adapter, int mode);
+typedef int (*zc_dev_notify)(void *rx_adapter, void *tx_adapter, u_int8_t device_in_use);
+typedef int (*zc_dev_set_time)(void *rx_adapter, u_int64_t time_ns);
+typedef int (*zc_dev_adjust_time)(void *rx_adapter, int64_t offset_ns);
+typedef int (*zc_dev_get_tx_time)(void *tx_adapter, u_int64_t *time_ns);
+typedef int (*zc_dev_control_queue)(void *rx_adapter, u_int8_t enable);
+typedef int (*zc_dev_get_stats)(void *rx_adapter, u_int64_t *rx_missed);
+
+typedef struct {
+  zc_dev_wait_packet wait_packet;
+  zc_dev_notify usage_notification;
+  zc_dev_set_time set_time;
+  zc_dev_adjust_time adjust_time;
+  zc_dev_get_tx_time get_tx_time;
+  zc_dev_control_queue control_queue;
+  zc_dev_get_stats get_stats;
+} __attribute__((packed))
+zc_dev_callbacks;
 
 typedef enum {
-  add_device_mapping = 0, remove_device_mapping
+  add_device_mapping = 0,
+  remove_device_mapping
 } zc_dev_operation;
 
 /* IMPORTANT NOTE
@@ -870,10 +928,10 @@ typedef enum {
   intel_e1000,
   intel_ixgbe_82599_ts,
   intel_i40e,
-  intel_fm10k,
   intel_ixgbe_vf,
   intel_ixgbe_x550,
-  intel_ice
+  intel_ice,
+  intel_i40e_vf
 } zc_dev_model;
 
 typedef struct {
@@ -885,11 +943,13 @@ typedef struct {
   u_int32_t vector;
   u_int32_t num_queues;
 } __attribute__((packed))
-mem_ring_info;
+zc_dev_ring_info;
+
+/* ************************************************* */
 
 typedef struct {
-  mem_ring_info rx;
-  mem_ring_info tx;
+  zc_dev_ring_info rx;
+  zc_dev_ring_info tx;
   u_int32_t phys_card_memory_len;
   zc_dev_model device_model;
 } __attribute__((packed))
@@ -909,10 +969,11 @@ typedef struct {
 #else
   void *packet_waitqueue;
 #endif
-  u_int8_t *interrupt_received, in_use;
-  void *rx_adapter_ptr, *tx_adapter_ptr;
-  zc_dev_wait_packet wait_packet_function_ptr;
-  zc_dev_notify usage_notification;
+  u_int8_t *interrupt_received;
+  u_int8_t in_use;
+  void *rx_adapter;
+  void *tx_adapter;
+  zc_dev_callbacks callbacks;
 } __attribute__((packed))
 zc_dev_info;
 
@@ -1049,18 +1110,21 @@ typedef struct {
   zc_dev_info zc_dev;
   struct list_head list;
   /*
-    In the ZC world only one application can open and enable the 
+    In the ZC world only one application can open and enable the
     device@channel per direction. The array below is used to keep
     pointers to the sockets bound to device@channel.
     No more than one socket can be enabled for RX and one for TX.
   */
   struct pf_ring_socket *bound_sockets[MAX_NUM_ZC_BOUND_SOCKETS];
-  rwlock_t lock;
+  spinlock_t lock;
 } zc_dev_list;
 
 #define MAX_NUM_IFINDEX 0x7FFFFFFF
 #define MAX_NUM_DEV_IDX 1024
 
+#define ANY_IFINDEX  (MAX_NUM_IFINDEX-1)
+#define NONE_IFINDEX (MAX_NUM_IFINDEX-2)
+
 /*
  * Linked-list of virtual filtering devices
  */
@@ -1119,7 +1183,7 @@ struct dma_memory_info {
   u_int32_t num_chunks, chunk_len;
   u_int32_t num_slots,  slot_len;
   unsigned long *virtual_addr;  /* chunks pointers */
-  u_int64_t     *dma_addr;      /* per-slot DMA adresses */
+  u_int64_t     *dma_addr;      /* per-slot DMA addresses */
   struct device *hwdev;         /* dev for DMA mapping */
 };
 
@@ -1172,14 +1236,14 @@ struct hash_fragment_node {
  * Ring options
  */
 struct pf_ring_socket {
-  rwlock_t ring_config_lock;
+  struct mutex ring_config_lock;
 
   u_int8_t ring_active, ring_shutdown, num_rx_channels, num_bound_devices;
   pf_ring_device *ring_dev;
 
   /* last device set with bind, needed to heck channels when multiple
    * devices are used with quick-mode */
-  pf_ring_device *last_bind_dev; 
+  pf_ring_device *last_bind_dev;
 
   DECLARE_BITMAP(pf_dev_mask, MAX_NUM_DEV_IDX /* bits */);
   int ring_pid;
@@ -1215,7 +1279,7 @@ struct pf_ring_socket {
      from user space */
   struct {
     u_int8_t enable_tx_with_bounce;
-    rwlock_t consume_tx_packets_lock;
+    spinlock_t consume_tx_packets_lock;
     int32_t last_tx_dev_idx;
     struct net_device *last_tx_dev;
   } tx;
@@ -1281,7 +1345,8 @@ struct pf_ring_socket {
   /* Locks */
   atomic_t num_ring_users;
   wait_queue_head_t ring_slots_waitqueue;
-  rwlock_t ring_index_lock, ring_rules_lock;
+  spinlock_t ring_index_lock;
+  rwlock_t ring_rules_lock;
 
   /* Indexes (Internal) */
   u_int32_t insert_page_id, insert_slot_id;
@@ -1302,10 +1367,18 @@ typedef struct {
   /* /proc entry for ring module */
   struct proc_dir_entry *proc;
   struct proc_dir_entry *proc_dir;
-  struct proc_dir_entry *proc_dev_dir; 
+  struct proc_dir_entry *proc_dev_dir;
   struct proc_dir_entry *proc_stats_dir;
 
-  struct list_head list;
+  /* Map ifindex to pf device idx (used for quick_mode_rings, num_rings_per_device) */
+  ifindex_map_item ifindex_map[MAX_NUM_DEV_IDX];
+
+  /* quick mode <ifindex, channel> to <ring> table */
+  struct pf_ring_socket* quick_mode_rings[MAX_NUM_DEV_IDX][MAX_NUM_RX_CHANNELS];
+
+  /* Keep track of number of rings per device (plus any) */
+  u_int8_t num_rings_per_device[MAX_NUM_DEV_IDX];
+  u_int8_t num_any_rings;
 } pf_ring_net;
 
 /* **************************************** */
@@ -1336,23 +1409,25 @@ int pf_ring_skb_ring_handler(struct sk_buff *skb,
 			     int32_t channel_id,
 			     u_int32_t num_rx_channels);
 
+/* ZC driver API */
+
 void pf_ring_zc_dev_handler(zc_dev_operation operation,
-			    mem_ring_info *rx_info,
-			    mem_ring_info *tx_info,
-			    void          *rx_descr_packet_memory,
-			    void          *tx_descr_packet_memory,
-			    void          *phys_card_memory,
-			    u_int          phys_card_memory_len,
-			    u_int channel_id,
+			    zc_dev_callbacks *callbacks,
+			    zc_dev_ring_info *rx_info,
+			    zc_dev_ring_info *tx_info,
+			    void *rx_descr_packet_memory,
+			    void *tx_descr_packet_memory,
+			    void *phys_card_memory,
+			    u_int32_t phys_card_memory_len,
+			    u_int32_t channel_id,
 			    struct net_device *dev,
 			    struct device *hwdev,
 			    zc_dev_model device_model,
 			    u_char *device_address,
 			    wait_queue_head_t *packet_waitqueue,
 			    u_int8_t *interrupt_received,
-			    void *rx_adapter_ptr, void *tx_adapter_ptr,
-			    zc_dev_wait_packet wait_packet_function_ptr,
-			    zc_dev_notify dev_notify_function_ptr);
+			    void *rx_adapter_ptr,
+			    void *tx_adapter_ptr);
 
 #endif /* __KERNEL__  */
 
