@@ -1,5 +1,5 @@
 /*
- * (C) 2003-2021 - ntop 
+ * (C) 2003-23 - ntop 
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -59,6 +59,8 @@
 #define DEFAULT_DEVICE     "eth0"
 #define NO_ZC_BUFFER_LEN     9000
 #define BURST_SIZE             32
+#define USE_PF_RING_LOOP_API
+//#define USE_PF_RING_POLL_API
 
 pfring  *pd;
 int verbose = 0, quiet = 0, num_threads = 1;
@@ -70,11 +72,13 @@ u_int string_id = 1;
 char *out_pcap_file = NULL;
 FILE *match_dumper = NULL;
 u_int8_t do_close_dump = 0, is_sysdig = 0, chunk_mode = 0, check_ts = 0;
-u_int8_t check_seq_ip = 0, asymm_rss = 0, burst_mode = 0;
+u_int8_t check_seq_ip = 0, asymm_rss = 0, burst_mode = 0, rss_q_0_only = 0;
 int num_packets = 0;
 time_t last_ts = 0;
 u_int32_t last_ip = 0;
 u_int16_t min_len = 0;
+int promisc = 1;
+u_int8_t rule_priority = 0;
 
 struct app_stats {
   u_int64_t numPkts[MAX_NUM_THREADS];
@@ -107,14 +111,14 @@ static void dumpMatch(char *str);
 void print_stats() {
   pfring_stat pfringStat;
   struct timeval endTime;
-  double deltaMillisec;
+  double delta_last;
   static u_int8_t print_all;
   static u_int64_t lastPkts = 0;
   static u_int64_t lastBytes = 0;
   double diff, bytesDiff;
   static struct timeval lastTime;
   char buf[256], buf1[64], buf2[64], buf3[64], buf4[64], timebuf[128];
-  u_int64_t deltaMillisecStart;
+  u_int64_t delta_abs;
 
   if(startTime.tv_sec == 0) {
     gettimeofday(&startTime, NULL);
@@ -123,7 +127,7 @@ void print_stats() {
     print_all = 1;
 
   gettimeofday(&endTime, NULL);
-  deltaMillisec = delta_time(&endTime, &startTime);
+  delta_last = delta_time(&endTime, &startTime);
 
   if(pfring_stats(pd, &pfringStat) >= 0) {
     double thpt;
@@ -136,19 +140,19 @@ void print_stats() {
       nMatches += stats->numStringMatches[i];
     }
 
-    deltaMillisecStart = delta_time(&endTime, &startTime);
+    delta_abs = delta_time(&endTime, &startTime);
     snprintf(buf, sizeof(buf),
              "Duration: %s\n"
              "Packets:  %lu\n"
              "Dropped:  %lu\n"
              "Bytes:    %lu\n",
-             msec2dhmsm(deltaMillisecStart, timebuf, sizeof(timebuf)),
+             msec2dhmsm(delta_abs, timebuf, sizeof(timebuf)),
              (long unsigned int) nPkts,
              (long unsigned int) pfringStat.drop,
              (long unsigned int) nBytes);
     pfring_set_application_stats(pd, buf);
 
-    thpt = ((double)8*nBytes)/(deltaMillisec*1000);
+    thpt = ((double)8*nBytes)/(delta_last*1000);
 
     fprintf(stderr, "=========================\n"
 	    "Absolute Stats: [%s pkts total][%s pkts dropped][%.1f%% dropped]\n",
@@ -163,7 +167,7 @@ void print_stats() {
 
     if(print_all)
       fprintf(stderr, "[%s %s/sec][%s Mbit/sec]\n",
-	      pfring_format_numbers((double)(nPkts*1000)/deltaMillisec, buf1, sizeof(buf1), 1),
+	      pfring_format_numbers((double)(nPkts*1000)/delta_last, buf1, sizeof(buf1), 1),
               chunk_mode == 1 ? "chunk" : "pkt",
 	      pfring_format_numbers(thpt, buf2, sizeof(buf2), 1));
     else
@@ -173,7 +177,7 @@ void print_stats() {
       fprintf(stderr, "String matched: %llu\n", nMatches);
 
     if(print_all && (lastTime.tv_sec > 0)) {
-      deltaMillisec = delta_time(&endTime, &lastTime);
+      delta_last = delta_time(&endTime, &lastTime);
       diff = nPkts-lastPkts;
       bytesDiff = nBytes - lastBytes;
       bytesDiff /= (1000*1000*1000)/8;
@@ -182,10 +186,10 @@ void print_stats() {
 	      "Actual Stats: [%s %s rcvd][%s ms][%s %s][%s Gbps]",
 	      pfring_format_numbers(diff, buf4, sizeof(buf4), 0),
               chunk_mode == 1 ? "chunks" : "pkts",
-	      pfring_format_numbers(deltaMillisec, buf1, sizeof(buf1), 1),
-	      pfring_format_numbers(((double)diff/(double)(deltaMillisec/1000)),  buf2, sizeof(buf2), 1),
+	      pfring_format_numbers(delta_last, buf1, sizeof(buf1), 1),
+	      pfring_format_numbers(((double)diff/(double)(delta_last/1000)),  buf2, sizeof(buf2), 1),
               chunk_mode == 1 ? "cps" : "pps",
-	      pfring_format_numbers(((double)bytesDiff/(double)(deltaMillisec/1000)),  buf3, sizeof(buf3), 1));
+	      pfring_format_numbers(((double)bytesDiff/(double)(delta_last/1000)),  buf3, sizeof(buf3), 1));
 
       fprintf(stderr, "=========================\n%s\n", buf);
     }
@@ -366,6 +370,73 @@ void sample_filtering_rules(){
     else
       printf("Rule %d added successfully...\n", r.rule_id );
   }
+
+  if (0) { /* Mellanox (Drop Src IP) */
+
+    /* Use promisc to guess what is the default the user wants */
+    generic_default_action_type default_action = promisc ? default_pass : default_drop;
+
+    /* Drop (or Pass if default is drop) UDP Src IP */
+
+    pfring_set_default_hw_action(pd, default_action);
+
+    hw_filtering_rule r = { 0 };
+
+    r.priority = rule_priority; /* Rule priority (0..2) */
+    r.rule_id = FILTERING_RULE_AUTO_RULE_ID; /* auto generate rule ID */
+    r.rule_family_type = generic_flow_tuple_rule;
+
+    if (default_action == default_pass)
+      r.rule_family.flow_tuple_rule.action = flow_drop_rule;
+    else
+      r.rule_family.flow_tuple_rule.action = flow_pass_rule;
+
+    r.rule_family.flow_tuple_rule.ip_version = 4;
+    r.rule_family.flow_tuple_rule.src_ip.v4 = ntohl(inet_addr("10.0.0.1"));
+
+    r.rule_family.flow_tuple_rule.protocol = IPPROTO_UDP;
+    //r.rule_family.flow_tuple_rule.dst_port = 3000;
+
+    if ((rc = pfring_add_hw_rule(pd, &r)) < 0)
+      fprintf(stderr, "pfring_add_hw_rule(id=%d) failed: rc=%d\n", r.rule_id, rc);
+    else
+      printf("Rule %d added successfully...\n", r.rule_id );
+  }
+
+  if (0) { /* Mellanox (2 Rules) */
+    /* Drop (or Pass if promisc is not set) TCP Dst 46.101.88.215:143 and TCP Dst 17.167.192.126:443 */
+
+    hw_filtering_rule r = { 0 };
+
+    r.priority = rule_priority; /* Rule priority (0..2) */
+    r.rule_family_type = generic_flow_tuple_rule;
+
+    if (promisc)
+      r.rule_family.flow_tuple_rule.action = flow_drop_rule;
+    else
+      r.rule_family.flow_tuple_rule.action = flow_pass_rule;
+
+    r.rule_family.flow_tuple_rule.ip_version = 4;
+    r.rule_family.flow_tuple_rule.protocol = IPPROTO_TCP;
+
+    r.rule_id = FILTERING_RULE_AUTO_RULE_ID; /* auto generate rule ID */
+    r.rule_family.flow_tuple_rule.dst_ip.v4 = ntohl(inet_addr("46.101.88.215"));
+    r.rule_family.flow_tuple_rule.dst_port = 143;
+
+    if ((rc = pfring_add_hw_rule(pd, &r)) < 0)
+      fprintf(stderr, "pfring_add_hw_rule(id=%d) failed: rc=%d\n", r.rule_id, rc);
+    else
+      printf("Rule %d added successfully...\n", r.rule_id );
+
+    r.rule_id = FILTERING_RULE_AUTO_RULE_ID; /* auto generate rule ID */
+    r.rule_family.flow_tuple_rule.dst_ip.v4 = ntohl(inet_addr("17.167.192.126"));
+    r.rule_family.flow_tuple_rule.dst_port = 443;
+
+    if ((rc = pfring_add_hw_rule(pd, &r)) < 0)
+      fprintf(stderr, "pfring_add_hw_rule(id=%d) failed: rc=%d\n", r.rule_id, rc);
+    else
+      printf("Rule %d added successfully...\n", r.rule_id );
+  }
 }
 
 /* ******************************** */
@@ -493,6 +564,28 @@ static int search_string(char *string_to_match, u_int string_to_match_len) {
 
 /* ****************************************************** */
 
+char* get_process_path(u_int32_t pid, char *pbuf, u_int pbuf_len) {
+  char *space, process_path[256];
+  int len;
+
+  pbuf[0] = '\0';
+  
+  if(pid == 0) return(pbuf);
+  
+  sprintf(process_path, "/proc/%d/exe", pid);
+  if((len = readlink(process_path, pbuf, pbuf_len)) > 0)
+    pbuf[len] = '\0';
+  
+  if((space = strchr(pbuf, ' ')) != NULL) {
+    if(space[1] == '(') /* (deleted) */
+      space[0] = '\0';
+  }
+  
+  return(pbuf);
+}
+
+/* ****************************************************** */
+
 static int32_t thiszone;
 
 void print_packet(const struct pfring_pkthdr *h, const u_char *p, u_int8_t dump_match) {
@@ -528,15 +621,20 @@ void print_packet(const struct pfring_pkthdr *h, const u_char *p, u_int8_t dump_
   }
 
   if(use_extended_pkt_header) {
-    char bigbuf[4096];
+    char bigbuf[4096], pbuf[64];;
     u_int len;
 
     snprintf(&dump_str[strlen(dump_str)], sizeof(dump_str)-strlen(dump_str), "%s[if_index=%d][hash=%u]%s",
-      h->extended_hdr.rx_direction ? "[RX]" : "[TX]",
-      h->extended_hdr.if_index,
-      h->extended_hdr.pkt_hash,
-      (h->extended_hdr.flags & PKT_FLAGS_FLOW_OFFLOAD_MARKER) ? "[MARKED]" : "");
+	     h->extended_hdr.rx_direction ? "[RX]" : "[TX]",
+	     h->extended_hdr.if_index,
+	     h->extended_hdr.pkt_hash,
+	     (h->extended_hdr.flags & PKT_FLAGS_FLOW_OFFLOAD_MARKER) ? "[MARKED]" : "");
 
+    if (h->extended_hdr.process.pid)
+      snprintf(&dump_str[strlen(dump_str)], sizeof(dump_str)-strlen(dump_str), "[pid=%u (%s)]",
+	       h->extended_hdr.process.pid,
+	       get_process_path(h->extended_hdr.process.pid, pbuf, sizeof(pbuf)));
+    
     pfring_print_parsed_pkt(bigbuf, sizeof(bigbuf), p, h);
     len = strlen(bigbuf);
 
@@ -559,7 +657,7 @@ void print_packet(const struct pfring_pkthdr *h, const u_char *p, u_int8_t dump_
   if(unlikely(dump_match)) {
     /* I need to find out which string matched */
     struct strmatch *m = matching_strings;
-    char *_payload = (char*)&p[42], payload[1500];
+    char *_payload = (char*)&p[42], payload[9000];
     u_int payload_len = h->caplen-42, i;
 
     if(payload_len > sizeof(payload)) payload_len = sizeof(payload);
@@ -662,44 +760,67 @@ void dummyProcessPacket(const struct pfring_pkthdr *h,
 
 /* *************************************** */
 
-void printDevs() {
+void printDevs(u_int8_t json) {
   pfring_if_t *dev;
   int i = 0;
 
   dev = pfring_findalldevs();
 
-  if (verbose)
-    printf("Name\tSystemName\tModule\tMAC\tBusID\tNumaNode\tStatus\tLicense\tExpiration\n");
+  if (json)
+    printf("{\"interfaces\":[");
+  else if (verbose)
+    printf("Name\tSystemName\tModule\tMAC\tBusID\tNumaNode\tStatus\tLicense\tExpiration\nModuleVersion\n");
   else
     printf("Available devices (-i):\n");
 
   while (dev != NULL) {
-    if (verbose) {
-      printf("%s\t%s\t%s\t", 
-        dev->name, dev->system_name ? dev->system_name : "unknown", dev->module);
+    char mac[20];
 
-      if (dev->sn)
-        printf("%s", dev->sn);
-      else
-        printf("%02X:%02X:%02X:%02X:%02X:%02X", 
+    if (!dev->sn)
+      sprintf(mac, "%02X:%02X:%02X:%02X:%02X:%02X", 
           dev->mac[0] & 0xFF, dev->mac[1] & 0xFF, dev->mac[2] & 0xFF, 
           dev->mac[3] & 0xFF, dev->mac[4] & 0xFF, dev->mac[5] & 0xFF);
 
-      printf("\t%04X:%02X:%02X.%X\t%d\t%s\t%s\t%ld\n",
+    if (json) {
+      printf("{");
+      printf("\"name\":\"%s\",", dev->name);
+      printf("\"system_name\":\"%s\",", dev->system_name ? dev->system_name : "unknown");
+      printf("\"module\":\"%s\",", dev->module);
+      printf("\"sn\":\"%s\",", dev->sn ? dev->sn : mac);
+      printf("\"bus_id\":\"%04X:%02X:%02X.%X\",", dev->bus_id.slot, dev->bus_id.bus, dev->bus_id.device, dev->bus_id.function);
+      printf("\"numa_node\":\"%d\",", busid2node(dev->bus_id.slot, dev->bus_id.bus, dev->bus_id.device, dev->bus_id.function));
+      printf("\"status\":\"%s\",", dev->status > 0 ? "Up" : (dev->status == 0 ? "Down" : "Unknown"));
+      printf("\"license\":\"%s\",", dev->license ? "Valid" : "NotFound");
+      printf("\"expiration\":\"%ld\",", dev->license_expiration);
+      printf("\"mod_version\":\"%s\"", dev->module_version ? dev->module_version : "");
+      printf("}");
+      if (dev->next)
+        printf(",");
+
+    } else if (verbose) {
+      printf("%s\t%s\t%s\t%s\t%04X:%02X:%02X.%X\t%d\t%s\t%s\t%ld\t%s\n",
+        dev->name, dev->system_name ? dev->system_name : "unknown", dev->module,
+        dev->sn ? dev->sn : mac,
         dev->bus_id.slot, dev->bus_id.bus, dev->bus_id.device, dev->bus_id.function,
         busid2node(dev->bus_id.slot, dev->bus_id.bus, dev->bus_id.device, dev->bus_id.function),
-        dev->status ? "Up" : "Down", dev->license ? "Valid" : "NotFound", dev->license_expiration);
+        dev->status > 0 ? "Up" : (dev->status == 0 ? "Down" : "Unknown"), dev->license ? "Valid" : "NotFound", dev->license_expiration,
+        dev->module_version ? dev->module_version : "");
+
     } else {
       printf(" %d. %s\n", i++, dev->name);
     }
+
     dev = dev->next;
   }
+
+  if (json)
+    printf("]}");
 }
 
 /* *************************************** */
 
 void printHelp(void) {
-  printf("pfcount - (C) 2005-2021 ntop.org\n\n");
+  printf("pfcount - (C) 2005-23 ntop\n\n");
   printf("-h              Print this help\n");
   printf("-i <device>     Device name. Use:\n"
 	 "                - ethX@Y for channels\n"
@@ -761,6 +882,8 @@ void printHelp(void) {
   printf("-u <1|2>        For each incoming packet add a drop rule (1=hash, 2=wildcard rule)\n");
   printf("-J              Do not enable promiscuous mode\n");
   printf("-R              Do not reprogram RSS indirection table (Intel ZC only)\n");
+  printf("-0              Send all traffic to RSS queue 0 (this also enabled -R)\n");
+  printf("-P <prio>       Set hw filter priority (0..2)\n");
   printf("-v <mode>       Verbose [1: verbose, 2: very verbose (print packet payload)]\n");
   printf("-K <len>        Print only packets with length > <len> with -v\n");
   printf("-z <mode>       Enabled hw timestamping/stripping. Currently the supported TS mode are:\n"
@@ -768,6 +891,7 @@ void printHelp(void) {
          "                metawatch\tTimestamped packets by Arista 7130 MetaWatch series devices\n"
          "                arista\tTimestamped packets by Arista 7150 series devices\n");
   printf("-L              List all interfaces and exit (use -v for more info)\n");
+  printf("-I              Output system information (interfaces) in JSON format\n");
 }
 
 /* *************************************** */
@@ -777,12 +901,9 @@ void* packet_consumer_thread(void* _id) {
   u_int numCPU = sysconf( _SC_NPROCESSORS_ONLN );
   u_char buffer[NO_ZC_BUFFER_LEN];
   u_char *buffer_p = buffer;
-
   u_long core_id = thread_id % numCPU;
   struct pfring_pkthdr hdr;
 
-  /* printf("packet_consumer_thread(%lu)\n", thread_id); */
-
   if((num_threads > 1) && (numCPU > 1)) {
     if(bind2core(core_id) == 0)
       if (!quiet)
@@ -791,41 +912,26 @@ void* packet_consumer_thread(void* _id) {
 
   memset(&hdr, 0, sizeof(hdr));
 
-  while(1) {
-    int rc;
-    u_int len;
+  while(!stats->do_shutdown) {
 
-    if(stats->do_shutdown) break;
-
-    if((rc = pfring_recv(pd, &buffer_p, NO_ZC_BUFFER_LEN, &hdr, wait_for_packet)) > 0) {
-      if(stats->do_shutdown) break;
-      dummyProcessPacket(&hdr, buffer, (u_char*)thread_id);
-#ifdef TEST_SEND
-      buffer[0] = 0x99;
-      buffer[1] = 0x98;
-      buffer[2] = 0x97;
-      pfring_send(pd, buffer, hdr.caplen);
+#ifdef USE_PF_RING_POLL_API
+    if(pfring_recv(pd, &buffer_p, NO_ZC_BUFFER_LEN, &hdr, 0) > 0) {
+#else
+    if(pfring_recv(pd, &buffer_p, NO_ZC_BUFFER_LEN, &hdr, wait_for_packet) > 0) {
 #endif
+
+      dummyProcessPacket(&hdr, buffer, (u_char*)thread_id);
+
     } else {
-      if(wait_for_packet == 0) sched_yield();
-    }
+      if (stats->do_shutdown)
+        break;
 
-    if(0) {
-      struct simple_stats {
-	u_int64_t num_pkts, num_bytes;
-      };
-      struct simple_stats stats;
-
-      len = sizeof(stats);
-      rc = pfring_get_filtering_rule_stats(pd, 5, (char*)&stats, &len);
-      if(rc < 0)
-	fprintf(stderr, "pfring_get_filtering_rule_stats() failed [rc=%d]\n", rc);
-      else {
-        if (!quiet)
-	  printf("[Pkts=%u][Bytes=%u]\n",
-	         (unsigned int)stats.num_pkts,
-	         (unsigned int)stats.num_bytes);
-      }
+      if(wait_for_packet == 0)
+        sched_yield();
+#ifdef USE_PF_RING_POLL_API
+      else
+        pfring_poll(pd, 1 /* msec */);
+#endif
     }
   }
 
@@ -885,12 +991,9 @@ void* chunk_consumer_thread(void* _id) {
   u_int numCPU = sysconf( _SC_NPROCESSORS_ONLN );
   void *chunk_p = NULL;
   pfring_chunk_info chunk_info;
-
   u_long core_id = thread_id % numCPU;
   struct pfring_pkthdr hdr;
 
-  /* printf("packet_consumer_thread(%lu)\n", thread_id); */
-
   if((num_threads > 1) && (numCPU > 1)) {
     if(bind2core(core_id) == 0)
       if (!quiet)
@@ -1033,11 +1136,10 @@ void handleSigHup(int signalId) {
 
 int main(int argc, char* argv[]) {
   char *device = NULL, c, buf[32], path[256] = { 0 }, *reflector_device = NULL;
-  u_char mac_address[6] = { 0 };
-  int promisc = 1, snaplen = DEFAULT_SNAPLEN, rc;
+  int snaplen = DEFAULT_SNAPLEN, rc;
   u_int clusterId = 0;
   u_int8_t enable_ixia_timestamp = 0, enable_arista_timestamp = 0, enable_metawatch_timestamp = 0;
-  u_int8_t list_interfaces = 0;
+  u_int8_t list_interfaces = 0, json_info = 0;
   u_int32_t flags = 0;
   int bind_core = -1;
   packet_direction direction = rx_and_tx_direction;
@@ -1049,7 +1151,7 @@ int main(int argc, char* argv[]) {
   startTime.tv_sec = 0;
   thiszone = gmt_to_local(0);
 
-  while((c = getopt(argc,argv,"Bhi:c:C:Fd:H:Jl:Lv:ae:n:w:o:p:qb:rg:u:mtsSx:f:z:N:MRTUK:")) != '?') {
+  while((c = getopt(argc,argv,"Bhi:Ic:C:Fd:H:Jl:Lv:ae:n:w:o:p:P:qb:rg:u:mtsSx:f:z:N:MRTUK:0")) != '?') {
     if((c == 255) || (c == -1)) break;
 
     switch(c) {
@@ -1107,6 +1209,9 @@ int main(int argc, char* argv[]) {
       device = strdup(optarg);
       if(strcmp(device, "sysdig:") == 0) is_sysdig = 1;
       break;
+    case 'I':
+      json_info = 1;
+      break;
     case 'J':
       promisc = 0;
       break;
@@ -1135,6 +1240,9 @@ int main(int argc, char* argv[]) {
     case 'p':
       poll_duration = atoi(optarg);
       break;
+    case 'P':
+      rule_priority = atoi(optarg);
+      break;
     case 'q':
       is_sysdig = 1;
       break;
@@ -1204,11 +1312,14 @@ int main(int argc, char* argv[]) {
     case 'R':
       asymm_rss = 1;
       break;
+    case '0':
+      rss_q_0_only = 1;
+      break;
     }
   }
 
-  if (list_interfaces) {
-    printDevs();
+  if (list_interfaces || json_info) {
+    printDevs(json_info);
     exit(0);
   }
 
@@ -1259,6 +1370,7 @@ int main(int argc, char* argv[]) {
   if(enable_metawatch_timestamp) flags |= PF_RING_METAWATCH_TIMESTAMP;
   if(asymm_rss)               flags |= PF_RING_ZC_NOT_REPROGRAM_RSS;
   else                        flags |= PF_RING_ZC_SYMMETRIC_RSS;  /* Note that symmetric RSS is ignored by non-ZC drivers */
+  if(rss_q_0_only)            flags |= PF_RING_ZC_NOT_REPROGRAM_RSS | PF_RING_ZC_FIXED_RSS_Q_0;
   /* flags |= PF_RING_FLOW_OFFLOAD | PF_RING_FLOW_OFFLOAD_NOUPDATES;  to receive FlowID on supported adapters*/
   /* flags |= PF_RING_USERSPACE_BPF; to force userspace BPF even with kernel capture  */
 
@@ -1281,24 +1393,6 @@ int main(int argc, char* argv[]) {
              version & 0x000000FF);
   }
 
-  if(is_sysdig) {
-    if (!quiet)
-      printf("Capturing from sysdig\n");
-  } else {
-    int ifindex = -1;
-    rc = pfring_get_bound_device_address(pd, mac_address);
-    pfring_get_bound_device_ifindex(pd, &ifindex);
-    if (!quiet)
-      printf("Capturing from %s [mac: %s][if_index: %d][speed: %uMb/s]\n",
-             device, rc == 0 ? etheraddr_string(mac_address, buf) : "unknown",
-             ifindex, pfring_get_interface_speed(pd));
-  }
-
-  if (!quiet) {
-    printf("# Device RX channels: %d\n", pfring_get_num_rx_channels(pd));
-    printf("# Polling threads:    %d\n", num_threads);
-  }
-
   if (enable_hw_timestamp) {
     struct timespec ltime;
     /* Setting current clock (please note that with standard driver this is not mandatory) */
@@ -1365,6 +1459,22 @@ int main(int argc, char* argv[]) {
     return(-1);
   }
 
+  if (!quiet) {
+    if(is_sysdig) {
+      printf("Capturing from sysdig\n");
+    } else {
+      int ifindex = -1;
+      u_char mac_address[6] = { 0 };
+      rc = pfring_get_bound_device_address(pd, mac_address);
+      pfring_get_bound_device_ifindex(pd, &ifindex);
+      printf("Capturing from %s [mac: %s][if_index: %d][speed: %uMb/s]\n",
+             device, rc == 0 ? etheraddr_string(mac_address, buf) : "unknown",
+             ifindex, pfring_get_interface_speed(pd));
+      printf("# Device RX channels: %d\n", pfring_get_num_rx_channels(pd));
+      printf("# Polling threads:    %d\n", num_threads);
+    }
+  }
+
   sample_filtering_rules();
 
   if(num_threads <= 1) {
@@ -1376,7 +1486,11 @@ int main(int argc, char* argv[]) {
     } else if (burst_mode && pd->recv_burst) {
       burst_consumer_thread(0);
     } else {
+#ifdef USE_PF_RING_LOOP_API 
       pfring_loop(pd, dummyProcessPacket, (u_char*)NULL, wait_for_packet);
+#else
+      packet_consumer_thread(0);
+#endif
     }
   } else {
     pthread_t my_thread;
