@@ -1,5 +1,5 @@
 /*
- * (C) 2003-2021 - ntop 
+ * (C) 2003-23 - ntop 
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -45,6 +45,7 @@
 
 #include "pfutils.c"
 
+#define DEFAULT_DEVICE     "eth0"
 #define ALARM_SLEEP             1
 #define DEFAULT_SNAPLEN       128
 #define MAX_NUM_THREADS        64
@@ -65,7 +66,6 @@ struct thread_stats {
 };
 
 int verbose = 0, num_channels = 1;
-pfring_stat pfringStats;
 
 struct timeval startTime;
 u_int8_t use_extended_pkt_header = 0, wait_for_packet = 1, do_shutdown = 0;
@@ -73,21 +73,23 @@ u_int numCPU;
 
 struct thread_stats *threads;
 
-#define DEFAULT_DEVICE     "eth0"
+u_int32_t src_ip_rule = 0;
+u_int8_t src_ip_rule_set = 0;
 
 /* ******************************** */
 
 void print_stats() {
   pfring_stat pfringStat;
   struct timeval endTime;
-  double deltaMillisec;
+  double delta_abs;
   static u_int64_t lastPkts[MAX_NUM_THREADS] = { 0 };
   u_int64_t diff;
   static struct timeval lastTime;
   int i;
-  unsigned long long nBytes = 0, nPkts = 0, pkt_dropped = 0;
-  unsigned long long nPktsLast = 0;
-  double pkt_thpt = 0, tot_thpt = 0, delta;
+  unsigned long long bytes_received = 0, pkt_received = 0, pkt_dropped = 0;
+  unsigned long long pkt_received_last = 0;
+  double pkt_thpt = 0, tot_thpt = 0, delta_last;
+  char buf1[64];
 
   if(startTime.tv_sec == 0) {
     gettimeofday(&startTime, NULL);
@@ -95,15 +97,14 @@ void print_stats() {
   }
 
   gettimeofday(&endTime, NULL);
-  deltaMillisec = delta_time(&endTime, &startTime);
-
-  delta = delta_time(&endTime, &lastTime);
+  delta_abs = delta_time(&endTime, &startTime);
+  delta_last = delta_time(&endTime, &lastTime);
 
   for(i=0; i < num_channels; i++) {
-    nBytes += threads[i].numBytes, nPkts += threads[i].numPkts;
+    bytes_received += threads[i].numBytes, pkt_received += threads[i].numPkts;
   
     if(pfring_stats(threads[i].ring, &pfringStat) >= 0) {
-      double thpt = ((double)8*threads[i].numBytes)/(deltaMillisec*1000);
+      double thpt = ((double)8*threads[i].numBytes)/(delta_abs*1000);
 
       fprintf(stderr, "=========================\n"
 	      "Absolute Stats: [channel=%d][%u pkts rcvd][%u pkts dropped]\n"
@@ -114,19 +115,22 @@ void print_stats() {
       fprintf(stderr, "%lu pkts - %lu bytes", 
 	      (long unsigned int)threads[i].numPkts,
 	      (long unsigned int)threads[i].numBytes);
-      fprintf(stderr, " [%.1f pkt/sec - %.2f Mbit/sec]\n", (double)(threads[i].numPkts*1000)/deltaMillisec, thpt);
+      fprintf(stderr, " [%s pps - %.2f Mbit/sec]\n",
+              pfring_format_numbers((double)(threads[i].numPkts*1000)/delta_abs, buf1, sizeof(buf1), 1),
+              thpt);
       pkt_dropped += pfringStat.drop;
 
       if(lastTime.tv_sec > 0) {
 	double pps;
 	
 	diff = threads[i].numPkts-lastPkts[i];
-	nPktsLast += diff;
+	pkt_received_last += diff;
 	tot_thpt += thpt;
-	pps = ((double)diff/(double)(delta/1000));
+	pps = ((double)diff/(double)(delta_last/1000));
 	fprintf(stderr, "=========================\n"
-		"Actual Stats: [channel=%d][%llu pkts][%.1f ms][%.1f pkt/sec]\n",
-		i, (long long unsigned int)diff, delta, pps);
+		"Actual Stats: [channel=%d][%llu pkts][%.1f ms][%s pps]\n",
+		i, (long long unsigned int)diff, delta_last,
+	        pfring_format_numbers(((double)diff/(double)(delta_last/1000)), buf1, sizeof(buf1), 1));
 	pkt_thpt += pps;
       }
 
@@ -137,8 +141,12 @@ void print_stats() {
   lastTime.tv_sec = endTime.tv_sec, lastTime.tv_usec = endTime.tv_usec;
 
   fprintf(stderr, "=========================\n");
-  fprintf(stderr, "Aggregate stats (all channels): [%.1f pkt/sec][%.2f Mbit/sec][%llu pkts dropped]\n", 
-	  (double)(nPktsLast*1000)/(double)delta, tot_thpt, pkt_dropped);
+  fprintf(stderr, "Aggregate stats (all channels): [%s pps][%.2f Mbit/sec][%llu pkts rcvd][%llu pkts dropped][%llu pkts total]\n", 
+	  pfring_format_numbers((double)(pkt_received_last*1000)/(double)delta_last, buf1, sizeof(buf1), 1), 
+          tot_thpt,
+          pkt_received, 
+          pkt_dropped,
+          pkt_received + pkt_dropped);
   fprintf(stderr, "=========================\n\n");
 }
 
@@ -153,14 +161,11 @@ void sigproc(int sig) {
   do_shutdown = 1;
   print_stats();
 
-  fprintf(stderr, "Shutting down sockets...\n");
   for(i=0; i<num_channels; i++) {
     threads[i].do_shutdown = 1;
+    fprintf(stderr, "Shutting down socket %d\n", i);
     pfring_shutdown(threads[i].ring);
-    printf("\t%d...\n", i);
   }
-
-  exit(0);
 }
 
 /* ******************************** */
@@ -176,13 +181,14 @@ void my_sigalarm(int sig) {
 /* *************************************** */
 
 void printHelp(void) {
-  printf("pfcount_multichannel\n(C) 2005-12 Deri Luca <deri@ntop.org>\n\n");
+  printf("pfcount_multichannel\n(C) 2005-23 ntop\n\n");
   printf("-h              Print this help\n");
   printf("-i <device>     Device name (No device@channel)\n");
 
   printf("-e <direction>  0=RX+TX, 1=RX only, 2=TX only\n");
+  printf("-P <0|1>        Enable (1 - default) or disable (0) promisc\n");
   printf("-l <len>        Capture length\n");
-  printf("-m              Long packet header (with PF_RING extensions)\n");
+  printf("-m              Print more metadata with -v (extended packet header)\n");
   printf("-w <watermark>  Watermark\n");
   printf("-p <poll wait>  Poll wait (msec)\n");
   printf("-b <cpu %%>      CPU pergentage priority (0-99)\n");
@@ -206,10 +212,8 @@ void print_packet(const struct pfring_pkthdr *h, const u_char *p, long threadId)
   char bigbuf[BUFSIZE]; // buf into which we spew prints
   int  buflen = 0;
 
-  if(h->ts.tv_sec == 0) {
-    memset((void*)&h->extended_hdr.parsed_pkt, 0, sizeof(struct pkt_parsing_info));
-    pfring_parse_pkt((u_char*)p, (struct pfring_pkthdr*)h, 5, 1, 1);
-  }
+  memset((void*)&h->extended_hdr.parsed_pkt, 0, sizeof(struct pkt_parsing_info));
+  pfring_parse_pkt((u_char*)p, (struct pfring_pkthdr*)h, 5, !h->ts.tv_sec, !h->extended_hdr.pkt_hash);
 
   s = (h->ts.tv_sec + thiszone) % 86400;
   nsec = h->extended_hdr.timestamp_ns % 1000;
@@ -294,65 +298,36 @@ void* packet_consumer_thread(void* _id) {
          dummyProcesssPacket(&hdr, buffer, (u_char*)thread_id);
 
       } else {
-         if(wait_for_packet == 0) 
-           usleep(1); //sched_yield();
+         //if(wait_for_packet == 0) 
+         //  usleep(1); //sched_yield();
       }
    }
 
    return(NULL);
 }
 
-int ethtool_set_flowdirector(int on) {
-   // @TODO: plop the ethtool ioctl code in here to set up flowdirector
-   // perfect tuple filters.
-   return 0;
-}
+/* *************************************** */
 
-int setup_steering(pfring* thering, const char* addr, int queue) {
-   int rc;
-   static int rule_id = 0; // @HACK!
-   hw_filtering_rule rule;
-   intel_82599_perfect_filter_hw_rule *perfect_rule;
+void sample_filtering_rules() {
+  int rc;
 
-   if (thering == 0 || addr == 0) {
-      errno = EINVAL;
-      return -1;
-   }
+  if (src_ip_rule_set) { /* Mellanox (Pass) */
+    /* Pass Src IP */
 
-   printf("### Perfect Rule Example ###\n");
+    hw_filtering_rule r = { 0 };
 
-   rc = pfring_set_filtering_mode(thering, hardware_only);
-   printf("pfring_set_filtering_mode: hardware_only %s(%d)\n",
-          (rc == 0) ? "SUCCEEDED" : "FAILED", rc );
+    r.rule_id = 0;
+    r.rule_family_type = generic_flow_tuple_rule;
+    r.rule_family.flow_tuple_rule.action = flow_pass_rule;
+    r.rule_family.flow_tuple_rule.ip_version = 4;
+    r.rule_family.flow_tuple_rule.protocol = 6;
+    r.rule_family.flow_tuple_rule.src_ip.v4 = src_ip_rule;
 
-   /*
-     NOTE:
-     - valid protocols: UDP or TCP
-   */
-   perfect_rule = &rule.rule_family.perfect_rule;
-
-   memset(&rule, 0, sizeof(rule));
-   rule.rule_family_type = intel_82599_perfect_filter_rule;
-
-   rule.rule_id = rule_id++;
-   perfect_rule->queue_id = queue;
-   perfect_rule->proto = IPPROTO_UDP;
-   perfect_rule->d_addr = ntohl(inet_addr(addr));
-
-   rc = pfring_add_hw_rule(thering, &rule);
-   
-   if(rc != 0)
-      printf("pfring_add_hw_rule(%d) failed [rc=%d]: "
-             "did you enable the FlowDirector "
-             "(ethtool -K ethX ntuple on)\n",
-             rule.rule_id, rc);
-   else
-      printf("pfring_add_hw_rule(%d) succeeded: "
-             "steering UDP traffic %s:* -> *\n",
-             rule.rule_id,
-             addr);
-
-   return rc;
+    if ((rc = pfring_add_hw_rule(threads[0].ring, &r)) < 0)
+      fprintf(stderr, "pfring_add_hw_rule(id=%d) failed: rc=%d\n", r.rule_id, rc);
+    else
+      printf("Rule %d added successfully...\n", r.rule_id );
+  }
 }
 
 /* *************************************** */
@@ -367,13 +342,14 @@ int main(int argc, char* argv[]) {
   u_int32_t flags = 0;
   pfring *ring[MAX_NUM_RX_CHANNELS];
   int threads_core_affinity[MAX_NUM_RX_CHANNELS];
+  int promisc = 1;
 
   memset(threads_core_affinity, -1, sizeof(threads_core_affinity));
   startTime.tv_sec = 0;
   thiszone = gmt_to_local(0);
   numCPU = sysconf( _SC_NPROCESSORS_ONLN );
 
-  while((c = getopt(argc,argv,"hi:l:mvae:w:b:rp:g:")) != -1) {
+  while((c = getopt(argc,argv,"hi:I:l:mvae:w:b:rp:P:g:")) != -1) {
     switch(c) {
     case 'h':
       printHelp();
@@ -397,11 +373,16 @@ int main(int argc, char* argv[]) {
     case 'i':
       device = strdup(optarg);
       break;
+    case 'I':
+      src_ip_rule = ntohl(inet_addr(optarg));
+      src_ip_rule_set = 1;
+      break;
     case 'm':
       use_extended_pkt_header = 1;
       break;
     case 'v':
       verbose = 1;
+      use_extended_pkt_header = 1;
       break;
     case 'w':
       watermark = atoi(optarg);
@@ -415,6 +396,9 @@ int main(int argc, char* argv[]) {
     case 'p':
       poll_duration = atoi(optarg);
       break;
+    case 'P':
+      promisc = atoi(optarg);
+      break;
     case 'g':
       bind_mask = strdup(optarg);
       break;
@@ -442,7 +426,8 @@ int main(int argc, char* argv[]) {
 
   printf("Capturing from %s\n", device);
 
-  flags |= PF_RING_PROMISC; /* hardcode: promisc=1 */
+  if (promisc)
+    flags |= PF_RING_PROMISC;
   flags |= PF_RING_ZC_SYMMETRIC_RSS;  /* Note that symmetric RSS is ignored by non-ZC drivers */
   if(use_extended_pkt_header) flags |= PF_RING_LONG_HEADER;
 
@@ -457,8 +442,7 @@ int main(int argc, char* argv[]) {
      printf("WARNING: Too many channels (%d), using %d channels\n", num_channels, MAX_NUM_THREADS);
      num_channels = MAX_NUM_THREADS;
   } else if (num_channels > numCPU) {
-     printf("WARNING: More channels (%d) than available cores (%d), using %d channels\n", num_channels, numCPU, numCPU);
-     num_channels = numCPU;
+     printf("WARNING: More channels (%d) than available cores (%d)\n", num_channels, numCPU);
   } else  {
     printf("Found %d channels\n", num_channels);
   }
@@ -496,10 +480,13 @@ int main(int argc, char* argv[]) {
        pfring_set_poll_duration(threads[i].ring, poll_duration);
 
     pfring_enable_ring(threads[i].ring);
-
-    pthread_create(&threads[i].pd_thread, NULL, packet_consumer_thread, (void*)i);
   }
 
+  sample_filtering_rules();
+
+  for(i=0; i<num_channels; i++)
+    pthread_create(&threads[i].pd_thread, NULL, packet_consumer_thread, (void*)i);
+
   if(cpu_percentage > 0) {
     if(cpu_percentage > 99) cpu_percentage = 99;
     pfring_config(cpu_percentage);
