@@ -1,5 +1,5 @@
 /*
- * (C) 2003-2021 - ntop 
+ * (C) 2003-23 - ntop 
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -43,16 +43,6 @@
 #include "pfring.h"
 #include "pfutils.c"
 
-#define MAX_PACKET_SIZE 16018
-
-struct packet {
-  u_int32_t id;
-  u_int16_t len;
-  u_int64_t ticks_from_beginning;
-  u_char *pkt;
-  struct packet *next;
-};
-
 struct ip_header {
 #if BYTE_ORDER == LITTLE_ENDIAN
   u_int32_t	ihl:4,		/* header length */
@@ -117,43 +107,6 @@ int daemon_mode = 0;
 
 /* *************************************** */
 
-int is_fd_ready(int fd) {
-  struct timeval timeout = {0};
-  fd_set fdset;
-  FD_ZERO(&fdset);
-  FD_SET(fd, &fdset);
-  return (select(fd+1, &fdset, NULL, NULL, &timeout) == 1);
-}
-
-int read_packet_hex(u_char *buf, int buf_len) {
-  int i = 0, d, bytes = 0;
-  char c;
-  char s[3] = {0};
-
-  if (!is_fd_ready(fileno(stdin)))
-    return 0;
-
-  while ((d = fgetc(stdin)) != EOF) {
-    if (d < 0) break;
-    c = (u_char) d;
-    if ((c >= '0' && c <= '9') 
-     || (c >= 'a' && c <= 'f')
-     || (c >= 'A' && c <= 'F')) {
-      s[i&0x1] = c;
-      if (i&0x1) {
-        bytes = (i+1)/2;
-        sscanf(s, "%2hhx", &buf[bytes-1]);
-	if (bytes == buf_len) break;
-      }
-      i++;
-    }
-  }
-
-  return bytes;
-}
-
-/* *************************************** */
-
 void print_stats() {
   double deltaMillisec, currentThpt, avgThpt, currentThptBits, currentThptBytes, avgThptBits, avgThptBytes;
   struct timeval now;
@@ -221,7 +174,7 @@ void sigproc(int sig) {
 /* *************************************** */
 
 void printHelp(void) {
-  printf("pfsend - (C) 2011-2021 ntop.org\n");
+  printf("pfsend - (C) 2011-23 ntop\n");
   printf("Replay synthetic traffic, or a pcap, or a packet in hex format from standard input.\n\n"); 
   printf("pfsend -i out_dev [-a] [-f <.pcap file>] [-g <core_id>] [-h]\n"
          "       [-l <length>] [-n <num>] "
@@ -255,7 +208,7 @@ void printHelp(void) {
   printf("-8 <num>        Send the same packets <num> times before moving to the next\n");
   printf("-A <num>        Add <num> different packets (e.g. -b) every second\n");
   printf("-O              On the fly reforging instead of preprocessing (-b)\n");
-  printf("-z              Randomize generated IPs sequence\n");
+  printf("-z              Randomize generated IPs sequence (requires -b)\n");
   printf("-o <num>        Offset for generated IPs (-b) or packets in pcap (-f)\n");
   printf("-W <ID>[,<ID>]  Forge VLAN packets with the specified VLAN ID (and QinQ ID if specified after comma)\n");
   printf("-L <num>        Forge VLAN packets with <num> different ids\n");
@@ -321,11 +274,11 @@ static int reforge_packet(u_char *buffer, u_int buffer_len, u_int idx, u_int use
 /* *************************************** */
 
 static void randomize_packets() {
-  struct packet *tobemoved, *add_before, *prev, *tmp;
+  struct packet *tobemoved, *add_before, *prev, *tmp, *last;
   int j, n, moved_pkts = 0;
  
   // keep the first item in pkt_head and detach the second
-  tobemoved = pkt_head->next;
+  last = tobemoved = pkt_head->next;
   pkt_head->next = NULL;
   moved_pkts++;
 
@@ -348,8 +301,11 @@ static void randomize_packets() {
     tobemoved->next = add_before;
     moved_pkts++;
 
+    last = tobemoved;
     tobemoved = tmp;
   }
+
+  last->next = pkt_head;
 }
 
 /* *************************************** */
@@ -831,9 +787,16 @@ int main(int argc, char* argv[]) {
   if (pps < 0) /* flush for sending at the exact original pcap speed only, otherwise let pf_ring flush when needed) */
     flush = 1;
 
-  if (randomize && !on_the_fly_reforging)
-    randomize_packets();
-
+  if (randomize) {
+    if(reforge_ip == 0) {
+      randomize = 0;
+      fprintf(stderr, "WARNING: -z requires you to use -b: ignored\n");
+    } else {
+      if(!on_the_fly_reforging)
+	randomize_packets();
+    }
+  }
+  
   if(!verbose) {
     signal(SIGALRM, my_sigalarm);
     alarm(1);
@@ -860,20 +823,23 @@ int main(int argc, char* argv[]) {
     if (unlikely(verbose))
       printf("[%d] pfring_send(%d) returned %d\n", i, tosend->len, rc);
 
-    if (rc == PF_RING_ERROR_INVALID_ARGUMENT) {
+    if (likely(rc >= 0)) {
+      num_pkt_good_sent++;
+      num_bytes_good_sent += tosend->len + 24 /* 8 Preamble + 4 CRC + 12 IFG */;
+    } else if (rc == PF_RING_ERROR_INVALID_ARGUMENT) {
       if (send_error_once) {
         printf("Attempting to send invalid packet [len: %u][MTU: %u]\n",
 	       tosend->len, pfring_get_mtu_size(pd));
         send_error_once = 0;
       }
-    } else if (rc < 0) {
+    } else if (rc == PF_RING_ERROR_NOT_SUPPORTED) {
+      printf("Transmission is not supporte on the selected interface\n");
+      goto close_socket;
+    } else /* Other rc < 0 */ {
       /* Not enough space in buffer */
       if(!active_poll)
 	usleep(1);
       goto redo;
-    } else {
-      num_pkt_good_sent++;
-      num_bytes_good_sent += tosend->len + 24 /* 8 Preamble + 4 CRC + 12 IFG */;
     }
 
     if (randomize && on_the_fly_reforging) {
@@ -902,11 +868,17 @@ int main(int argc, char* argv[]) {
         if (unlikely(do_shutdown)) break;
       }
     } else if (pps < 0) {
+      int tx_syncronized = 0;
       /* real pcap rate */
       if (tosend->ticks_from_beginning == 0)
         tick_start = getticks(); /* first packet, resetting time */
-      while((getticks() - tick_start) < tosend->ticks_from_beginning)
+      while((getticks() - tick_start) < tosend->ticks_from_beginning) {
+        if (!tx_syncronized) {
+          pfring_flush_tx_packets(pd);
+          tx_syncronized = 1;
+        }
         if (unlikely(do_shutdown)) break;
+      }
     }
 
     /* add N uniq packets per second */
@@ -927,9 +899,12 @@ int main(int argc, char* argv[]) {
     if(num_to_send > 0) i++;
   } /* for */
 
+  pfring_flush_tx_packets(pd);
+
   print_stats();
   printf("Sent %llu packets\n", (long long unsigned int) num_pkt_good_sent);
 
+ close_socket:
   pfring_close(pd);
 
   if (pidFileName)
