@@ -1,5 +1,5 @@
 /*
- * (C) 2003-2021 - ntop 
+ * (C) 2003-23 - ntop 
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -50,6 +50,16 @@
 
 #define POW2(n) ((n & (n - 1)) == 0)
 
+#define MAX_PACKET_SIZE 16018
+
+struct packet {
+  u_int32_t id;
+  u_int16_t len;
+  u_int64_t ticks_from_beginning;
+  u_char *pkt;
+  struct packet *next;
+};
+
 struct compact_eth_hdr {
   unsigned char   h_dest[ETH_ALEN];
   unsigned char   h_source[ETH_ALEN];
@@ -624,3 +634,41 @@ static char *etheraddr2string(const u_char *ep, char *buf) {
 
 /* *************************************** */
 
+int is_fd_ready(int fd) {
+  struct timeval timeout = {0};
+  fd_set fdset;
+  FD_ZERO(&fdset);
+  FD_SET(fd, &fdset);
+  return (select(fd+1, &fdset, NULL, NULL, &timeout) == 1);
+}
+
+/* Read packet HEX from stdin */
+int read_packet_hex(u_char *buf, int buf_len) {
+  int i = 0, d, bytes = 0;
+  char c;
+  char s[3] = {0};
+
+  if (!is_fd_ready(fileno(stdin)))
+    return 0;
+
+  while ((d = fgetc(stdin)) != EOF) {
+    if (d < 0) break;
+    c = (u_char) d;
+    if ((c >= '0' && c <= '9') 
+     || (c >= 'a' && c <= 'f')
+     || (c >= 'A' && c <= 'F')) {
+      s[i&0x1] = c;
+      if (i&0x1) {
+        bytes = (i+1)/2;
+        sscanf(s, "%2hhx", &buf[bytes-1]);
+	if (bytes == buf_len) break;
+      }
+      i++;
+    }
+  }
+
+  return bytes;
+}
+
+/* *************************************** */
+
