@@ -1,5 +1,5 @@
 /*
- * (C) 2018-2021 - ntop
+ * (C) 2018-23 - ntop
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -93,6 +93,7 @@ static u_int32_t pps = 0;
 static struct ether_addr if_mac = { 0 };
 static u_int32_t rx_ring_size = RX_RING_SIZE;
 static u_int32_t tx_ring_size = TX_RING_SIZE;
+static int hwts_dynfield_offset = -1;
 
 static struct lcore_stats {
   u_int64_t num_pkts;
@@ -152,10 +153,26 @@ static int port_init(void) {
 
   for (i = 0; i < num_ports; i++) {
     u_int8_t port_id = (i == 0) ? port : twin_port;
+    struct rte_eth_dev_info dev_info;
     unsigned int numa_socket_id;
-    
+
     printf("Configuring port %u...\n", port_id);
 
+    retval = rte_eth_dev_info_get(port_id, &dev_info);
+
+    if (retval != 0) {
+      printf("Error during getting device (port %u) info: %s\n", port_id, strerror(-retval));
+    } else if (!(dev_info.rx_offload_capa & DEV_RX_OFFLOAD_TIMESTAMP)) {
+      printf("Port %u does not support hardware timestamping\n", port_id);
+    } else {
+      port_conf.rxmode.offloads |= DEV_RX_OFFLOAD_TIMESTAMP; //RTE_ETH_RX_OFFLOAD_TIMESTAMP
+
+      rte_mbuf_dyn_rx_timestamp_register(&hwts_dynfield_offset, NULL);
+
+      if (hwts_dynfield_offset < 0)
+        printf("Failed to register timestamp field\n");
+    }
+
     fc_conf.mode = RTE_FC_NONE;
     fc_conf.autoneg = 0;
 
@@ -178,6 +195,12 @@ static int port_init(void) {
     if (tx_csum_offload)
       port_conf.txmode.offloads = DEV_TX_OFFLOAD_IPV4_CKSUM | DEV_TX_OFFLOAD_UDP_CKSUM;
 
+    if (num_queues > 1) {
+      /* Configure RSS (on some adapters this has to be explicitly set) */
+      port_conf.rxmode.mq_mode = ETH_MQ_RX_RSS;
+      port_conf.rx_adv_conf.rss_conf.rss_hf = (ETH_RSS_IP | ETH_RSS_TCP | ETH_RSS_UDP);
+    }
+
     retval = rte_eth_dev_configure(port_id, num_queues /* RX */, num_queues /* TX */, &port_conf);
     
     if (retval != 0)
@@ -362,6 +385,14 @@ static void tx_test(u_int8_t port_id, u_int16_t queue_id) {
 
 /* ************************************ */
 
+static inline u_int64_t hwts_field(struct rte_mbuf *mbuf) {
+  rte_mbuf_timestamp_t *ts;
+  ts = RTE_MBUF_DYNFIELD(mbuf, hwts_dynfield_offset, rte_mbuf_timestamp_t *);
+  return *ts;
+}
+
+/* ************************************ */
+
 static int processing_thread(__attribute__((unused)) void *arg) {
   unsigned lcore_id = rte_lcore_id();
   unsigned lcore_index = rte_lcore_index(lcore_id);
@@ -424,10 +455,20 @@ static int processing_thread(__attribute__((unused)) void *arg) {
 	char *data = rte_pktmbuf_mtod(bufs[i], char *);
 	int len = rte_pktmbuf_pkt_len(bufs[i]);
 	pfring_ft_action action = PFRING_FT_ACTION_DEFAULT;
+
+        //if (hwts_dynfield_offset != -1)
+        //  printf("TS=%ju\n", hwts_field(bufs[i]));
 	
 	if (likely(compute_flows)) {
           h.len = h.caplen = len;
-          gettimeofday(&h.ts, NULL);
+
+          if (hwts_dynfield_offset != -1) {
+            u_int64_t ns = hwts_field(bufs[i]);
+            h.ts.tv_sec = ns / 1000000000;
+            h.ts.tv_usec = (ns / 1000) % 1000000;
+          } else {
+            gettimeofday(&h.ts, NULL);
+          }
 
 	  action = pfring_ft_process(ft, (const u_char *) data, &h, &ext_hdr);
         }
@@ -479,7 +520,7 @@ static int processing_thread(__attribute__((unused)) void *arg) {
 /* ************************************ */
 
 static void print_help(void) {
-  printf("ftflow_dpdk - (C) 2018-2021 ntop.org\n");
+  printf("ftflow_dpdk - (C) 2018-23 ntop\n");
   printf("Usage: ftflow_dpdk [EAL options] -- [options]\n");
   printf("-p <id>[,<id>]  Port id (up to 2 ports are supported)\n");
   printf("-7              Enable L7 protocol detection (nDPI)\n");
