@@ -1,5 +1,5 @@
 /*
- * (C) 2003-2021 - ntop 
+ * (C) 2003-23 - ntop 
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -109,6 +109,9 @@ bitmap64_t(allowed_vlans, 1024);
 u_int8_t map_vlan_size = 0;
 u_int16_t map_vlan[MAX_MAP_VLAN_SIZE];
 
+#define ETH_P_8585         0x8585
+u_int16_t eth_distr_type = ETH_P_8585;
+
 #ifdef HAVE_PF_RING_FT
 int notify_fd;
 int notify_wd;
@@ -289,8 +292,8 @@ void print_stats() {
       tot_recv += stats.recv, tot_drop += stats.drop;
 
   if (!daemon_mode && !proc_stats_only) {
-    trace(TRACE_NORMAL, "=========================");
-    trace(TRACE_NORMAL, "Queue TX Stats (Packets sent to applications):");
+    trace(TRACE_INFO, "=========================");
+    trace(TRACE_INFO, "Queue TX Stats (Packets sent to applications):");
   }
   
   for (i = 0; i < num_consumer_queues; i++)
@@ -298,14 +301,14 @@ void print_stats() {
       tot_slave_sent += stats.sent, tot_slave_recv += stats.recv, tot_slave_drop += stats.drop;
       
       if (!daemon_mode && !proc_stats_only)
-	trace(TRACE_NORMAL, "                   Queue %2u: %s pkts (%s drops)\n", i,
+	trace(TRACE_INFO, "                   Queue %2u: %s pkts (%s drops)\n", i,
 	      pfring_format_numbers((double)stats.sent+stats.drop, buf1, sizeof(buf1), 0),
 	      pfring_format_numbers((double)stats.drop, buf2, sizeof(buf2), 0));      
     }
   
   if (!daemon_mode && !proc_stats_only) {
-    trace(TRACE_NORMAL, "");
-    trace(TRACE_NORMAL, "Total Absolute Stats: Recv %s pkts (%s drops) - Forwarded %s pkts (%s drops)\n", 
+    trace(TRACE_INFO, "");
+    trace(TRACE_INFO, "Total Absolute Stats: Recv %s pkts (%s drops) - Forwarded %s pkts (%s drops)\n", 
             pfring_format_numbers((double)tot_recv, buf1, sizeof(buf1), 0),
 	    pfring_format_numbers((double)tot_drop, buf2, sizeof(buf2), 0),
 	    pfring_format_numbers((double)tot_slave_sent, buf3, sizeof(buf3), 0),
@@ -323,7 +326,7 @@ void print_stats() {
     unsigned long long diff_slave_drop = tot_slave_drop - last_tot_slave_drop;
 
     if (!daemon_mode && !proc_stats_only) {
-      trace(TRACE_NORMAL, "Actual Stats:         Recv %s pps (%s drops) - Forwarded %s pps (%s drops)\n",
+      trace(TRACE_INFO, "Actual Stats:         Recv %s pps (%s drops) - Forwarded %s pps (%s drops)\n",
 	      pfring_format_numbers(((double)diff_recv/(double)(delta_msec/1000)),  buf1, sizeof(buf1), 1),
 	      pfring_format_numbers(((double)diff_drop/(double)(delta_msec/1000)),  buf2, sizeof(buf2), 1),
 	      pfring_format_numbers(((double)diff_slave_sent/(double)(delta_msec/1000)),  buf3, sizeof(buf3), 1),
@@ -363,7 +366,7 @@ void print_stats() {
         tot_if_recv += stats.recv;
         tot_if_drop += stats.drop;
         if (!daemon_mode && !proc_stats_only) {
-          trace(TRACE_NORMAL, "                %s RX %lu pkts Dropped %lu pkts (%.1f %%)\n", 
+          trace(TRACE_INFO, "                %s RX %lu pkts Dropped %lu pkts (%.1f %%)\n", 
                   devices[i], stats.recv, stats.drop, 
 	          stats.recv == 0 ? 0 : ((double)(stats.drop*100)/(double)(stats.recv + stats.drop)));
         }
@@ -375,12 +378,12 @@ void print_stats() {
 	     (long unsigned int)tot_if_recv, 
 	     (long unsigned int)tot_if_drop);
 
-    trace(TRACE_NORMAL, "Queue RX Stats (Packets read by applications):");
+    trace(TRACE_INFO, "Queue RX Stats (Packets read by applications):");
     
     for (i = 0; i < num_consumer_queues; i++) {
       if (pfring_zc_stats(outzqs[i], &stats) == 0) {
         if (!daemon_mode && !proc_stats_only) {
-          trace(TRACE_NORMAL, "                   Queue %2u: RX %lu pkts Dropped %lu pkts (%.1f %%)\n", 
+          trace(TRACE_INFO, "                   Queue %2u: RX %lu pkts Dropped %lu pkts (%.1f %%)\n", 
                   i, stats.recv, stats.drop, 
 	          stats.recv == 0 ? 0 : ((double)(stats.drop*100)/(double)(stats.recv + stats.drop)));
         }
@@ -404,7 +407,7 @@ void print_stats() {
   pfring_zc_set_proc_stats(zc, stats_buf);
   
   if (!daemon_mode && !proc_stats_only)
-    trace(TRACE_NORMAL, "=========================\n\n");
+    trace(TRACE_INFO, "=========================\n\n");
  
   last_tot_recv = tot_recv, last_tot_slave_sent = tot_slave_sent;
   //last_tot_slave_recv = tot_slave_recv;
@@ -595,7 +598,7 @@ void set_outzq_bpf() {
 /* *************************************** */
 
 void printHelp(void) {
-  printf("zbalance_ipc - (C) 2014-20 ntop.org\n");
+  printf("zbalance_ipc - (C) 2014-23 ntop\n");
   printf("Using PFRING_ZC v.%s\n", pfring_zc_version());
   printf("A master process balancing packets to multiple consumer processes.\n\n");
   printf("Usage: zbalance_ipc -i <device> -c <cluster id> -n <num inst>\n"
@@ -610,15 +613,17 @@ void printHelp(void) {
          "                 instances of multiple applications, using a comma-separated list\n");
   printf("-m <hash mode>   Hashing modes:\n"
          "                 0 - No hash: Round-Robin (default)\n"
-         "                 1 - IP hash (or Thread-ID in case of sysdig)\n"
+         "                 1 - Source/Dest IP hash (or Thread-ID in case of sysdig)\n"
          "                 2 - Fan-out\n"
          "                 3 - Fan-out (1st) + Round-Robin (2nd, 3rd, ..)\n"
-         "                 4 - GTP hash (Inner IP/Port or Seq-Num or Outer IP/Port)\n"
-         "                 5 - GRE hash (Inner or Outer IP)\n"
+         "                 4 - GTP hash (Inner Source/Dest IP/Port or Seq-Num or Outer Source/Dest IP/Port)\n"
+         "                 5 - GRE hash (Inner or Outer Source/Dest IP)\n"
          "                 6 - Interface X to queue X\n"
-         "                 7 - Ethernet type (check 0x8585 type and select the queue, other to queue 0)\n");
+         "                 7 - VLAN ID encapsulated in Ethernet type 0x8585 (see -Y). Queue is selected based on -M. Other Ethernet types to queue 0.\n");
   printf("-r <queue>:<dev> Replace egress queue <queue> with device <dev> (multiple -r can be specified)\n");
   printf("-M <vlans>       Comma-separated list of VLANs to map VLAN to egress queues (-m 7 only)\n");
+  printf("-X               Capture also TX packets (standard drivers only - not supported with ZC drivers)\n");
+  printf("-Y <eth type>    Ethernet type used in -m 7. Default: %u (0x8585)\n", ntohs(ETH_P_8585));
   printf("-S <core id>     Enable Time Pulse thread and bind it to a core\n");
   printf("-R <nsec>        Time resolution (nsec) when using Time Pulse thread\n"
          "                 Note: in non-time-sensitive applications use >= 100usec to reduce cpu load\n");
@@ -652,7 +657,7 @@ void printHelp(void) {
   printf("-A <policy>      Set default policy (0: drop, 1: accept) to be used with -Z (default: accept)\n");
 #endif
   printf("-G <queue>:<ver> Forward GTP-C version <ver> to queue <queue> (with -m 4)\n");
-  printf("-E               Debug mode\n");
+  printf("-J               Debug mode\n");
   printf("-v               Verbose\n");
   exit(-1);
 }
@@ -794,8 +799,6 @@ int64_t direct_distribution_func(pfring_zc_pkt_buff *pkt_handle, pfring_zc_queue
 
 /* *************************************** */
 
-#define ETH_P_8585         0x8585
-
 int64_t eth_distribution_func(pfring_zc_pkt_buff *pkt_handle, pfring_zc_queue *in_queue, void *user) {
   long num_out_queues = (long) user;
   u_char *data = pfring_zc_pkt_buff_data(pkt_handle, in_queue);
@@ -805,7 +808,7 @@ int64_t eth_distribution_func(pfring_zc_pkt_buff *pkt_handle, pfring_zc_queue *i
 
   if (time_pulse) SET_TS_FROM_PULSE(pkt_handle, *pulse_timestamp_ns);
 
-  if (eth_type == ETH_P_8585) {
+  if (eth_type == eth_distr_type) {
     u_int32_t vlan_offset = sizeof(struct ethhdr);
 
     idx = -1;
@@ -859,6 +862,15 @@ int64_t fo_distribution_func(pfring_zc_pkt_buff *pkt_handle, pfring_zc_queue *in
 
 /* *************************************** */
 
+__int128_t fo_distribution_func_v3(pfring_zc_pkt_buff *pkt_handle, pfring_zc_queue *in_queue, void *user) {
+  int64_t l = 0xffffffffffffffff;
+  int64_t h = 0xffffffffffffffff;
+  if (time_pulse) SET_TS_FROM_PULSE(pkt_handle, *pulse_timestamp_ns);
+  return (__int128_t) ((__int128_t) h << 64) | l; 
+}
+
+/* *************************************** */
+
 int64_t fo_rr_distribution_func(pfring_zc_pkt_buff *pkt_handle, pfring_zc_queue *in_queue, void *user) {
   long num_out_queues = (long) user;
 
@@ -888,6 +900,25 @@ int64_t fo_multiapp_ip_distribution_func(pfring_zc_pkt_buff *pkt_handle, pfring_
 
 /* *************************************** */
 
+__int128_t fo_multiapp_ip_distribution_func_v3(pfring_zc_pkt_buff *pkt_handle, pfring_zc_queue *in_queue, void *user) {
+  int32_t i, offset = 0, app_instance, hash;
+  __int128_t consumers_mask = 0; 
+
+  if (time_pulse) SET_TS_FROM_PULSE(pkt_handle, *pulse_timestamp_ns);
+
+  hash = pfring_zc_builtin_ip_hash(pkt_handle, in_queue);
+
+  for (i = 0; i < num_apps; i++) {
+    app_instance = hash % instances_per_app[i];
+    consumers_mask |= ((__int128_t) 1 << (offset + app_instance));
+    offset += instances_per_app[i];
+  }
+
+  return consumers_mask;
+}
+
+/* *************************************** */
+
 int64_t fo_multiapp_gtp_distribution_func(pfring_zc_pkt_buff *pkt_handle, pfring_zc_queue *in_queue, void *user) {
   int32_t i, offset = 0, app_instance, hash;
   int64_t consumers_mask = 0;
@@ -973,9 +1004,10 @@ int main(int argc, char* argv[]) {
   u_int wait_time_sec = 0;
   char **opt_argv;
   char *user = NULL;
-  int num_consumer_queues_limit = 0;
-  u_int32_t flags;
-  const char *opt_string = "ab:c:dD:Ef:G:g:hi:l:m:M:n:N:pr:Q:q:P:R:S:u:wvx:zW:"
+  int num_consumer_queues_limit = 0, use_api_v3 = 0;
+  u_int32_t cluster_flags = 0;
+  u_int32_t rx_open_flags;
+  const char *opt_string = "ab:c:dD:f:G:g:hi:Jl:m:M:n:N:pr:Q:q:P:R:S:u:wvx:Y:zW:X"
 #ifdef HAVE_PF_RING_FT
     "TC:O:"
 #endif
@@ -991,10 +1023,13 @@ int main(int argc, char* argv[]) {
 #endif
   pfring_zc_idle_callback idle_func = NULL;
   pfring_zc_distribution_func distr_func = NULL;
+  pfring_zc_distribution_func_v3 distr_func_v3 = NULL;
   pfring_zc_filtering_func filter_func = NULL;
 
   start_time.tv_sec = 0;
 
+  rx_open_flags = PF_RING_ZC_DEVICE_CAPTURE_INJECTED;
+
   if (argc == 1) {
     if (load_args_from_file(DEFAULT_CONF_FILE, &opt_argc, &opt_argv) != 0) {
       trace(TRACE_ERROR, "Please specify all mandatory options via cli or configuration file (default path: %s)\n", DEFAULT_CONF_FILE);
@@ -1030,7 +1065,7 @@ int main(int argc, char* argv[]) {
     case 'D':
       user = strdup(optarg);
       break;
-    case 'E':
+    case 'J':
       pfring_zc_debug();
       break;
     case 'f':
@@ -1107,6 +1142,12 @@ int main(int argc, char* argv[]) {
     case 'x':
       vlan_filter = strdup(optarg);
     break;
+    case 'X':
+      rx_open_flags |= PF_RING_ZC_DEVICE_CAPTURE_TX;
+    break;
+    case 'Y':
+      eth_distr_type = htons(atoi(optarg));
+    break;
     case 'z':
       proc_stats_only = 1;
       break;
@@ -1206,10 +1247,12 @@ int main(int argc, char* argv[]) {
   if (num_apps > 1) {
     switch (hash_mode) {
       case 1: 
+        num_consumer_queues_limit = PF_RING_ZC_SEND_PKT_MULTI_V3_MAX_QUEUES;
+        break;
       case 4:
       case 5:
       case 6:
-        num_consumer_queues_limit = 64; /* egress mask is 64 bit */
+        num_consumer_queues_limit = PF_RING_ZC_SEND_PKT_MULTI_MAX_QUEUES;
         break;
       default:
         printHelp();
@@ -1217,17 +1260,24 @@ int main(int argc, char* argv[]) {
     }
   }
   switch (hash_mode) {
-    case 1: 
+    case 2: 
+      num_consumer_queues_limit = PF_RING_ZC_SEND_PKT_MULTI_V3_MAX_QUEUES;
+      break;
     case 3:
-      num_consumer_queues_limit = 64; /* egress mask is 64 bit */
+      num_consumer_queues_limit = PF_RING_ZC_SEND_PKT_MULTI_MAX_QUEUES;
       break;
     default:
       break;
   }
 
-  if (num_consumer_queues_limit && num_consumer_queues > num_consumer_queues_limit) { 
-    trace(TRACE_ERROR, "Misconfiguration detected: you cannot use more than %d egress queues in fan-out or multi-app mode\n", num_consumer_queues_limit);
-    return -1;
+  if (num_consumer_queues_limit) {
+    if (num_consumer_queues > num_consumer_queues_limit) {
+      trace(TRACE_ERROR, "Misconfiguration detected: you cannot use more than %d egress queues in fan-out (-m 1|3) or multi-app mode (-n X,Y)\n", num_consumer_queues_limit);
+      return -1;
+    }
+
+    if (num_consumer_queues_limit > PF_RING_ZC_SEND_PKT_MULTI_MAX_QUEUES)
+      use_api_v3 = 1;
   }
 
   for (i = 0; i < num_devices; i++) {
@@ -1273,10 +1323,10 @@ int main(int argc, char* argv[]) {
   if (daemon_mode)
     daemonize();
 
-  flags = 0;
+  cluster_flags = 0;
 
   if (enable_vm_support)
-    flags |= PF_RING_ZC_ENABLE_VM_SUPPORT;
+    cluster_flags |= PF_RING_ZC_ENABLE_VM_SUPPORT;
 
   zc = pfring_zc_create_cluster(
     cluster_id, 
@@ -1287,7 +1337,7 @@ int main(int argc, char* argv[]) {
      + (num_outdevs * MAX_CARD_SLOTS) - (num_outdevs * (queue_len /* replaced queues */ - 1 /* dummy queues */)), 
     pfring_zc_numa_get_cpu_node(bind_worker_core),
     hugepages_mountpoint,
-    flags 
+    cluster_flags 
   );
 
   if (zc == NULL) {
@@ -1295,6 +1345,7 @@ int main(int argc, char* argv[]) {
     switch (errno) {
       case ENOBUFS:
         trace(TRACE_ERROR, "Insufficient hugepage memory, please try increasing your hugepage count");
+	trace(TRACE_ERROR, "Please see https://www.ntop.org/guides/pf_ring/hugepages.html");
       break;
       case EAFNOSUPPORT:
         trace(TRACE_ERROR, "PF_RING kernel module not loaded, please start the pfring service");
@@ -1309,7 +1360,7 @@ int main(int argc, char* argv[]) {
   for (i = 0; i < num_devices; i++) {
     if (strcmp(devices[i], "Q") != 0) {
 
-      inzqs[i] = pfring_zc_open_device(zc, devices[i], rx_only, 0);
+      inzqs[i] = pfring_zc_open_device(zc, devices[i], rx_only, rx_open_flags);
 
       if (inzqs[i] == NULL) {
         trace(TRACE_ERROR, "pfring_zc_open_device error [%s] Please check that %s is up and not already used\n",
@@ -1586,10 +1637,13 @@ int main(int argc, char* argv[]) {
     switch (hash_mode) {
       case 1: 
         distr_func = fo_multiapp_ip_distribution_func;
+        distr_func_v3 = fo_multiapp_ip_distribution_func_v3;
       break;
       case 2: 
-        if (time_pulse) 
+        if (time_pulse) {
           distr_func = fo_distribution_func; /* else built-in send-to-all */
+          distr_func_v3 = fo_distribution_func_v3;
+        }
       break;
       case 3: 
         distr_func = fo_rr_distribution_func;
@@ -1605,20 +1659,36 @@ int main(int argc, char* argv[]) {
       break;
     }
 
-    zw = pfring_zc_run_fanout_v2(
-      inzqs, 
-      outzmq, 
-      num_devices,
-      wsp,
-      round_robin_bursts_policy, 
-      idle_func,
-      filter_func,
-      NULL,
-      distr_func,
-      (void *) ((long) num_consumer_queues),
-      !wait_for_packet, 
-      bind_worker_core
-    );
+    if (use_api_v3)
+      zw = pfring_zc_run_fanout_v3(
+        inzqs, 
+        outzmq, 
+        num_devices,
+        wsp,
+        round_robin_bursts_policy, 
+        idle_func,
+        filter_func,
+        NULL,
+        distr_func_v3,
+        (void *) ((long) num_consumer_queues),
+        !wait_for_packet, 
+        bind_worker_core
+      );
+    else
+      zw = pfring_zc_run_fanout_v2(
+        inzqs, 
+        outzmq, 
+        num_devices,
+        wsp,
+        round_robin_bursts_policy, 
+        idle_func,
+        filter_func,
+        NULL,
+        distr_func,
+        (void *) ((long) num_consumer_queues),
+        !wait_for_packet, 
+        bind_worker_core
+      );
 
   }
 
