@@ -1,5 +1,5 @@
 /*
- * (C) 2003-2021 - ntop 
+ * (C) 2003-23 - ntop 
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -43,13 +43,6 @@
 
 #include "zutils.c"
 
-struct packet {
-  u_int16_t len;
-  u_int64_t ticks_from_beginning;
-  char *pkt;
-  struct packet *next;
-};
-
 #define ALARM_SLEEP             1
 #define CACHE_LINE_LEN         64
 #define MAX_CARD_SLOTS      32768
@@ -154,43 +147,6 @@ struct udp_header {
 
 /* *************************************** */
 
-int is_fd_ready(int fd) {
-  struct timeval timeout = {0};
-  fd_set fdset;
-  FD_ZERO(&fdset);
-  FD_SET(fd, &fdset);
-  return (select(fd+1, &fdset, NULL, NULL, &timeout) == 1);
-}
-
-int read_packet_hex(u_char *buf, int buf_len) {
-  int i = 0, d, bytes = 0;
-  char c;
-  char s[3] = {0};
-
-  if (!is_fd_ready(fileno(stdin)))
-    return 0;
-
-  while ((d = fgetc(stdin)) != EOF) {
-    if (d < 0) break;
-    c = (u_char) d;
-    if ((c >= '0' && c <= '9') 
-	|| (c >= 'a' && c <= 'f')
-	|| (c >= 'A' && c <= 'F')) {
-      s[i&0x1] = c;
-      if (i&0x1) {
-        bytes = (i+1)/2;
-        sscanf(s, "%2hhx", &buf[bytes-1]);
-	if (bytes == buf_len) break;
-      }
-      i++;
-    }
-  }
-
-  return bytes;
-}
-
-/* *************************************** */
-
 void print_stats() {
   struct timeval endTime;
   double deltaMillisec;
@@ -257,7 +213,7 @@ void sigproc(int sig) {
 /* *************************************** */
 
 void printHelp(void) {
-  printf("zsend - (C) 2014-2021-2021 ntop.org\n");
+  printf("zsend - (C) 2014-23 ntop\n");
   printf("Using PFRING_ZC v.%s\n", pfring_zc_version());
   printf("A traffic generator able to replay synthetic udp packets or hex from standard input.\n"); 
   printf("Usage:    zsend -i <device> -c <cluster id>\n"
@@ -345,7 +301,7 @@ void *send_traffic(void *user) {
 	if (datalink == DLT_LINUX_SLL) p->len -= 2;
 	p->ticks_from_beginning = (((h->ts.tv_sec - beginning.tv_sec) * 1000000) + (h->ts.tv_usec - beginning.tv_usec)) * hz / 1000000;
 	p->next = NULL;
-	p->pkt = (char*)malloc(p->len);
+	p->pkt = (u_char*)malloc(p->len);
 
 	if(p->pkt == NULL) {
 	  printf("Not enough memory\n");
