@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2005-2021 - ntop.org
+ * (C) 2005-23 - ntop
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -34,6 +34,10 @@
 #include "pfring_mod_sysdig.h"
 #include "pfring_mod_pcap.h"
 
+#ifndef DLT_EN10MB
+#define DLT_EN10MB 1
+#endif
+
 #ifdef HAVE_AF_XDP
 #ifdef HAVE_PF_RING_ZC
 /* AF_XDP */
@@ -56,11 +60,6 @@
 #include "pfring_mod_fiberblaze.h"
 #endif
 
-#ifdef HAVE_MELLANOX
-/* Mellanox */
-#include "pfring_mod_mlx.h"
-#endif
-
 #ifdef HAVE_ACCOLADE
 /* Accolade */
 #include "pfring_mod_accolade.h"
@@ -71,6 +70,11 @@
 #include "pfring_mod_myricom.h"
 #endif
 
+#ifdef HAVE_MLX
+/* Mellanox */
+#include "pfring_mod_mlx.h"
+#endif
+
 #ifdef HAVE_INVEATECH
 /* InveaTech */
 #include "pfring_mod_invea.h"
@@ -156,14 +160,6 @@ static pfring_module_info pfring_module_list[] = {
   },
 #endif
 
-#ifdef HAVE_MELLANOX
-  {
-    .name = "mlx",
-    .open = pfring_mlx_open,
-    .findalldevs = NULL
-  },
-#endif
-
 #ifdef HAVE_ACCOLADE
   {
     .name = "anic",
@@ -180,6 +176,14 @@ static pfring_module_info pfring_module_list[] = {
   },
 #endif
 
+#ifdef HAVE_MLX
+  {
+    .name = "mlx",
+    .open = pfring_mlx_open,
+    .findalldevs = pfring_mlx_findalldevs
+  },
+#endif
+
 #ifdef HAVE_INVEATECH
   {
     .name = "invea",
@@ -233,8 +237,10 @@ pfring *pfring_open(const char *device_name, u_int32_t caplen, u_int32_t flags)
   char *ft_conf_file;
   pfring *ring;
 
-  if (device_name == NULL)
+  if (device_name == NULL) {
+    errno = EINVAL;
     return NULL;
+  }
 
 #ifdef RING_DEBUG
   printf("[PF_RING] Attempting to pfring_open(%s)\n", device_name);
@@ -302,6 +308,7 @@ pfring *pfring_open(const char *device_name, u_int32_t caplen, u_int32_t flags)
       ret = pfring_ft_load_ndpi_protocols(ring->ft, ft_proto_file);
 
       if (ret != 0) {
+        errno = EINVAL;
         return NULL;
       }
     }
@@ -310,6 +317,7 @@ pfring *pfring_open(const char *device_name, u_int32_t caplen, u_int32_t flags)
       ret = pfring_ft_load_configuration(ring->ft, ft_conf_file);
 
       if (ret != 0) {
+        errno = EINVAL;
         return NULL;
       }
     }
@@ -348,7 +356,6 @@ pfring *pfring_open(const char *device_name, u_int32_t caplen, u_int32_t flags)
 
   /* default */
   if(!mod_found) {
-    errno = ENODEV;
     ring->device_name = strdup(device_name ? device_name : "any");
     if (ring->device_name == NULL) {
       errno = ENOMEM;
@@ -359,6 +366,8 @@ pfring *pfring_open(const char *device_name, u_int32_t caplen, u_int32_t flags)
   }
 
   if(ret < 0) {
+    if (!errno)
+      errno = ENODEV;
     if (ring->device_name != NULL) free(ring->device_name);
     free(ring);
     return NULL;
@@ -367,6 +376,7 @@ pfring *pfring_open(const char *device_name, u_int32_t caplen, u_int32_t flags)
   if(unlikely(ring->reentrant)) {
     if (pfring_rwlock_init(&ring->rx_lock, PTHREAD_PROCESS_PRIVATE) != 0 || 
         pfring_rwlock_init(&ring->tx_lock, PTHREAD_PROCESS_PRIVATE) != 0) {
+      errno = ENOTSUP;
       free(ring);
       return NULL;
     }
@@ -377,9 +387,11 @@ pfring *pfring_open(const char *device_name, u_int32_t caplen, u_int32_t flags)
   ring->rdi.device_id = ring->rdi.port_id = -1; /* Default */
 
   ring->mtu = pfring_get_mtu_size(ring);
-  if(ring->mtu == 0) ring->mtu =  9000 /* Jumbo MTU */;
+  if(ring->mtu == 0) ring->mtu = 9000 /* Jumbo MTU */;
 
   pfring_get_bound_device_ifindex(ring, &ring->device_id);
+  errno = 0; /* Ignore errno from pfring_get_bound_device_ifindex */
+
   ring->initialized = 1;
 
 #ifdef RING_DEBUG
@@ -396,15 +408,23 @@ u_int8_t pfring_open_multichannel(const char *device_name, u_int32_t caplen,
 				  pfring *ring[MAX_NUM_RX_CHANNELS]) {
   u_int8_t num_channels, i, num = 0;
   char *at;
-  char base_device_name[32];
+  const char *dev;
+  char base_dev[32];
 
-  snprintf(base_device_name, sizeof(base_device_name), "%s", device_name);
-  at = strchr(base_device_name, '@');
+  dev = device_name;
+
+  /* Use linux device in case of zc to avoid opening in ZC mode for read only */
+  if (strncmp(dev, "zc:", 3) == 0)
+    dev = &dev[3];
+
+  snprintf(base_dev, sizeof(base_dev), "%s", dev);
+
+  at = strchr(base_dev, '@');
   if(at != NULL)
     at[0] = '\0';
 
   /* Count how many RX channel the specified device supports */
-  ring[0] = pfring_open(base_device_name, caplen, flags);
+  ring[0] = pfring_open(base_dev, caplen, flags);
 
   if(ring[0] == NULL)
     return(0);
@@ -417,11 +437,18 @@ u_int8_t pfring_open_multichannel(const char *device_name, u_int32_t caplen,
     num_channels = MAX_NUM_RX_CHANNELS;
 
   /* Now do the real job */
-  for(i=0; i<num_channels; i++) {
-    char dev[64];
 
-    snprintf(dev, sizeof(dev), "%s@%d", base_device_name, i);
-    ring[i] = pfring_open(dev, caplen, flags);
+  snprintf(base_dev, sizeof(base_dev), "%s", device_name);
+
+  at = strchr(base_dev, '@');
+  if(at != NULL)
+    at[0] = '\0';
+
+  for(i=0; i<num_channels; i++) {
+    char dev_queue[64];
+
+    snprintf(dev_queue, sizeof(dev_queue), "%s@%d", base_dev, i);
+    ring[i] = pfring_open(dev_queue, caplen, flags);
 
     if(ring[i] == NULL)
       return(num);
@@ -623,9 +650,7 @@ int pfring_recv(pfring *ring, u_char** buffer, u_int buffer_len,
 
     ring->break_recv_loop = 0;
 
-#if defined(ENABLE_BPF) || defined(HAVE_PF_RING_FT)
 recv_next:
-#endif
 
     rc = ring->recv(ring, buffer, buffer_len, hdr, wait_for_incoming_packet);
 
@@ -638,7 +663,7 @@ recv_next:
         pfring_handle_ixia_hw_timestamp(*buffer, hdr);
       else if(ring->vss_apcon_timestamp_enabled)
         pfring_handle_vss_apcon_hw_timestamp(*buffer, hdr);
-      else if(ring->flags & PF_RING_ARISTA_TIMESTAMP)
+      else if(ring->flags & PF_RING_METAWATCH_TIMESTAMP)
         pfring_handle_metawatch_hw_timestamp(*buffer, hdr);
       else if(ring->flags & PF_RING_ARISTA_TIMESTAMP) {
         if (pfring_handle_arista_hw_timestamp(*buffer, hdr) == 1)
@@ -746,6 +771,15 @@ int pfring_set_tx_watermark(pfring *ring, u_int16_t watermark) {
 
 /* **************************************************** */
 
+int pfring_set_default_hw_action(pfring *ring, generic_default_action_type action) {
+   if(ring && ring->set_default_hw_action)
+    return ring->set_default_hw_action(ring, action);
+
+  return(PF_RING_ERROR_NOT_SUPPORTED);
+}
+
+/* **************************************************** */
+
 int pfring_add_hw_rule(pfring *ring, hw_filtering_rule *rule) {
   if(ring && ring->add_hw_rule)
     return ring->add_hw_rule(ring, rule);
@@ -1222,6 +1256,21 @@ int pfring_get_link_status(pfring *ring) {
 
 /* **************************************************** */
 
+int pfring_get_link_type(pfring *ring) {
+  if(ring && ring->get_link_type)
+    return ring->get_link_type(ring);
+
+  return DLT_EN10MB;
+}
+
+/* **************************************************** */
+
+int pfring_get_caplen(pfring *ring) {
+  return ring->caplen;
+}
+
+/* **************************************************** */
+
 u_int16_t pfring_get_slot_header_len(pfring *ring) {
   if(ring && ring->get_slot_header_len)
     return ring->get_slot_header_len(ring);
