@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2005-2021 - ntop.org
+ * (C) 2005-23 - ntop
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -173,6 +173,7 @@ typedef struct pfring_if {
   char *name;
   char *system_name;
   char *module;
+  char *module_version;
   char *sn;
   char mac[6];
   struct { /* Bus ID: "%04X:%02X:%02X.%X", slot, bus, device, function */
@@ -181,7 +182,7 @@ typedef struct pfring_if {
     int device;
     int function;
   } bus_id;
-  int status; /* 1: up, 0: down*/
+  int status; /* 1: up, 0: down, -1: unknown */
   int license; /* 1: valid, 0: invalid or not installed */
   time_t license_expiration; /* expiration epoch */
   struct pfring_if *next;
@@ -287,6 +288,7 @@ struct __pfring {
   int       (*get_device_ifindex)           (pfring *, char *, int *);
   u_int16_t (*get_slot_header_len)          (pfring *);
   int       (*set_virtual_device)           (pfring *, virtual_filtering_device_info *);
+  int       (*set_default_hw_action)        (pfring *, generic_default_action_type);
   int       (*add_hw_rule)                  (pfring *, hw_filtering_rule *);
   int       (*remove_hw_rule)               (pfring *, u_int16_t);
   int       (*loopback_test)                (pfring *, char *, u_int, u_int);
@@ -307,6 +309,7 @@ struct __pfring {
   int       (*set_bound_dev_name)           (pfring *, char *);
   int       (*get_metadata)         	    (pfring *, u_char **, u_int32_t *);
   u_int32_t (*get_interface_speed)	    (pfring *);
+  int       (*get_link_type)		    (pfring *);
 
   /* Silicom Redirector Only */
   struct {
@@ -581,6 +584,14 @@ int pfring_set_poll_duration(pfring *ring, u_int duration);
  */
 int pfring_set_tx_watermark(pfring *ring, u_int16_t watermark);
 
+/**
+ * Set a default action for filtering into the NIC.
+ * @param ring The PF_RING handle on which the default rule will be set. 
+ * @param action. The action (default_pass or default_drop).
+ * @return 0 on success, a negative value otherwise.
+ */
+int pfring_set_default_hw_action(pfring *ring, generic_default_action_type action);
+
 /**
  * Set a specified filtering rule into the NIC. Note that no PF_RING filter is added, but only a NIC filter.
  *
@@ -914,7 +925,7 @@ int pfring_enable_rss_rehash(pfring *ring);
  * Performs passive wait on a PF_RING socket, similar to the standard poll(), taking care of data structures synchronization. 
  * @param ring          The PF_RING socket to poll. 
  * @param wait_duration The poll timeout in msec.
- * @return 0 on success, a negative value otherwise.
+ * @return 0 (no packet) or 1 (packet available) on success, a negative value otherwise.
  */
 int pfring_poll(pfring *ring, u_int wait_duration);
 
@@ -1099,6 +1110,20 @@ int pfring_send_last_rx_packet(pfring *ring, int tx_interface_id);
  */
 int pfring_get_link_status(pfring *ring);
 
+/**
+ * Return the Network link type.
+ * @param ring The PF_RING handle.
+ * @return The link type (e.g. DLT_EN10MB)
+ */
+int pfring_get_link_type(pfring *ring);
+
+/**
+ * Return the max capture length.
+ * @param ring The PF_RING handle.
+ * @return The capture length.
+ */
+int pfring_get_caplen(pfring *ring);
+
 /**
  * Synchronizes the egress ring indexes/registers flushing enqueued packets.
  * @param ring The PF_RING handle.
@@ -1322,6 +1347,13 @@ void pfring_handle_vss_apcon_hw_timestamp(u_char* buffer, struct pfring_pkthdr *
  */
 u_int32_t pfring_get_interface_speed(pfring *ring);
 
+/**
+ * Get interface speed by linux interface name.
+ * @param ifname The interface name.
+ * @return 0 if interface speed is unknown, the interface speed (Mbit/s) otherwise.
+ */
+u_int32_t pfring_get_ethtool_link_speed(const char *ifname);
+
 /**
  * List all interfaces.
  * @return The interface list.
