@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2016-2020 ntop.org
+ *  Copyright (C) 2016-23 ntop
  *
  *      http://www.ntop.org/
  *
@@ -76,6 +76,7 @@ static char *proto_to_string(int protoq) {
   switch(protoq) {
     case NBPF_Q_LINK: return "Eth";
     case NBPF_Q_IP:   return "IP";
+    case NBPF_Q_ICMP: return "ICMP";
     case NBPF_Q_SCTP: return "SCTP";
     case NBPF_Q_TCP:  return "TCP";
     case NBPF_Q_UDP:  return "UDP";
@@ -297,6 +298,10 @@ void dump_rules(nbpf_rule_list_item_t *pun) {
     dump_rule(c);
 
     if(pun->bidirectional) printf("[BIDIRECTIONAL] ");
+
+    if(pun->fields.not_rule) printf("[DROP] ");
+    else printf("[PASS] ");
+
     printf("\n");
 
     pun = pun->next;
@@ -439,8 +444,8 @@ int main(int argc, char *argv[]) {
 
   printf("\nDumping Rules\n-------------\n");
 
+  printf("Default: %s\n", tree->default_pass ? "PASS" : "DROP");
   dump_rules(pun);
-
   if(dump_napatech)   napatech_dump_rules(pun);
   if(dump_fiberblaze) fiberblaze_dump_rules(pun);
 
@@ -450,6 +455,8 @@ int main(int argc, char *argv[]) {
 
   memset(&pkt, 0, sizeof(pkt));
 
+  pkt.device_id = 400;
+  pkt.interface_id = 45;
   pkt.vlan_id = 34;
   pkt.tuple.eth_type = 0x0800;
   pkt.tuple.ip_version = 4;
@@ -460,7 +467,9 @@ int main(int argc, char *argv[]) {
   pkt.tuple.l4_dst_port = htons(345);
   pkt.l7_proto = 7;
 
-  printf("VlanID=%u IPv%u Src=%s Dst=%s Proto=%u SrcPort=%u DstPort=%u L7Proto=%u -> %s\n",
+  printf("Device=%u Interface=%u VlanID=%u IPv%u Src=%s Dst=%s Proto=%u SrcPort=%u DstPort=%u L7Proto=%u -> %s\n",
+    pkt.device_id,
+    pkt.interface_id,
     pkt.vlan_id,
     pkt.tuple.ip_version,
     bpf_intoaV4(ntohl(pkt.tuple.ip_src.v4), tmp1, sizeof(tmp1)),
