@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2016-2020 ntop.org
+ *  Copyright (C) 2016-23 ntop
  *
  *      http://www.ntop.org/
  *
@@ -120,7 +120,7 @@ static nbpf_node_t* node_clone(nbpf_node_t *t) {
 /* ****************************************** */
 
 nbpf_tree_t* tree_clone(nbpf_tree_t *t) {
-  nbpf_tree_t *c = (nbpf_tree_t *) malloc(sizeof(nbpf_tree_t));
+  nbpf_tree_t *c = (nbpf_tree_t *) calloc(1, sizeof(nbpf_tree_t));
 
   if (!c) 
     return NULL;
@@ -133,7 +133,7 @@ nbpf_tree_t* tree_clone(nbpf_tree_t *t) {
 
 static void node_purge(nbpf_node_t *n) {
   if (n->custom_key) free(n->custom_key);
-  if (n->custom_key) free(n->custom_value);
+  if (n->custom_value) free(n->custom_value);
   if (n->l) node_purge(n->l);
   if (n->r) node_purge(n->r);
   free(n);
@@ -189,7 +189,7 @@ static nbpf_node_t *tree_parse(const char *buffer) {
 /* ****************************************** */
 
 nbpf_tree_t *nbpf_parse(const char *bpf_filter, l7protocol_by_name_func l7proto_by_name_callback) {
-  nbpf_tree_t *t = (nbpf_tree_t *) malloc(sizeof(nbpf_tree_t));
+  nbpf_tree_t *t = (nbpf_tree_t *) calloc(1, sizeof(nbpf_tree_t));
 
   if (t == NULL)
     return NULL;
@@ -530,6 +530,7 @@ nbpf_node_t *nbpf_create_protocol_node(int proto) {
     case NBPF_Q_TCP:
     case NBPF_Q_UDP:
     case NBPF_Q_SCTP:
+    case NBPF_Q_ICMP:
       n->qualifiers.protocol = NBPF_Q_IP;
       break;
     default:
@@ -542,6 +543,7 @@ nbpf_node_t *nbpf_create_protocol_node(int proto) {
     case NBPF_Q_TCP:  n->protocol = 6;      break;
     case NBPF_Q_UDP:  n->protocol = 17;     break;
     case NBPF_Q_SCTP: n->protocol = 132;    break;
+    case NBPF_Q_ICMP: n->protocol = 1;      break;
     //default:     n->protocol = proto;  break;
   }
 
